/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2011,2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

#pragma once


#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <memory.h>
#include <string.h>
#include <wchar.h>

#ifndef _WIN32
   #include <sys/types.h>
   #include <sys/errno.h>
#endif

extern "C"
{
   #include "ODPPCmnAPI.h"
   #include "ODPPCmnOprH.h"
   #include "ODPPDataTypesH.h"
   #include "ODPPCmnErrCodes.h"
}

/* Local Function Declarations */
void CreateRowsChar(DP_ROWSET_DEF *pRowSet);
void CreateRowsNum(DP_ROWSET_DEF *pRowSet);
void Clean(DP_ROWSET_DEF *pRowSet);

#define FALSE                 0
#define TRUE                  1
//Maximum path length
#define MAX_PATH              512
//Maximum size of error message buffer
#define ERR_MSG_BUF_SIZE      512
//Maximum size of destination buffer
#define DST_BUF_MAX_SIZE      772
#define ODPP_UCHAR            unsigned char
#define ODPP_USHORT           unsigned short
#define bool                  char

/* Optim DP Provider supports input information in 2 formats
   either declared as an array of structures or as a list
   of structures (SEE pNext in each structure. This enables the
   caller to supply the information they want arrays are generally
   faster to allocate and manage then a list. This example shows
   FldDef and Operands being used as arrays while Rows in a RowSet
   being used as a list. */

/* =========================IMPORTANT: ============================
   Irrespective of the way you send the data it is important
   to set the count fields to correct number of elements sent. Provoider
   internally work on the count information provided. Even in case of list
   You should not assume that provider will automatically stop after the last
   element in the list, instead it will malfunction
   =========================IMPORTANT: ============================*/

//Maximum number of operands
#define PARAMETER_MAX_SIZE  10 
//Maximum number of columns
#define MAX_COL    20 
//Maximum size of the column name
#define MAX_COLNAME_SIZE   256 

//Tracing specific defines
#define TRACELEVEL "1"

//service definition
DP_SVC_DEF        SvcDef; 
//field definition
DP_FIELD_DEF      FldDef[MAX_COL];  
//Parameters Definition
DP_INIT_OP_DEF    Parameters[PARAMETER_MAX_SIZE]; 
//To call CCN specific API's
int TestCCN(void); 
//To get error messages
int TestGetErrorMsg(void); 

int main(int argc, char* argv[])
{
   RETVAL         retVal       = ODPPFAILURE;//Return code   
   short          sCnt         = 0; //Counter
   short          sPrvCount    = 0; //Holds Provider count
   DP_PRV_DEF     *pPrvDef     = NULL; //Pointer to the DP_PRV_DEF 
   DP_PRV_DETAILS *pPrvDetails = NULL; //Pointer to the DP_PRV_DETAILS
   DP_PRV_DEF     PrvDef; 
   
   //Point the pPrvDef to DP_PRV_DEF structure
   pPrvDef = &PrvDef;

   //Initialize the ODPP Framework
   retVal = Provider_FrmwInit(NULL);
   if(ODPPSUCCESS != retVal)
   {
      printf("\nFramework init failed Err=%#x\n", retVal);
      return retVal;
   }

   printf("\n\nEnumerating Providers\n");

   //Get the provider count by passing NULL as first parameter
   retVal = Provider_Enumerate(NULL, &sPrvCount);
   if(ODPPSUCCESS != retVal)
   {
      printf("\nProvider Enumerate failed Err=%#x\n", retVal);
      return retVal;
   }

   printf("\nProvider Count = %d\n", sPrvCount);

   //Allocate the provider definition block to retrieve the provider details

   //Initialize the DP_PRV_DEF structure
   memset(&PrvDef, 0, sizeof(DP_PRV_DEF));
   INITIALIZE_ODPP_STRUCT(PrvDef, DP_PRV_DEF)

   //Allocate the memory for provider details
   pPrvDetails = (DP_PRV_DETAILS *) malloc(sizeof(DP_PRV_DETAILS) * sPrvCount);
   if(NULL == pPrvDetails)
   {
      printf("\nFailed to allocate memory for Provider details list\n");
      return ODPPFAILURE;
   }
   memset(pPrvDetails, 0, sizeof(DP_PRV_DETAILS) * sPrvCount);

   //Set the details list pointer
   PrvDef.pPrvDetails = pPrvDetails;

   //To get formatted error messages in mixe character (SBCS/MBCS) format define USE_ODPP_MBCS_CHAR_CALLS in visual studio
   //settings in  preprocessors definition before building the sample 
#ifndef USE_ODPP_MBCS_CHAR_CALLS
   //If wide character (Unicode) format is used
      
   //To indicate that service provider details will be in wide character (Unicode) format 
   //and DPPD_WC_SS structure will be used to retrieve Service provider details.
   PrvDef.cSubType = 'W'; 

   //Allocate the memory for all service provider to get provider name and description
   for(sCnt = 0; sCnt < sPrvCount; sCnt++)
   {
      //Allocate pWC for each pPrvDetails
      (pPrvDetails + sCnt)->PD.pWC = (DPPD_WC_SS *) malloc(sizeof(DPPD_WC_SS)); 
      if(NULL == (pPrvDetails + sCnt)->PD.pWC)
      {
         printf("\nMem Alloc failed for pWC of DP_PRV_DETAILS\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pWC, 0, sizeof(DPPD_WC_SS));

         
      //Once pWC structure is allocated, allocate the memory for pProviderName and pDescription to hold
      //Service Provider name and description

      //Store the size of pProviderName buffer, in bytes
      (pPrvDetails + sCnt)->iPrvNameBufBytes = (ODPP_PROVIDER_NAME_LEN + 1) * sizeof(ODPP_WCHAR); 

      //Allocate the provider name with maximum buffer size
      (pPrvDetails + sCnt)->PD.pWC->pProviderName = (ODPP_WCHAR *) malloc (sizeof(ODPP_WCHAR) * (int)((pPrvDetails + sCnt)->iPrvNameBufBytes/sizeof(ODPP_WCHAR)));
      if(NULL == (pPrvDetails + sCnt)->PD.pWC->pProviderName)
      {
         printf("\nMem Alloc failed for pWC->Provider Name\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pWC->pProviderName, 0, (pPrvDetails + sCnt)->iPrvNameBufBytes);

         
      //Store the size of pDescription buffer, in bytes.
      (pPrvDetails + sCnt)->iPrvDescBufBytes = (ODPP_PROVIDER_DESCRIPTION_LEN + 1) * sizeof(ODPP_WCHAR); 
        
      //Allocate the provider description with maximum buffer size
      (pPrvDetails + sCnt)->PD.pWC->pDescription = (ODPP_WCHAR *) malloc (sizeof(ODPP_WCHAR) * (int)((pPrvDetails + sCnt)->iPrvDescBufBytes/sizeof(ODPP_WCHAR)));
      if(NULL == (pPrvDetails + sCnt)->PD.pWC->pDescription)
      {
         printf("\nMem Alloc failed for pWC->Provider Description\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pWC->pDescription, 0, (pPrvDetails + sCnt)->iPrvDescBufBytes);

   }
#else
   //If mixed character (MBCS/SBCS) format is used

   //To indicate that service provider details will be in mixed character(MBCS/SBCS) format 
   //and DPPD_MC_SS structure will be used to retrieve service provider details.
   PrvDef.cSubType = 'M';  

   //code page value.
   PrvDef.iPDCodePage = -1; 

   //DBMS type
   PrvDef.cPDDBMSType = 'N'; 
       
   //Allocate the memory for provider name and provider description for all service provider
   for(sCnt=0; sCnt < sPrvCount; sCnt++)
   {
      //Allocate pMC for each pPrvDtls
      (pPrvDetails + sCnt)->PD.pMC = (DPPD_MC_SS *) malloc(sizeof(DPPD_MC_SS));
      if(NULL == (pPrvDetails + sCnt)->PD.pMC)
      {
         printf("\nMem Alloc failed for pMC of DP_PRV_DETAILS\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pMC, 0, sizeof(DPPD_MC_SS));

      //Store the size of pProviderName buffer, in bytes
      (pPrvDetails + sCnt)->iPrvNameBufBytes = (ODPP_PROVIDER_NAME_LEN + 1) * ODPP_MBCS_CHAR_BYTES;
         
      //Allocate the provider name with maximum buffer size
      (pPrvDetails + sCnt)->PD.pMC->pProviderName = (char *) malloc ((pPrvDetails + sCnt)->iPrvNameBufBytes);
      if(NULL == (pPrvDetails + sCnt)->PD.pMC->pProviderName)
      {
         printf("\nMem Alloc failed for pMC->Provider Name\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pMC->pProviderName, 0, (pPrvDetails + sCnt)->iPrvNameBufBytes);

      //Store the size of pDescription buffer, in bytes.
      (pPrvDetails + sCnt)->iPrvDescBufBytes = (ODPP_PROVIDER_DESCRIPTION_LEN + 1) * ODPP_MBCS_CHAR_BYTES;
        
      //Allocate the provider description with maximum buffer size
      (pPrvDetails + sCnt)->PD.pMC->pDescription = (char *) malloc ((pPrvDetails + sCnt)->iPrvDescBufBytes);
      if(NULL == (pPrvDetails + sCnt)->PD.pMC->pDescription)
      {
         printf("\nMem Alloc failed for pMC->Provider Description\n");
         goto CleanPRVDetails;
      }
      memset((pPrvDetails + sCnt)->PD.pMC->pDescription, 0, (pPrvDetails + sCnt)->iPrvDescBufBytes);
   }
#endif

   //Call Provider_Enumerate() to get details of all available providers
   retVal = Provider_Enumerate(&PrvDef, &sPrvCount);
   if(ODPPSUCCESS != retVal)
   {
      printf("Provider Enumerate failed Err=%#x", retVal);
      goto CleanPRVDetails;
   }
  
   //Print details of all service provider retrieved in call to Provider_Enumerate()
   for(int iCnt=0; iCnt < sPrvCount; iCnt++)
   {
      #ifndef USE_ODPP_MBCS_CHAR_CALLS
         wprintf(L"\nProvider %d: %ws,\t%ws\n",iCnt, (pPrvDef->pPrvDetails +iCnt)->PD.pWC->pProviderName,
                                               (pPrvDef->pPrvDetails +iCnt)->PD.pMC->pDescription);
      #else
         printf("\nProvider %d: %s,\t%s\n",iCnt, (pPrvDef->pPrvDetails +iCnt)->PD.pMC->pProviderName,
                                           (pPrvDef->pPrvDetails +iCnt)->PD.pMC->pDescription);
      #endif
   }
   

   printf("\nOptim Data Privacy Provider v11.3.0.0 \n\n");

   /* Enable the ODPP trace with appropriate tracing level. Following call Sets entry exit trace level setting for all ODPP
    *  component. To set different trace level setting see the description of ODPPSetTraceLevel() in the ODPP
    *  Developer guide. Some of the examples are shows below 
    *    Sample calls:
    *
    *    RC = ODPPSetTraceLevel(1);
    *    This sets standard entry/exit tracing (i.e. Level 1) for the entire system. 
    *    The lack of any keyword in the string indicates the fact that its systemwide.
    *
    *    RC = ODPPSetTraceLevel(0);
    *    This turns off all systemwide tracing  (other than exception traces, which cannot be turned off).  
    *
    *    RC = ODPPSetTraceLevel(AGEP=1);
    *    This turns on standard entry/exit tracing for the AGE provider.
    *
    *    RC = ODPPSetTraceLevel(AGEP=1/CCNP=1);
    *    This turns on standard entry/exit tracing for the AGE and CCN providers.
    *
    *    RC = ODPPSetTraceLevel(1/AGEP=3/CCNP.Init=1:32);
    *    This turns on standard systemside entry/exit tracing, 
    *    turns on trace level 3 for the AGE provider, 
    *    and turns on all trace levels (1-32) for the CCN providers Init function.
    */
    retVal = ODPPSetTraceLevel(TRACELEVEL);
    if(ODPPSUCCESS != retVal)
    {
       printf("Provider Init Failed Err=%#x", retVal);
       goto CleanPRVDetails;
    }

   //Call the CNN provider specific API's
   TestCCN();


CleanPRVDetails:
   //Clean privider details block
   for(sCnt=0; sCnt < sPrvCount; sCnt++)
   {
      if(NULL != (PrvDef.pPrvDetails + sCnt))
      {
         #ifndef USE_ODPP_MBCS_CHAR_CALLS   
            //Clear the DPPD_WC_SS structures 
            if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pWC)
            {
               if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pWC->pProviderName)
               {
                  free((PrvDef.pPrvDetails + sCnt)->PD.pWC->pProviderName);
                  (PrvDef.pPrvDetails + sCnt)->PD.pWC->pProviderName = NULL;
               }

               if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pWC->pDescription)
               {
                  free((PrvDef.pPrvDetails + sCnt)->PD.pWC->pDescription);
                  (PrvDef.pPrvDetails + sCnt)->PD.pWC->pDescription = NULL;
               }

               free((PrvDef.pPrvDetails + sCnt)->PD.pWC);
               (PrvDef.pPrvDetails + sCnt)->PD.pWC = NULL;
            }
         #else
            //Clear the DPPD_MC_SS structures
            if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pMC)
            {
               if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pMC->pProviderName)
               {
                  free((PrvDef.pPrvDetails + sCnt)->PD.pMC->pProviderName);
                  (PrvDef.pPrvDetails + sCnt)->PD.pMC->pProviderName = NULL;
               }

               if(NULL != (PrvDef.pPrvDetails + sCnt)->PD.pMC->pDescription)
               {
                  free((PrvDef.pPrvDetails + sCnt)->PD.pMC->pDescription);
                  (PrvDef.pPrvDetails + sCnt)->PD.pMC->pDescription = NULL;
               }

               free((PrvDef.pPrvDetails + sCnt)->PD.pMC);
               (PrvDef.pPrvDetails + sCnt)->PD.pMC = NULL;
            }
         #endif
      }
   }
   free(PrvDef.pPrvDetails);

   //Terminate the ODPP Framework
   retVal = Provider_FrmwTerm();
   if(ODPPSUCCESS != retVal)
   {
      printf("\nFramework Term failed Err=%#x\n", retVal);
      return retVal;
   }

   return retVal;
}

int TestCCN()
{
   int      iSvcToken; //Service Token
   int      iDataType      = 0;
   RETVAL   retVal         = ODPPFAILURE; //return code
   int      iTempCnt       = 0; //Temporary counter
   int      i              = 0;
   short    sMethod        = 0; //Service method
   char     PrvName[]      = "CCN"; //Provider name in MBCS
   DP_ROWSET_DEF     rowSet;
   
   //Initialize the DP_FIELD_DEF structure
   memset(FldDef, 0, MAX_COL * sizeof(DP_FIELD_DEF));
   for(iTempCnt=0; iTempCnt < MAX_COL; iTempCnt++)
   {
      INITIALIZE_ODPP_STRUCT(FldDef[iTempCnt], DP_FIELD_DEF)
   }

   //Initialize te DP_INIT_OP_DEF structure
   memset(Parameters, 0, PARAMETER_MAX_SIZE * sizeof(DP_INIT_OP_DEF));
   for(iTempCnt=0; iTempCnt < PARAMETER_MAX_SIZE; iTempCnt++)
   {
      INITIALIZE_ODPP_STRUCT(Parameters[iTempCnt], DP_INIT_OP_DEF)
   }

   //Initialize the DP_SVC_DEF structure
   memset(&SvcDef, 0, sizeof(DP_SVC_DEF));
   INITIALIZE_ODPP_STRUCT(SvcDef, DP_SVC_DEF)

   /* list of operands */
   SvcDef.pParams = &Parameters[0];
   SvcDef.pFldDef = &FldDef[0];

   SvcDef.bCopyToDest = TRUE;

   //Create the list of Field Defines
#ifndef USE_ODPP_MBCS_CHAR_CALLS
   //For wide character (Unicode) format

   //Column 1
   //To indicate that column name is in wide character format and USE_ODPP_MBCS_CHAR_CALLS is not defined
   FldDef[0].cSubType = 'W';

   //Allocate the pWC struct
   FldDef[0].CN.pWC = (DPFD_WC_SS *) malloc(sizeof(DPFD_WC_SS));
   if(NULL == FldDef[0].CN.pWC)
   {
      printf("\nFailed to allocate pWC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[0].CN.pWC, 0, sizeof(DPFD_WC_SS));
   
   //Code page
   FldDef[0].iDataCodePage = 0; 

   //DBMS type
   FldDef[0].cDataDBMSType = 0; 

   //Store the maximum size for column name buffer
   FldDef[0].iColNameBytes = (MAX_COLNAME_SIZE + 1) * sizeof(ODPP_WCHAR);

   //Allocate buffer for column name in wide character (Unicode) format
   FldDef[0].CN.pWC->pColName = (ODPP_WCHAR *) malloc(sizeof(ODPP_WCHAR) * (int)(FldDef[0].iColNameBytes / sizeof(ODPP_WCHAR))); 
   if(NULL == FldDef[0].CN.pWC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[0].CN.pWC->pColName, 0, FldDef[0].iColNameBytes);
   
   //Set the source column name in FldDef
   wcscpy(FldDef[0].CN.pWC->pColName, L"Col1");    
   
   /* Data type is the key to converting input data into service provider format
      so always use the correct data type */

   /* Change this variable to choose different data sets */
   iDataType = 2;
   switch(iDataType)
   {
      case 1:
         FldDef[0].sDatatype = ODPPDATATYPE_VARCHAR_SZ;
         break;
      case 2:
         FldDef[0].sDatatype = ODPPDATATYPE_U_LONG_LONG;
         break;
   }

   /* Right now we cannot check iLength as data can be in various
      format like UTF16 or in a specific code page. We are checking
      for length after we convert the input data to service provider
      required format, but this parameter is there in case we need it some time.
      */
   FldDef[0].iLength = 0;

   /* These columns (Col2 and Col3) have been added just to show how we can actually
      send mutliple column in every Row to the provider. In this example
      source column will be Col1 and will contain the credit card no.
      to mask. Sending mutiple column would be needed for Lookup and
      also if any provider supports switch like NID 
      */

   //Column 2
   //To indicate that column name is in wide character(Unicode) format and USE_ODPP_MBCS_CHAR_CALLS is not defined
   FldDef[1].cSubType = 'W';
   
   //Allocate the pWC struct
   FldDef[1].CN.pWC = (DPFD_WC_SS *) malloc(sizeof(DPFD_WC_SS));
   if(NULL == FldDef[1].CN.pWC)
   {
      printf("\nFailed to allocate pWC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[1].CN.pWC, 0, sizeof(DPFD_WC_SS));
   
   //code page
   FldDef[1].iDataCodePage = 0; 

   //DBMS type
   FldDef[1].cDataDBMSType = 0; 

   //Store the maximum size for column name buffer
   FldDef[1].iColNameBytes = (MAX_COLNAME_SIZE + 1) * sizeof(ODPP_WCHAR);
   
   //Allocate buffer for column name in wide character (Unicode) format
   FldDef[1].CN.pWC->pColName = (ODPP_WCHAR *) malloc(sizeof(ODPP_WCHAR) * (int) (MAX_COLNAME_SIZE + 1)); 
   if(NULL == FldDef[1].CN.pWC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[1].CN.pWC->pColName, 0, FldDef[1].iColNameBytes);

   //Set the column name in FldDef   
   wcscpy(FldDef[1].CN.pWC->pColName, L"Col2"); 
   
   //Set the data type for the column   
   FldDef[1].sDatatype = ODPPDATATYPE_WVARCHAR_SZ; 

   //Column 3
   //To indicate that column name is in wide character(Unicode) format and USE_ODPP_MBCS_CHAR_CALLS is not defined
   FldDef[2].cSubType = 'W'; 
   
   //Allocate the pWC struct
   FldDef[2].CN.pWC = (DPFD_WC_SS *) malloc(sizeof(DPFD_WC_SS));
   if(NULL == FldDef[2].CN.pWC)
   {
      printf("\nFailed to allocate pWC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[2].CN.pWC, 0, sizeof(DPFD_WC_SS));

   //Code Page
   FldDef[2].iDataCodePage = 0;

   //DBMS type
   FldDef[2].cDataDBMSType = 0;

   //Store the maximum size for column name buffer
   FldDef[2].iColNameBytes = (MAX_COLNAME_SIZE + 1) * sizeof(ODPP_WCHAR);

   //Allocate buffer for column name in wide character (Unicode) format
   FldDef[2].CN.pWC->pColName = (ODPP_WCHAR *) malloc(sizeof(ODPP_WCHAR) * (int)(MAX_COLNAME_SIZE + 1)); 
   if(NULL == FldDef[2].CN.pWC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[2].CN.pWC->pColName, 0, FldDef[2].iColNameBytes);

   //Set the column name in FldDef   
   wcscpy(FldDef[2].CN.pWC->pColName, L"Col3"); 

   //Set the data type for the column   
   FldDef[2].sDatatype = ODPPDATATYPE_WVARCHAR_SZ; 
   
#else
   //For mixed character (MBCS/SBCS) format

   //Column 1
   //To indicate that column name is in mized character(MBCS/SBCS) format and USE_ODPP_MBCS_CHAR_CALLS is defined
   FldDef[0].cSubType = 'M'; 
   
   //Allocate the pMC struct
   FldDef[0].CN.pMC = (DPFD_MC_SS *) malloc(sizeof(DPFD_MC_SS));
   if(NULL == FldDef[0].CN.pMC)
   {
      printf("\nFailed to allocate pMC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[0].CN.pMC, 0, sizeof(DPFD_MC_SS));
   
   //Code page
   FldDef[0].iDataCodePage = -1; 

   //DBMS type
   FldDef[0].cDataDBMSType = 'N'; 

   //Store the maximum size for column name buffer
   FldDef[0].iColNameBytes = (MAX_COLNAME_SIZE + 1) * ODPP_MBCS_CHAR_BYTES;

   //Allocate buffer for column name in mixed character (MBCS/SBCS) format
   FldDef[0].CN.pMC->pColName = (char *) malloc(FldDef[0].iColNameBytes); 
   if(NULL == FldDef[0].CN.pMC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[0].CN.pMC->pColName, 0, FldDef[0].iColNameBytes);

   //Set the source column name in FldDef   
   strcpy(FldDef[0].CN.pMC->pColName, "Col1"); 
   
   /* Data type is the key to converting input data into service provider format
      so always use the correct data type */

   /* Change this variable to choose different data sets */
   iDataType = 2;
   switch(iDataType)
   {
      case 1:
         FldDef[0].sDatatype = ODPPDATATYPE_VARCHAR_SZ;
         break;
      case 2:
         FldDef[0].sDatatype = ODPPDATATYPE_U_LONG_LONG;
         break;
   }

   /* Right now we cannot check iLength as data can be in various
      format like UTF16 or in a specific code page. We are checking
      for length after we convert the input data to service provider
      required format, but this parameter is there in case we need it some time.
      */
   FldDef[0].iLength = 0;

   /* These columns (Col2 and Col3) have been added just to show how we can actually
      send mutliple column in every Row to the provider. In this example
      source column will be Col1 and will contain the credit card no.
      to mask. Sending mutiple column would be needed for Lookup and
      also if any provider supports switch like NID 
      */

   
   //Column 2

   //To indicate that column name is in mixed character (MBCS/SBCS) format and USE_ODPP_MBCS_CHAR_CALLS is defined
   FldDef[1].cSubType = 'M';
   
   //Allocate the pMC struct
   FldDef[1].CN.pMC = (DPFD_MC_SS *) malloc(sizeof(DPFD_MC_SS));
   if(NULL == FldDef[1].CN.pMC)
   {
      printf("\nFailed to allocate pMC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[1].CN.pMC, 0, sizeof(DPFD_MC_SS));
   
   //code page
   FldDef[1].iDataCodePage = -1; 

   //DBMS type
   FldDef[1].cDataDBMSType = 'N'; 

   //Set the maximum size for column name buffer
   FldDef[1].iColNameBytes = (MAX_COLNAME_SIZE + 1) * ODPP_MBCS_CHAR_BYTES ;

   //Allocate buffer for column name in mixed character (SBCS/MBCS) format
   FldDef[1].CN.pMC->pColName = (char *) malloc(FldDef[0].iColNameBytes); 
   if(NULL == FldDef[1].CN.pMC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[1].CN.pMC->pColName, 0, FldDef[1].iColNameBytes);

   //Set the column name in FldDef   
   strcpy(FldDef[1].CN.pMC->pColName, "Col2"); 

   //Set the data type for the column   
   FldDef[1].sDatatype = ODPPDATATYPE_WVARCHAR_SZ; 

   //Column 3

   //To indicate that column name is in mixed character (SBCS/MBCS) format and USE_ODPP_MBCS_CHAR_CALLS is defined
   FldDef[2].cSubType = 'M';
   
   //Allocate the pMC struct
   FldDef[2].CN.pMC = (DPFD_MC_SS *) malloc(sizeof(DPFD_MC_SS));
   if(NULL == FldDef[2].CN.pMC)
   {
      printf("\nFailed to allocate pMC struct of DP_FIELD_DEF\n");
      goto CleanSvcDef;
   }
   memset(FldDef[2].CN.pMC, 0, sizeof(DPFD_MC_SS));

   //Code page
   FldDef[2].iDataCodePage = -1;

   //DBMS type
   FldDef[2].cDataDBMSType = 'N';

   //Store the maximum length for column name buffer
   FldDef[2].iColNameBytes = (MAX_COLNAME_SIZE + 1) * ODPP_MBCS_CHAR_BYTES;

   //Allocate buffer for column name in mixed character (MBCS/SBCS) format
   FldDef[2].CN.pMC->pColName = (char *) malloc(FldDef[0].iColNameBytes); 
   if(NULL == FldDef[2].CN.pMC->pColName)
   {
      printf("\nFailed to allocate memory for Column name\n");
      goto CleanSvcDef;
   }
   memset(FldDef[2].CN.pMC->pColName, 0, FldDef[2].iColNameBytes);

   //Set the column name in FldDef   
   strcpy(FldDef[2].CN.pMC->pColName, "Col3"); 

   //Set the data type for the destination column   
   FldDef[2].sDatatype = ODPPDATATYPE_WVARCHAR_SZ; 
   
#endif

   /*=======================================*/
   /* Setting the Field count to 3 as we are supplying 3 fields
      Count information should always be correct and should never
      be skipped */
   SvcDef.sFldCount = 3;

   /* Setting the operands. There are 2 sets of Operands
      Framework common operands and Service Provider Specific
      Operands. Supply each operand separately */
   
   /* Switch is not needed but it is a mandatory operand
      so set it to ODPP_OPR_SWITCH_NA (Switch Not Applicable)*/
   Parameters[0].usParameterID = ODPP_OPR_SWITCH_NA;

   //To indicate that this parameter does not hold any value.
   Parameters[0].iValueSubType = PARAM_VAL_NONE; 

   /* ODPP_OPR_SOURCE_COLNAME and ODPP_OPR_SOURCE_COLINDEX are mutually
      exclusive and any one of them can be provided */

   /*==============================================================================
    * Example:
    * If ODPP_OPR_SOURCE_COLNAME parameter is used then following code can be used:
    *
    * Parameters[1].usParameterID = ODPP_OPR_SOURCE_COLNAME;
    * Parameters[1].iValueSubType = PARAM_VAL_WC; 
    * Parameters[1].PV.pWC = (DPPRM_VAL_WC_SS *) malloc(sizeof(DPPRM_VAL_WC_SS));
    * if(NULL == Parameters[1].PV.pWC)
    * {
    *    printf("\nFailed to allocate memory for pWC->Parameter value\n");
    *    goto CleanSvcDef;
    * }
    * memset(Parameters[1].PV.pWC, 0, sizeof(DPPRM_VAL_WC_SS));
    * iLen = wcslen(FldDef[0].CN.pWC->pColName);
    * Parameters[1].PV.pWC->pParmVal  = (ODPP_WCHAR *) malloc((iLen + 1) * sizeof(ODPP_WCHAR));  //Allocate memory for the source column name in wide character (Unicode) format                      
    * if(NULL == Parameters[1].PV.pWC->pParmVal) //Check if memory was allocated successfully
    * {      
    *    printf("Failed to allocate memory for parameter value pParamVal");
    *    goto CleanSvcDef;
    * }                     
    * wcsncpy(Parameters[1].PV.pWC->pParmVal, FldDef[0].CN.pWC->pColName, iLen); //Copy source column name from FldDef[0].CN.pWC->pColName to pParmVal
    * Parameters[1].PV.pWC->pParmVal[iLen] = '\0';  //NULL terminate the pParmVal buffer
    * Parameters[1].iValueBufBytes = (iLen + 1) * sizeof(ODPP_WCHAR); // Set the size of the parameter value buffer in bytes
    *
    *===============================================================================================================================*/
   
   //Index of the source column, to use column name above commented code can be used
   Parameters[1].usParameterID = ODPP_OPR_SOURCE_COLINDEX;

   //To indicate that current parameter will hold numeric value.
   Parameters[1].iValueSubType = PARAM_VAL_NUM; 
   
   /* index values always start with 0 */
   Parameters[1].PV.uiVal = 0; 

   /* Method to choose during masking :
      This is a mandatory Operand and it changes the way masking is done.
      ODPP_METHOD_MASK:  Generates a consistently masked output
      ODPP_METHOD_RANDOM:  Generates a random masked output
   */
   Parameters[2].usParameterID = ODPP_OPR_METHOD;
   
   //To indicate that current parameter will hold numeric value.
   Parameters[2].iValueSubType = PARAM_VAL_NUM; 

   //ODPP_METHOD_MASK:  Generates a consistently masked output
   Parameters[2].PV.uiVal = ODPP_METHOD_MASK;

   /* sending a CCN specific operand */
   /* This operand change the way masking is done. For e.g.
   ODPP_FLAG_CCN_ISSUER6 copies first 6 bytes from source to destination
   ODPP_FLAG_CCN_RANDOM copies first 4 bytes from source to destination and
   generates randomly next 2 digits.
   in both the cases the remaining digits i.e. from 7 to end are masked consistently

   Not supplying this operand copies first 4 digits and remaining are consistently
   masked
   */
   Parameters[3].usParameterID = ODPP_OPR_CCN_FLAGS;

   //To indicate that current parameter will hold numeric value.
   Parameters[3].iValueSubType = PARAM_VAL_NUM; 

   //ODPP_FLAG_CCN_RANDOM copies first 4 bytes from source to destination and
   Parameters[3].PV.uiVal = ODPP_FLAG_CCN_RANDOM;

   /* Setting the Operand count to 4 as we are supplying 4 operands
      Count information should always be correct and should never
      be skipped */
   SvcDef.sParamCount = 4;

   /* Initialize the Provider */
   retVal = Provider_Init(&iSvcToken, &PrvName[0], sizeof(PrvName), &SvcDef, FALSE);
   if(ODPPSUCCESS != retVal)
   {
      printf("Provider Init Failed Err=%#x", retVal);
      goto CleanSvcDef;
   }

   // Create a Rowset from the data base
   memset(&rowSet, 0, sizeof(DP_ROWSET_DEF));

   // Initialize the DP_ROW_DEF structure
   INITIALIZE_ODPP_STRUCT(rowSet, DP_ROWSET_DEF)

   printf("\nCreating Data List : \n");

   switch(iDataType)
   {
      case 1:
         CreateRowsChar(&rowSet);
         break;
      case 2:
         CreateRowsNum(&rowSet);
         break;
      default:
         printf ("\nInvalid option \n");
         retVal = ODPPFAILURE;
         goto Terminate;
         break;
   }

   printf("Data List Done: Count=%d\n", rowSet.iCount);

   printf("\nCalling Service : \n");

   sMethod = ODPP_METHOD_DEFAULT;
   retVal = Provider_Service(iSvcToken, sMethod, NULL, &rowSet);
   if(ODPPSUCCESS != retVal)
   {
      printf("Provider service Failed %d", retVal);
      goto Terminate;
   }
   printf("\nService Done : \n");

Terminate:
   /* terminate the provider */
   retVal = Provider_Term(iSvcToken);
   if(ODPPSUCCESS != retVal)
   {
      printf("Provider Terminate Failed %d", retVal);
   }

   /* clean the rowset created */
   Clean(&rowSet);

CleanSvcDef:
   
   for(i=0; i < MAX_COL; i++)
   {
      //Clear the field definition
      #ifndef USE_ODPP_MBCS_CHAR_CALLS
         if(NULL != FldDef[i].CN.pWC)
         {
            if(NULL != FldDef[i].CN.pWC->pColName)
            {
               free(FldDef[i].CN.pWC->pColName);
               FldDef[i].CN.pWC->pColName = NULL;

               free(FldDef[i].CN.pWC);
               FldDef[i].CN.pWC = NULL;
            }
         }
      #else
         if(NULL != FldDef[i].CN.pMC)
         {
            if(NULL != FldDef[i].CN.pMC->pColName)
            {
               free(FldDef[i].CN.pMC->pColName);
               FldDef[i].CN.pMC->pColName = NULL;

               free(FldDef[i].CN.pMC);
               FldDef[i].CN.pMC = NULL;
            }
         }
      #endif
   }
   
   for(i=0;i<PARAMETER_MAX_SIZE; i++)
   {
      //Clear the parameter defination
      #ifndef USE_ODPP_MBCS_CHAR_CALLS
         if(PARAM_VAL_WC == Parameters[i].iValueSubType)
         {
            if(NULL != Parameters[i].PV.pWC)
            {
               if(NULL != Parameters[i].PV.pWC->pParamVal)
               {
                  free(Parameters[i].PV.pWC->pParamVal);
                  Parameters[i].PV.pWC->pParamVal = NULL;

                  free(Parameters[i].PV.pWC);
                  Parameters[i].PV.pWC = NULL;
               }
            }
         }
      #else
         if(PARAM_VAL_MC == Parameters[i].iValueSubType)
         {
            if(NULL != Parameters[i].PV.pMC)
            {
               if(NULL != Parameters[i].PV.pMC->pParamVal)
               {
                  free(Parameters[i].PV.pMC->pParamVal);
                  Parameters[i].PV.pMC->pParamVal = NULL;
            
                  free(Parameters[i].PV.pMC);
                  Parameters[i].PV.pMC = NULL;
               }
            }
         }
      #endif
   }
   return retVal;
}



void CreateRowsChar(DP_ROWSET_DEF *pRowSet)
{
   DP_ROW_DEF *pRowCur = NULL; 
   DP_ROW_DEF *pRow = NULL;

   /* Allocate 1st Row */
   pRow = (DP_ROW_DEF*)malloc(sizeof(DP_ROW_DEF));
   memset(pRow, 0, sizeof(DP_ROW_DEF));

   /* Allocate One Column */
   pRow->pFldDataDefine = (DP_FIELD_DATA_DEF*)malloc(sizeof(DP_FIELD_DATA_DEF));
   memset(pRow->pFldDataDefine, 0, sizeof(DP_FIELD_DATA_DEF));

   pRow->sCount = 1;

   /* Allocate Column Data Buffers */
   pRow->pFldDataDefine->pSrcBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pSrcBuf, 0, 20);
   pRow->pFldDataDefine->iSrcBufLen = 20;

   pRow->pFldDataDefine->pDstBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pDstBuf, 0, 20);
   pRow->pFldDataDefine->iDstBufLen = 20;
   /* Copy a credit card number */
   strcpy((char*)pRow->pFldDataDefine->pSrcBuf, "180027485132398");
   
   /* 1st Row Allocated */
   
   /* Assign the 1st row to RowSet */
   pRowSet->pRowDefine = pRow;

   pRowCur = pRow;

   /* Allocate 2nd Row */
   pRow  = NULL;
   pRow = (DP_ROW_DEF*)malloc(sizeof(DP_ROW_DEF));
   memset(pRow, 0, sizeof(DP_ROW_DEF));

   /* Allocate One Column */
   pRow->pFldDataDefine = (DP_FIELD_DATA_DEF*)malloc(sizeof(DP_FIELD_DATA_DEF));
   memset(pRow->pFldDataDefine, 0, sizeof(DP_FIELD_DATA_DEF));
   pRow->sCount = 1;

   /* Allocate Column Data Buffers */
   pRow->pFldDataDefine->pSrcBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pSrcBuf, 0, 20);
   pRow->pFldDataDefine->iSrcBufLen = 20;
   pRow->pFldDataDefine->pDstBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pDstBuf, 0, 20);
   pRow->pFldDataDefine->iDstBufLen = 20;

   /* Copy a credit card number */
   strcpy((char*)pRow->pFldDataDefine->pSrcBuf, "30000019965395");

   pRowCur->pNext = pRow;

   pRowCur = pRowCur->pNext;
   /* 2nd Row Allocated */

   pRowSet->iCount = 2;
   pRowSet->iLastPrcsRow = 0;
}

void CreateRowsNum(DP_ROWSET_DEF *pRowSet)
{
   DP_ROW_DEF *pRowCur = NULL; 
   DP_ROW_DEF *pRow = NULL;
   __int64 iCCNVal = 0;

   /* Allocate 1st Row */
   pRow = (DP_ROW_DEF*)malloc(sizeof(DP_ROW_DEF));
   memset(pRow, 0, sizeof(DP_ROW_DEF));

   /* Allocate One Column */
   pRow->pFldDataDefine = (DP_FIELD_DATA_DEF*)malloc(sizeof(DP_FIELD_DATA_DEF));
   memset(pRow->pFldDataDefine, 0, sizeof(DP_FIELD_DATA_DEF));
   pRow->sCount = 1;

   /* Allocate Column Data Buffers */
   pRow->pFldDataDefine->pSrcBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pSrcBuf, 0, 20);
   pRow->pFldDataDefine->iSrcBufLen = 20;
   pRow->pFldDataDefine->pDstBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pDstBuf, 0, 20);
   pRow->pFldDataDefine->iDstBufLen = 20;

   /* Copy a credit card number */
   iCCNVal = 180027485132398;
   memcpy(pRow->pFldDataDefine->pSrcBuf, &iCCNVal, sizeof(__int64));
   
   /* 1st Row Allocated */
   
   /* Assign the 1st row to RowSet */
   pRowSet->pRowDefine = pRow;

   pRowCur = pRow;

   /* Allocate 2nd Row */
   pRow  = NULL;
   pRow = (DP_ROW_DEF*)malloc(sizeof(DP_ROW_DEF));
   memset(pRow, 0, sizeof(DP_ROW_DEF));

   /* Allocate One Column */
   pRow->pFldDataDefine = (DP_FIELD_DATA_DEF*)malloc(sizeof(DP_FIELD_DATA_DEF));
   memset(pRow->pFldDataDefine, 0, sizeof(DP_FIELD_DATA_DEF));
   pRow->sCount = 1;

   /* Allocate Column Data Buffers */
   pRow->pFldDataDefine->pSrcBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pSrcBuf, 0, 20);
   pRow->pFldDataDefine->iSrcBufLen = 20;

   pRow->pFldDataDefine->pDstBuf  = (ODPP_UCHAR*)malloc(20);
   memset(pRow->pFldDataDefine->pDstBuf, 0, 20);
   pRow->pFldDataDefine->iDstBufLen = 20;

   /* Copy a credit card number */
   iCCNVal = 30000019965395;
   memcpy(pRow->pFldDataDefine->pSrcBuf, &iCCNVal, sizeof(__int64));

   pRowCur->pNext = pRow;

   pRowCur = pRowCur->pNext;
   /* 2nd Row Allocated */

   pRowSet->iCount = 2;
   pRowSet->iLastPrcsRow = 0;
}

void Clean(DP_ROWSET_DEF *pRowSet)
{
   DP_ROW_DEF *pRow = pRowSet->pRowDefine, *pPrevRow;;
   DP_FIELD_DATA_DEF *pData, *pPrev;
   
   for(int i = 0;((i < pRowSet->iCount) && (NULL != pRow)); i++)
   {
      pPrevRow = pRow;
      pData = pRow->pFldDataDefine;
      for(int j = 0;((j < pRow->sCount) && (NULL != pData)); j++)
      {
         pPrev = pData;
         if(pData->pSrcBuf != NULL)
            free(pData->pSrcBuf);
         if(pData->pDstBuf != NULL)
            free(pData->pDstBuf);                  
         pData = pData->pNext;
         free(pPrev);
      }      
      pRow = pRow->pNext;
      free(pPrevRow);      
   }
   pRowSet->iCount = 0;
}

