/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* Copyright IBM Corp. 1996,2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*****************************************************************
** ODPPAPI.H - This is the include file for the Optim
**             Column Map Exits that use ODPP functions.
**             The ODPP headers listed below are provided
**             with the ODPP standalone package.
**
** preconditions:
**      ODPPCmnAPI.h
**      ODPPCmnOprH.h
**      ODPPDataTypesH.h
**      ODPPCmnErrCodes.h
**
*********************************************************************/

#ifndef _ODPP_API_H_
#define _ODPP_API_H_

#ifdef __cplusplus  
extern "C" {
#endif

// ODPP public headers that are provided with the standalone package
#include "ODPPCmnAPI.h"
#include "ODPPCmnOprH.h"
#include "ODPPDataTypesH.h"
#include "ODPPCmnErrCodes.h"

#if defined(WIN32)
   #define  ODPPLIBHANDLE            HMODULE
   #define  ODPPFUNPTR               FARPROC
#else
   #define ODPPLIBHANDLE            void *
   #define ODPPFUNPTR               void *
#endif

// ODPP Provider
#define ODPP_FUNC_PROVIDER_FRMW_INIT         "Provider_FrmwInit"
#define ODPP_FUNC_PROVIDER_FRMW_TERM         "Provider_FrmwTerm"
#define ODPP_FUNC_PROVIDER_INIT              "Provider_Init"
#define ODPP_FUNC_PROVIDER_SERVICE           "Provider_Service"
#define ODPP_FUNC_PROVIDER_TERM              "Provider_Term"
#define ODPP_FUNC_PROVIDER_GET_ERR_CNT       "Provider_GetErrorCount"
#define ODPP_FUNC_PROVIDER_GET_ERR           "Provider_GetError"
#define ODPP_FUNC_PROVIDER_GET_FMT_ERR_MSG   "Provider_GetFormattedErrorMsg"
#define ODPP_FUNC_PROVIDER_TERM_ERRORS       "Provider_TermErrors"
#define ODPP_FUNC_PROVIDER_ENUMERATE         "Provider_Enumerate"
#define ODPP_FUNC_PROVIDER_GET_INFO          "Provider_GetInfo"

// ODPP Parser
#define ODPP_FUNC_PARSER_INIT             "Parser_Init"
#define ODPP_FUNC_PARSER_SERVICE          "Parser_Service"
#define ODPP_FUNC_PARSER_TERM             "Parser_Term"
#define ODPP_FUNC_PARSER_GET_ERR_CNT      "Parser_GetErrorCount"
#define ODPP_FUNC_PARSER_GET_ERR          "Parser_GetError"
#define ODPP_FUNC_PARSER_GET_FMT_ERR_MSG  "Parser_GetFormattedErrorMsg"
#define ODPP_FUNC_PARSER_TERM_ERRORS      "Parser_TermErrors"
// ODPP Adapter or Parameter Generator
#define ODPP_FUNC_PARMGEN_INIT		      "ParmGen_Init"
#define ODPP_FUNC_PARMGEN_SERVICE	      "ParmGen_Service"
#define ODPP_FUNC_PARMGEN_CLEAN           "ParmGen_Clean"
#define ODPP_FUNC_PARMGEN_TERM		      "ParmGen_Term"
#define ODPP_FUNC_PARMGEN_GET_ERR_CNT     "ParmGen_GetErrorCount"
#define ODPP_FUNC_PARMGEN_GET_ERR         "ParmGen_GetError"
#define ODPP_FUNC_PARMGEN_GET_FMT_ERR_MSG "ParmGen_GetFormattedErrorMsg"
//ODPP Tracing
#define ODPP_FUNC_QUERY_TRACE_FILE        "ODPPQueryTraceFileName"
#define ODPP_FUNC_SET_TRACE_LEVEL         "ODPPSetTraceLevel"

// Function pointer for ODPP Provider_FrmwInit() 
typedef RETVAL (*ODPP_PROVIDER_FRMW_INIT) ( DP_FRMW_PARAMS_DEF *pInitParams );

// Function pointer for ODPP Provider_Init() 
typedef RETVAL (*ODPP_PROVIDER_INIT)(int *pSvcToken,
                                    char *pSvcID,
                                    int iSvcIdLen,
                                    DP_SVC_DEF *pSvcdef,
                                    char bAllocInitBlock );

// Function pointer for ODPP Provider_Service 
typedef RETVAL (*ODPP_PROVIDER_SERVICE)(int iSvcToken, 
                                       short sMethod, 
                                       DP_SVC_DEF * pSvcDef, 
                                       DP_ROWSET_DEF * pRowSet); 

// Function pointer for ODPP Provider_GetErrorCount 
typedef RETVAL (*ODPP_PROVIDER_GET_ERROR_COUNT)(int  iSvcToken,
                                                short *pErrCount,
                                                int *pErrAreaLen); 

// Function pointer for ODPP Provider_GetError 
typedef RETVAL (*ODPP_PROVIDER_GET_ERROR)(int  iSvcToken,
                                          DP_ECB *pECB ); 

// Function pointer for ODPP Provider_GetFormattedErrorMsg 
typedef RETVAL (*ODPP_PROVIDER_GET_FORMATTED_ERROR_MSG)(DP_ERR_MSG *pErrMsg); 

// Function pointer for ODPP Provider_TermErrors() 
typedef RETVAL (*ODPP_PROVIDER_TERM_ERRORS)( int iSvcToken );

// Function pointer for ODPP Provider_Term() 
typedef RETVAL (*ODPP_PROVIDER_TERM)( int iSvcToken );

// Function pointer for ODPP Provider_FrmwTerm() 
typedef RETVAL (*ODPP_PROVIDER_FRMW_TERM)( void );

// Function pointer for ODPP Provider_Enumerate()    
typedef RETVAL (*ODPP_PROVIDER_ENUMERATE)( DP_PRV_DEF *pPrvDetails, 
                                           short *pCount );

// Function pointer for ODPP Provider_GetInfo()    
typedef RETVAL (*ODPP_PROVIDER_GET_INFO)( int iSvcToken, 
                                          short sRequest,  
                                          void *ptr,
                                          int *pBufLen );

// Parser:

// Function pointer for ODPP Parser_Init() 
typedef int (*ODPP_PARSER_INIT)(void );
// Function pointer for ODPP Parser_Service 
typedef int (*ODPP_PARSER_SERVICE)(DP_PARAMS_DEF *pParamDef); 
// Function pointer for ODPP Parser_GetErrorCount 
typedef int (*ODPP_PARSER_GET_ERROR_COUNT)(short * pErrCount, int * pErrAreaLen); 
// Function pointer for ODPP Parser_GetError 
typedef int (*ODPP_PARSER_GET_ERROR)(DP_ECB * pECB); 
// Function pointer for ODPP Parser_GetFormattedErrorMsg 
typedef int (*ODPP_PARSER_GET_FORMATTED_ERROR_MSG)(DP_ERR_MSG *pErrMsg); 
// Function pointer for ODPP Parser_Term() 
typedef int (*ODPP_PARSER_TERM)( void );

// Adapter or Parameter Generator:

// Function pointer for ODPP Adapter_Init
typedef int (*ODPP_PARMGEN_INIT)( void ); 
// Function pointer for ODPP Adapter_Service 
typedef int (*ODPP_PARMGEN_SERVICE)(DP_PARMGEN_DEF *pParmGen); 
// Function pointer for ODPP Adapter_Clean 
typedef int (*ODPP_PARMGEN_CLEAN)(DP_SVC_DEF *pSvcDef);
// Function pointer for ODPP Adapter_Term
typedef int (*ODPP_PARMGEN_TERM)(void); 
// Function pointer for ODPP Adapter_GetErrorCount 
typedef int (*ODPP_PARMGEN_GET_ERROR_COUNT)(short *pErrCount,
                                            int *pErrAreaLen);
// Function pointer for ODPP Adapter_GetError 
typedef int (*ODPP_PARMGEN_GET_ERROR)(DP_ECB *pECB);
// Function pointer for ODPP Adapter_GetFormattedErrorMsg 
typedef int (*ODPP_PARMGEN_GET_FORMATTED_ERROR_MSG)(DP_ERR_MSG *pErrMsg);

// Tracing:

// Function pointer for ODPPQueryTraceFileName 
typedef char * (*ODPP_QUERY_TRACE_FILE_NAME)();
// Function pointer for ODPPSetTraceLevel 
typedef RETVAL (*ODPP_SET_TRACE_LEVEL)(char* TrcLvlString);

// Structure containing function pointers for the ODPP API
typedef struct ODPP_PROVIDER_API_DEF
{   
   // Provider
   ODPP_PROVIDER_FRMW_INIT                pFrmwInit;           // Provider_FrmwInit func pointer
   ODPP_PROVIDER_ENUMERATE                pEnumerate;          // Provider_Enumerate func pointer
   ODPP_PROVIDER_INIT                     pInit;               // Provider_Init func pointer
   ODPP_PROVIDER_SERVICE                  pService;            // Provider_Service func pointer
   ODPP_PROVIDER_GET_ERROR_COUNT          pGetErrCount;        // Provider_GetErrorCount func pointer
   ODPP_PROVIDER_GET_ERROR                pGetErr;             // Provider_GetError func pointer
   ODPP_PROVIDER_GET_FORMATTED_ERROR_MSG  pGetFormattedErrMsg; // Provider_GetFormattedErrorMsg func pointer
   ODPP_PROVIDER_TERM_ERRORS              pTermErrors;         // Provider_TermErrors func pointer
   ODPP_PROVIDER_TERM                     pTerm;               // Provider_Term func pointer
   ODPP_PROVIDER_FRMW_TERM                pFrmwTerm;           // Provider_FrmwTerm func pointer
   ODPP_PROVIDER_GET_INFO                 pGetInfo;            // Provider_GetInfo func pointer
   // Parser 
   ODPP_PARSER_INIT                       pParserInit;            // Parser_Init func pointer
   ODPP_PARSER_SERVICE                    pParserService;         // Parser_Service func pointer
   ODPP_PARSER_GET_ERROR_COUNT            pParserGetErrCount;     // Parser_GetErrorCount func pointer
   ODPP_PARSER_GET_ERROR                  pParserGetErr;          // Parser_GetError func pointer
   ODPP_PARSER_GET_FORMATTED_ERROR_MSG    pParserGetFormattedErrMsg; // Parser_GetFormattedErrorMsg func pointer
   ODPP_PARSER_TERM                       pParserTerm;            // Parser_Term func pointer
   // Adapter or Parameter generator
   ODPP_PARMGEN_INIT                      pParmGenInit;			// Adapter Init API
   ODPP_PARMGEN_SERVICE                   pParmGenService;		// Adapter Service API
   ODPP_PARMGEN_CLEAN                     pParmGenClean;       // Adapter Clean API
   ODPP_PARMGEN_TERM                      pParmGenTerm;	      // Adapter Term API
   ODPP_PARMGEN_GET_ERROR_COUNT           pParmGenGetErrCount;
   ODPP_PARMGEN_GET_ERROR                 pParmGenGetErr;
   ODPP_PARMGEN_GET_FORMATTED_ERROR_MSG   pParmGenGetFormattedErrMsg;
   // Trace
   ODPP_QUERY_TRACE_FILE_NAME             pQueryTraceFileName; // ODPPQueryTraceFileName func pointer
   ODPP_SET_TRACE_LEVEL                   pSetTraceLevel;      // ODPPSetTraceLevel func pointer
   // Library handles
   #if defined(_AIX)
   ODPPLIBHANDLE                          hProviderLibHandle; // Handle for the ODPP Provider lib
   ODPPLIBHANDLE                          hParserLibHandle;   // Handle for the ODPP Parser lib
   ODPPLIBHANDLE                          hAdapterLibHandle;  // Handle for the ODPP Adapter lib
   #else
   short                                  sProviderDLLToken;  // DLLToken for the ODPP Provider lib
   short                                  sParserDLLToken;    // DLLToken for the ODPP Parser lib
   short                                  sParmGenDLLToken;   // DLLToken for the ODPP Adapter lib
   #endif
}ODPP_PROVIDER_API_DEF;

ODPP_PROVIDER_API_DEF *gpODPP_Api; 

// Macro to set the ODPP API pointer
#define ODPP_INIT_API_PTR(ptr)          \
        {gpODPP_Api = ptr;}

// Use the following defines for invoking ODPP functions. Be sure to initialize the pointer
// using ODPP_INIT_API_PTR first !!!

// ODPP Provider
#define ODPP_Framework_Init                     (*(gpODPP_Api->pFrmwInit))
#define ODPP_Provider_Enumerate                 (*(gpODPP_Api->pEnumerate))
#define ODPP_Provider_Init                      (*(gpODPP_Api->pInit))
#define ODPP_Provider_Service                   (*(gpODPP_Api->pService))
#define ODPP_Provider_Get_Error_Count           (*(gpODPP_Api->pGetErrCount))
#define ODPP_Provider_Get_Error                 (*(gpODPP_Api->pGetErr))
#define ODPP_Provider_Get_Formatted_Error_Msg   (*(gpODPP_Api->pGetFormattedErrMsg))
#define ODPP_Provider_Term_Errors               (*(gpODPP_Api->pTermErrors))
#define ODPP_Provider_Term                      (*(gpODPP_Api->pTerm))
#define ODPP_Framework_Term                     (*(gpODPP_Api->pFrmwTerm))
#define ODPP_Provider_Get_Info                  (*(gpODPP_Api->pGetInfo))
// Parser 
#define ODPP_Parser_Init                        (*(gpODPP_Api->pParserInit))
#define ODPP_Parser_Service                     (*(gpODPP_Api->pParserService))
#define ODPP_Parser_Get_Error_Count             (*(gpODPP_Api->pParserGetErrCount))
#define ODPP_Parser_Get_Error                   (*(gpODPP_Api->pParserGetErr))
#define ODPP_Parser_Get_Formatted_Error_Msg     (*(gpODPP_Api->pParserGetFormattedErrMsg))
#define ODPP_Parser_Term                        (*(gpODPP_Api->pParserTerm))
// Adapter or Parameter Generator
#define ODPP_ParmGen_Init                       (*(gpODPP_Api->pParmGenInit))
#define ODPP_ParmGen_Service                    (*(gpODPP_Api->pParmGenService))
#define ODPP_ParmGen_Clean                      (*(gpODPP_Api->pParmGenClean))
#define ODPP_ParmGen_Get_Error_Count            (*(gpODPP_Api->pParmGenGetErrCount))
#define ODPP_ParmGen_Get_Error                  (*(gpODPP_Api->pParmGenGetErr))
#define ODPP_ParmGen_Get_Formatted_Error_Msg    (*(gpODPP_Api->pParmGenGetFormattedErrMsg))
// Trace
#define ODPP_Query_Trace_File_Name              (*(gpODPP_Api->pQueryTraceFileName))
#define ODPP_Set_Trace_Level                    (*(gpODPP_Api->pSetTraceLevel))

#ifdef __cplusplus
}
#endif

#endif 