/*----------------------------------------------------------------------------
File Name:  ODPPCmnAPI.H

Purpose:    Declares ODPP related structures and API's

----------------------------------------------------------------------------*/

#ifndef __OdppCcnExit_H
#define __OdppCcnExit_H

#ifdef __cplusplus
extern "C" {
#endif  

// Include the required ODPP headers
#include "PSTEXIT.H"
#include "PSTCMWXT.H"
#include "ODPPAPI.h"

// Manifest constants used in this column map exit
#define PST_WORK_AREA_LEN           256
#define EXITERROR                   L"ODPPCCN"
#define SOURCE                      L"ODPPCCNEXIT"
#define MAX_MSG_BUFFER_SIZE         512
#define MAX_WORK_BUFFER_SIZE        256
// Maximum value for Credit Card Number
#define MAX_DATA_BUFFER_SIZE        19

// Define ODPPSUCCESS and ODPPFAILURE
#define FALSE                       0
#define TRUE                        1
#define ODPPSUCCESS                 0
#define ODPPFAILURE                 -1

// Maximum number of parameters
#define MAX_PARAMETER               3
#define MAX_COLUMN                  1
#define MAX_COLNAME_SIZE            256

//Tracing specific defines
#define TRACELEVEL "0"

// Platform specific manifest constants...
#ifdef _WIN32 
   #define SWPRINTF                _snwprintf
   #define PST_ERROR_FILE_NAME     "../PSTEXIT.TXT"//Err File Name
#else
   // The following will be used on UNIX/Linux
   #define SWPRINTF                swprintf
   #define PST_ERROR_FILE_NAME     "../PSTEXIT"//Err File Name
#endif


// OPPPCCN Work Area structure
typedef struct s_CCNWA_WORK_AREA
{
    int                                 iSvcToken;  // Service token.  
                                                    // Must be passed to ODPP in each provider service call
    PST_CM_WEXIT_COL                   *pSCol;      // Source column pointer
    short SDataType;                                // PST_C_ for source datatype 
    PST_CM_WEXIT_COL                   *pDCol;      // Destination column pointer
    short DDataType;                                // PST_C_ for Destination datatype 
    int                                 iRowNum;    // Row number.  Used with PrintErrors().
    DP_ROWSET_DEF                       *pRowSet;   // ODPP row set definition structure 
    DP_ROW_DEF                          *pRowDef;   // ODPP row definition structure
    DP_FIELD_DATA_DEF                   *pDataDef;  // ODPP data definition structure
    DP_SVC_DEF                          *pSvcDef;   // Service definition
    DP_FIELD_DEF                        *pFldDef;   // DP_FIELD_DEF structures
    DP_INIT_OP_DEF                      *pParamDef; // DP_INIT_OP_DEF structures
    wchar_t                             *pwcInpData;// Input buffer contains the source data
    wchar_t                             *pwcOutData;// Output buffer contains the masked data destination data
} CCNWA;

// Function prototypes:

// Check Column Attribute 
static short CheckColumnW(PST_CM_WEXIT_COL *,
                         short *,
                         wchar_t *);

// Output error message to PSTEXIT.TXT File 
void OutputErrorMessage(const wchar_t *,
                        wchar_t *);

// Print Error Message for failed rows from execution of Provider_Service() 
void ProcessODPPErrors(struct s_CCNWA_WORK_AREA *pCcnWa, 
                       unsigned char bIsInfoMsg );

// Clean the FIELD definition and PARAMETER definition memory
void clean(CCNWA *pCcnWa);


#ifdef __cplusplus
}       
#endif  

#endif  /* #ifndef __OdppCcnExit_H */

