/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

//---------------------------------------------------------------------------
// ODPPCCNExitS.c
// The following is the source for an Optim column map exit sample
// that uses the ODPP Credit Card Number (CCN) service provider
// for masking credit card number data.
//----------------------------------------------------------------------------
// Include the required standard C-type headers
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <wchar.h>
#ifdef _WIN32
   #include <windows.h>
#else
   #include <dlfcn.h>
#endif

// Include the required Optim headers for a column map exit
#include "OdppCcnExitH.h"

//----------------------------------------------------------------------------
// Function:      PSTColMapWExit()
// Purpose:       Main entry point for an Optim Column Map User Exit.
// Description:   This function is called by Optim for each row
//                in a mapped table when the DLL name is specified
//                as a user exit for a column map source.
//
// Parameters:    pInputParms       A pointer to a PST_STRUCT_CMW_EXIT_PARM
//                                  structure.
//                                  This structure contains information
//                                  about the source and destination tables,
//                                  the nature of the current call, and
//                                  pointers to the call-back functions
//                                  and work areas.
//                pSrcColList       A pointer to a PST_STRUCT_CMW_EXIT_COL_LIST
//                                  structure describing the source columns.
//                pDstColList       A pointer to a PST_STRUCT_CMW_EXIT_COL_LIST
//                                  structure describing the destination
//                                  columns.
// Returns:       PST_CMW_EXIT_SUCCESS        Column successfully transformed
//                PST_CMW_EXIT_REJECT_ROW     Destination column cannot be
//                                            transformed.. Discard the row.
//                PST_CMW_EXIT_ABORT_PROCESS  Fatal error, terminate processing.
//----------------------------------------------------------------------------
#ifdef _WIN32
__declspec(dllexport)
#endif

short PSTColMapWExit(PST_STRUCT_CM_WEXIT_PARM     *pInputParms,
                     PST_STRUCT_CM_WEXIT_COL_LIST *pSrcColList,
                     PST_STRUCT_CM_WEXIT_COL_LIST *pDestColList)
{
   int      iMsg;
   int      Loop       = 0;
   short    sIndVar    = 0;
   short    sLen       = 0;
   short    sMethod    = 0;
   short    rc         = 0;
   unsigned char bInfoMsg = FALSE; //To identify that ODPP returned info message
   //Return Code Of ODPP Function 
   RETVAL   retVal     = 0;
   long     Length     = 0;
   wchar_t  Work[MAX_WORK_BUFFER_SIZE];
   wchar_t  Message[MAX_MSG_BUFFER_SIZE]; 
   wchar_t  SkipReason[MAX_WORK_BUFFER_SIZE];
   wchar_t  *pSData;
   wchar_t  *pDData;
   DP_FIELD_DEF     **pTmpFldDef; // Temporary Field definition pointer
   DP_INIT_OP_DEF   **pTmpParamDef,*pTmpParamDef1; // Temporary Parameter definition pointers
   PST_CM_WEXIT_COL *pDCol;
   PST_CM_WEXIT_COL *pSCol;   
   wchar_t        *pWorkArea = (wchar_t *)pInputParms->pLclWorkArea;               
   // iLclArea the size of pLclWorkArea in wchar_t.
   int   iLclArea = L_LCL_WORK_AREA / sizeof(wchar_t); 
   // Assign the sample work area to local work Area of Col Map Exit
   CCNWA          *pCcnWa = (CCNWA *)pWorkArea;
   // Service identifier
   static char    *PrvName = "CCN";

   //Initialize the ODPP API pointer
   ODPP_INIT_API_PTR(pInputParms->pODPPApi);
   // Is this an Optim column map exit termination-type call?
   if ((pInputParms->FuncCode) == PST_CMW_FUNC_TERMINATE ) 
   {    
      // Yes, ...
      // Call the ODPP provider termination
      retVal = ODPP_Provider_Term(pCcnWa->iSvcToken);
      if (ODPPSUCCESS != retVal)
      {

         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"ODPP provider termination failed. Return code:%d",retVal);
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }

      // Clear the memory allocated for ODPP Structures
      clean(pCcnWa);
      
      // Return to the caller indicating success   
      return PST_CMW_EXIT_SUCCESS;        
   }

   // Is this the 1st Optim column map exit entry?
   if ((pDCol = pCcnWa->pDCol) == NULL)
   {
      // Yes, ...
      //---------------------------------------------------------------------
      // Setup and initialize ODPP for masking CCN data
      //---------------------------------------------------------------------
      // The following code parses the parm fields, resolves the source column name
      // and sets up the definition of the source and destination columns
     
      // Clear the work area
      memset(Work,0,sizeof(Work));
      
      // Setup the source and destination pointers
      pDCol = &(pDestColList->Col[pInputParms->DestColIndex]);
      pCcnWa->pDCol = pDCol;
      // NOTE: By default, the source is same as destination
      pSCol = &(pSrcColList->Col[pInputParms->DestColIndex]);
      pCcnWa->pSCol = pSCol;

      // Allocate memory for DP_SVC_DEF structue
      pCcnWa->pSvcDef = (DP_SVC_DEF *) malloc(sizeof(struct DP_SVC_DEF));
      if(NULL == pCcnWa->pSvcDef)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"Failed to allocate memory for pSvcDef");                              
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;

      }
      // Clear the memory allocated for DP_SVC_DEF Structures
      memset(pCcnWa->pSvcDef, 0, sizeof(DP_SVC_DEF));
      // Initialize the ODPP DP_SVC_DEF structures
      INITIALIZE_ODPP_STRUCT_PTR(pCcnWa->pSvcDef, DP_SVC_DEF)

      // Allocate memory for all fields
      pTmpFldDef = &pCcnWa->pFldDef;
      pCcnWa->pFldDef = (DP_FIELD_DEF *) malloc(sizeof(struct DP_FIELD_DEF));
      if(NULL == pCcnWa->pFldDef)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"Failed to allocate memory for pFldDef");                              
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      // Clean the memory for allocated field
      memset(pCcnWa->pFldDef, 0, sizeof(DP_FIELD_DEF));
      // Initialize ODPP DP_FIELD_DEF structure. 
      INITIALIZE_ODPP_STRUCT_PTR(pCcnWa->pFldDef, DP_FIELD_DEF)
     
      // Allocate memory for Initialization Parameters
      pTmpParamDef = &pCcnWa->pParamDef;
      for(Loop = 0; Loop < MAX_PARAMETER; Loop++)
      {
         (*pTmpParamDef) = (DP_INIT_OP_DEF *) malloc(sizeof(struct DP_INIT_OP_DEF));
         if(NULL == (*pTmpParamDef))
         {
            SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"Failed to allocate memory for pParamDef");                              
            rc = PST_CMW_EXIT_ABORT_PROCESS;
            goto Exit;
         }
         // Clean memory for allocated Parameter
         memset((*pTmpParamDef), 0, sizeof(DP_INIT_OP_DEF));
         pTmpParamDef = &(*pTmpParamDef)->pNext;
      }    
      // Initialize the ODPP DP_INIT_OP_DEF for total number of parameters
      pTmpParamDef1 = pCcnWa->pParamDef;
      for(Loop=0; Loop < MAX_PARAMETER; Loop++)
      {
         INITIALIZE_ODPP_STRUCT_PTR(pTmpParamDef1, DP_INIT_OP_DEF)
         pTmpParamDef1 = pTmpParamDef1->pNext;
      }

      // Initialize the row number
      pCcnWa->iRowNum = 0;
   
      // Validate the destination column construction
      // and set the format flags for later processing
      rc = CheckColumnW(pDCol, &(pCcnWa->DDataType), Work);                 
      if (rc != PSTEXIT_SUCCESS) 
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"For the destination column:%ls - the %ls is not valid.",
                  pDCol->pColName, Work);                              
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }

      // Validate the source column construction 
      // and set the format flags for later processing
      rc = CheckColumnW(pSCol, &(pCcnWa->SDataType), Work);               
      if (rc != PSTEXIT_SUCCESS) 
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"For the source column:%ls - the %ls is not valid.", 
                     pSCol->pColName, Work);
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      
      /* Enable the ODPP trace with appropriate tracing level. Following call Sets entry exit trace level setting for all ODPP
      *  component. To set different trace level setting see the description of ODPPSetTraceLevel() in the ODPP
      *  Developer guide. Some of the examples are shows below 
      *    Sample calls:
      *
      *    RC = ODPPSetTraceLevel(1);
      *    This sets standard entry/exit tracing (i.e. Level 1) for the entire system. 
      *    The lack of any keyword in the string indicates the fact that its systemwide.
      *
      *    RC = ODPPSetTraceLevel(0);
      *    This turns off all systemwide tracing  (other than exception traces, which cannot be turned off).  
      *
      *    RC = ODPPSetTraceLevel(AGEP=1);
      *    This turns on standard entry/exit tracing for the AGE provider.
      *
      *    RC = ODPPSetTraceLevel(AGEP=1/CCNP=1);
      *    This turns on standard entry/exit tracing for the AGE and CCN providers.
      *
      *    RC = ODPPSetTraceLevel(1/AGEP=3/CCNP.Init=1:32);
      *    This turns on standard systemside entry/exit tracing, 
      *    turns on trace level 3 for the AGE provider, 
      *    and turns on all trace levels (1-32) for the CCN providers Init function.
      */
      retVal = ODPP_Set_Trace_Level(TRACELEVEL);
      if(ODPPSUCCESS != retVal)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"ODPP failed to set trace level. Return code:%d ",retVal);
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
            
      // ODPP service definition structure initialization
      pCcnWa->pSvcDef->pParams       =   pCcnWa->pParamDef;
      pCcnWa->pSvcDef->pFldDef       =   pCcnWa->pFldDef;
      pCcnWa->pSvcDef->bCopyToDest   =   TRUE;
      
      // ODPP field definition structure initialization
      // To indicate that column name is wide character and USE_ODPP_MBCS_CHAR_CALLS is not defined
      pCcnWa->pFldDef->cSubType = 'W';

      // Allocate the pWC struct
      pCcnWa->pFldDef->CN.pWC = (DPFD_WC_SS *) malloc(sizeof(DPFD_WC_SS));
      if(NULL == pCcnWa->pFldDef->CN.pWC)
      {
         printf("\nFailed to allocate pWC struct of DP_FIELD_DEF\n");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      // Clear the memory allocated for DPFD_WC_SS
      memset(pCcnWa->pFldDef->CN.pWC, 0, sizeof(DPFD_WC_SS));
   
      // Code page is zero for wide character (Unicode) format
      pCcnWa->pFldDef->iDataCodePage = 0; 

      // DBMS type is zero for wide character (Unicode) format
      pCcnWa->pFldDef->cDataDBMSType = 0; 

      // Store the maximum size for column name buffer
      pCcnWa->pFldDef->iColNameBytes = (wcslen(pSCol->pColName) + 1) * sizeof(ODPP_WCHAR);

      // Allocate buffer for column name in wide character (Unicode) format
      pCcnWa->pFldDef->CN.pWC->pColName = (ODPP_WCHAR *) malloc((int)pCcnWa->pFldDef->iColNameBytes); 
      if(NULL == pCcnWa->pFldDef->CN.pWC->pColName)
      {
         printf("\nFailed to allocate memory for Column name\n");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      // Clear the memory for column name
      memset(pCcnWa->pFldDef->CN.pWC->pColName, 0, pCcnWa->pFldDef->iColNameBytes);

      // Set the source column name in sFldDef
      wcscpy(pCcnWa->pFldDef->CN.pWC->pColName, pCcnWa->pSCol->pColName);    

      // Set the source column data type
      pCcnWa->pFldDef->sDatatype           =   ODPPDATATYPE_WVARCHAR_SZ;

      //Set the number of ODPP CCN columns
      pCcnWa->pSvcDef->sFldCount              =   1;

      //Set the number Of ODPP CCN Parameters
      pCcnWa->pSvcDef->sParamCount            =   3;

      // ODPP parameter structure initialization with ODPP CCN required parameters
      
      // Parameter 1
      /* Switch is not needed but it is a mandatory operand
         so set it to ODPP_OPR_SWITCH_NA (Switch Not Applicable)*/
      pTmpParamDef1 = pCcnWa->pParamDef;
      pTmpParamDef1->usParameterID = ODPP_OPR_SWITCH_NA;

      // To indicate that this parameter does not hold any value.
      pTmpParamDef1->iValueSubType = PARAM_VAL_NONE; 

      /* ODPP_OPR_SOURCE_COLNAME and ODPP_OPR_SOURCE_COLINDEX are mutually
         exclusive and any one of them can be provided */
      /*=====================================================================================================
       * Example :
       * If ODPP_OPR_SOURCE_COLNAME parameter is used then following code can be used:
       *
       * pTmpParamDef1->usParameterID = ODPP_OPR_SOURCE_COLNAME;
       * pTmpParamDef1->iValueBufBytes = PARAM_VAL_WC; 
       * pTmpParamDef1->PV.pWC = (DPPRM_VAL_WC_SS *) malloc(sizeof(DPPRM_VAL_WC_SS));
       * if(NULL == pTmpParamDef1->PV.pWC)
       * {
       *    printf("\nFailed to allocate memory for pWC->Parameter value\n");
       *    rc = PST_CMW_EXIT_ABORT_PROCESS;
       *    goto Exit;
       * }
       * memset(pTmpParamDef1->PV.pWC, 0, sizeof(DPPRM_VAL_WC_SS));
       * iLen = wcslen(pCcnWa->pFldDef->CN.pWC->pColName);
       * pTmpParamDef1->PV.pWC->pParmVal  = (ODPP_WCHAR *) malloc((iLen + 1) * sizeof(ODPP_WCHAR));  //Allocate memory for the source column name in wide character (Unicode) format                      
       * if(NULL == pTmpParamDef1->PV.pWC->pParmVal) //Check if memory was allocated successfully
       * {      
       *    printf("Failed to allocate memory for parameter value pParamVal");
       *    rc = PST_CMW_EXIT_ABORT_PROCESS;
       *    goto Exit;
       * }                     
       * wcsncpy(pTmpParamDef1->PV.pWC->pParmVal, pCcnWa->pFldDef->CN.pWC->pColName, iLen); //Copy source column name from sFldDef[0].CN.pWC->pColName to pParmVal
       * pTmpParamDef1->PV.pWC->pParmVal[iLen] = '\0';  //NULL terminate the pParmVal buffer
       * pTmpParamDef1->iValueBufBytes = (iLen + 1) * sizeof(ODPP_WCHAR); // Set the size of the parameter value buffer in bytes
       *
       *==================================================================================================================*/
      // Parameter 2
      pTmpParamDef1 = pTmpParamDef1->pNext;
      // Index of the source column, to use column name above commented code can be used
      pTmpParamDef1->usParameterID = ODPP_OPR_SOURCE_COLINDEX;

      // To indicate that current parameter will hold numeric value.
      pTmpParamDef1->iValueSubType = PARAM_VAL_NUM; 
   
      // index values always start with 0
      pTmpParamDef1->PV.uiVal = 0; 

      // Are there any input parameters?
      if(pInputParms->NumParms == 0)// If there is no input in the Col Map  i.e Default Masking
      {
         // No, ...
         // Then setup ODPP for default masking
         // Parameter 3
         pTmpParamDef1 = pTmpParamDef1->pNext;
         pTmpParamDef1->usParameterID = ODPP_OPR_METHOD;
   
         // To indicate that current parameter will hold numeric value.
         pTmpParamDef1->iValueSubType = PARAM_VAL_NUM; 

         // ODPP_METHOD_MASK:  Generates a consistently masked output
         pTmpParamDef1->PV.uiVal = ODPP_METHOD_MASK;
      }

      // Does the user want Random Masking? 
      // NOTE: Signified by the Key Value Pair METHOD=RANDOM as parameter in the column map
      else if(pInputParms->NumParms == 1 && wcsicmp(pInputParms->pParm[0],L"METHOD=RANDOM") == 0)
      {
         // Yes, ...
         // Then setup ODPP for random masking
         //Parameter 3
         pTmpParamDef1 = pTmpParamDef1->pNext;
         pTmpParamDef1->usParameterID = ODPP_OPR_METHOD;
   
         // To indicate that current parameter will hold numeric value.
         pTmpParamDef1->iValueSubType = PARAM_VAL_NUM; 

         // ODPP_METHOD_RANDOM:  Generates a random output
         pTmpParamDef1->PV.uiVal = ODPP_METHOD_RANDOM;
      }

      // Anything else is an error
      else
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"A valid Method-type parameter was not specified.");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }

      // Calling the ODPP CCN provider initialization
      retVal = ODPP_Provider_Init(&(pCcnWa->iSvcToken),&PrvName[0], strlen(PrvName),pCcnWa->pSvcDef,TRUE);
      if(ODPPSUCCESS != retVal)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"ODPP provider initialization failed. Return code:%d ",retVal);
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }

      // Allocate memory for ODPP rowset DP_ROWSET_DEF structure
      pCcnWa->pRowSet = (DP_ROWSET_DEF *) malloc(sizeof(DP_ROWSET_DEF));
      if(NULL == pCcnWa->pRowSet)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to allocate memory for pRowSet");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;

      }
      // Clear the ODPP rowset structure
      memset(pCcnWa->pRowSet,0,sizeof(DP_ROWSET_DEF));
      // Initialize the DP_ROWSET_DEF structure
      INITIALIZE_ODPP_STRUCT_PTR(pCcnWa->pRowSet, DP_ROWSET_DEF)

      // Setup the number of rows in the ODPP rowset structure
      pCcnWa->pRowSet->iCount      =  1;
      // Allocate memory for row definition structure
      pCcnWa->pRowDef = (DP_ROW_DEF *) malloc(sizeof(DP_ROW_DEF));
      if(NULL == pCcnWa->pRowDef)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to allocate memory for pRowDef");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;

      }
      // Clear the ODPP row definition structures
      memset(pCcnWa->pRowDef ,0,sizeof(DP_ROW_DEF));     
      // Initialize the DP_ROW_DEF structure
      INITIALIZE_ODPP_STRUCT_PTR(pCcnWa->pRowDef, DP_ROW_DEF)

      // Allocate memory for ODPP field data definition structure
      pCcnWa->pDataDef = (DP_FIELD_DATA_DEF *) malloc(sizeof(DP_FIELD_DATA_DEF));
      if(NULL == pCcnWa->pDataDef)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to allocate memory for pRowDef");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;

      }      
      // Clear the ODPP field data definition structure
      memset(pCcnWa->pDataDef,0,sizeof(DP_FIELD_DATA_DEF));
      // Initialize the DP_FIELD_DATA_DEF structure
      INITIALIZE_ODPP_STRUCT_PTR(pCcnWa->pDataDef, DP_FIELD_DATA_DEF)
      
      // Setup row definition structure
      pCcnWa->pRowSet->pRowDefine   =   pCcnWa->pRowDef;
      // Setup the ODPP data definition structure   
      pCcnWa->pRowDef->pFldDataDefine =    pCcnWa->pDataDef;

      // Allocate the memory for input source buffer
      pCcnWa->pwcInpData = (wchar_t *) malloc((MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));
      if(NULL == pCcnWa->pwcInpData)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to allocate memory for pwcInpData");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      // Clean the memory allocated for input source buffer
      memset(pCcnWa->pwcInpData,0,(MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));

      // Allocate the memory for output buffer
      pCcnWa->pwcOutData = (wchar_t *) malloc((MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));
      if(NULL == pCcnWa->pwcOutData)
      {
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to allocate memory for pwcOutData");
         rc = PST_CMW_EXIT_ABORT_PROCESS;
         goto Exit;
      }
      // Clean the memory allocated for output buffer
      memset(pCcnWa->pwcOutData,0,(MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));
   

   }  // endif ((pDCol = pCcnWa->pDCol) == NULL)

   // Bump the row count
   pCcnWa->iRowNum++;

   //------------------------------------------------------------------------
   // Setup buffers and pointers to retrieve the source data from Optim
   //------------------------------------------------------------------------
   pSCol     =   pCcnWa->pSCol;
   pDCol     =   pCcnWa->pDCol;
   // Setup the source column length
   Length    =   (pSCol->Length + 1) * sizeof(wchar_t);
   memset(pCcnWa->pwcInpData,0,(MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));
   memset(pCcnWa->pwcOutData,0,(MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t));
   memset(SkipReason,0,sizeof(SkipReason));

   // Call the Optim function to fetch the source data into the InpData[] Buffer
   rc = (pInputParms->pPSTGetColValue)(pSCol->hCol,         // Source call handle
                                       pCcnWa->SDataType,   // Source data type
                                       pSCol->Precision,    // Source precision
                                       pSCol->Scale,        // Source scale
                                       pCcnWa->pwcInpData,  // Buffer to hold the source data
                                       Length,              // Length of the source data
                                       &sIndVar);           // Flag indicates whether source column has NULL value

   // Was there an error fetching the source data from Optim
   if ( rc != PSTEXIT_SUCCESS)
   {
      // Yes, ...
      // Was it a conversion error
      if( rc == PSTEXIT_CONVERSION_ERROR)
      {
         // Yes, ...
         // Then skip this row
         goto SkipRow;
      }
      else
      {
         // No, ...
         // Then format a message and set a return value to indicate an error
         SWPRINTF(Message,MAX_MSG_BUFFER_SIZE,EXITERROR L"Failed to fetch column:%ls data from Optim. Return code:%#x",pSCol->pColName, rc);
         rc = PST_CMW_EXIT_ABORT_PROCESS;
      }
      goto Exit;
   }

   // Assign the source input buffer to a pointer
   pSData = pCcnWa->pwcInpData;

   // Is the source column nullable and does it contain a null indicator?
   if (pSCol->Nullable && sIndVar == -1) 
   {
      // Yes, ...
      // Is the destination not nullable?
      if (! pDCol->Nullable) 
      {                    
         // Yes, ...
         // Then skip this row
         goto SkipRow;
      }
   }
   
   // Populate the ODPP rowset structure
   pCcnWa->pRowSet->iLastPrcsRow =   0;
   pCcnWa->pRowDef->bProcessed  =   FALSE;
   pCcnWa->pRowDef->bHasError   =   FALSE;

   // Is there no source data?
   if (NULL == pSData)
   {
      // Yes, ...
      // Set an indication for ODPP that the source is empty
      pCcnWa->pDataDef->bSrcNull =   TRUE;
   }
   else
   {
      // No, ...
      // Set an indication for ODPP that there is source data
      pCcnWa->pDataDef->bSrcNull =   FALSE;
   }

   sLen = (short)wcslen(pCcnWa->pwcInpData);// Length of Input Data

   // Setup the ODPP data definition structure...
   // Set the length of the source data
   pCcnWa->pDataDef->iSrcBufLen =   (sLen)*sizeof(wchar_t);
   pCcnWa->pDataDef->pSrcBuf    =   (unsigned char*)pSData;
   pCcnWa->pDataDef->iDstBufLen =   (MAX_DATA_BUFFER_SIZE + 1) * sizeof(wchar_t);
   pCcnWa->pDataDef->pDstBuf    =   (unsigned char*)pCcnWa->pwcOutData;

   // Number of Column
   pCcnWa->pRowDef->sCount         =    1; 

   // Initialize a variable with the ODPP default method
   sMethod         =   ODPP_METHOD_DEFAULT;
   // Clear the error code
   pCcnWa->pRowDef->pFldDataDefine->iErrorCode = 0;
   
   // Call the ODPP provider service to mask the credit card number-type data
   retVal = ODPP_Provider_Service(pCcnWa->iSvcToken, sMethod, NULL, pCcnWa->pRowSet);
   switch(retVal)
   {
   case ODPPSUCCESS:
      break;   
   case ODPPSUCCESS_WITH_INFO:   
      // Are there ODPP row-type errors?
      if( pCcnWa->pRowDef->bHasError == TRUE)
      {
         // Yes, ...
         // Process the ODPP errors and skip this row
         bInfoMsg = FALSE;
         ProcessODPPErrors(pCcnWa, bInfoMsg);
         goto SkipRow;
      }
      else
      {
         // Informational message such as source NULL.
         // Check if there are any informational message from ODPP and process them to free 
         // the captured ODPP error control block (ECB).
         // This will ensure that memory consumption remains stable when there are several NULL values.
         bInfoMsg = TRUE;
         ProcessODPPErrors(pCcnWa, bInfoMsg);
      }
      break;
   default:
      // Format an error message, output it to the ODPP error file, and terminate
      SWPRINTF(SkipReason,MAX_MSG_BUFFER_SIZE, L"ODPP provider service failed. Return code:%d",retVal);
      OutputErrorMessage(SOURCE,SkipReason);
      bInfoMsg = FALSE;
      ProcessODPPErrors(pCcnWa, bInfoMsg);
      rc = PST_CMW_EXIT_ABORT_PROCESS;
      goto Exit;      
   }

   // Setup a wchar_t pointer to the masked destination data
   pDData =(wchar_t *)pCcnWa->pDataDef->pDstBuf;

   // Move the masked data from the ODPP structure 
   // into the Optim structure and return it to the Optim column processing
   rc = pInputParms->pPSTPutColValue(PST_C_WCHAR_STR,
                                     pDCol->Precision,
                                     pDCol->Scale,
                                     pDData,
                                     pDCol->Length,
                                     sIndVar);
   // Was there an error?
   if (rc != PSTEXIT_SUCCESS)
   {
      // Yes, ...
      // Format an error message and set the return code
      SWPRINTF(Message,MAX_MSG_BUFFER_SIZE, EXITERROR L"Outputting the ODPP masked destination data into the Optim column map processing for column %ls failed. Return code:%hd", pDCol->pColName, rc);
      rc = PST_CMW_EXIT_ABORT_PROCESS;
     goto Exit;
   }
   goto Exit;

SkipRow:
//----------
   // Tell Optim column map processing to reject the row
   rc = PST_CMW_EXIT_REJECT_ROW;

Exit:       
//---
   // Take final action based upon the current return code
   switch (rc) 
   {
   case PST_CMW_EXIT_SUCCESS:
      break;
   case PST_CMW_EXIT_REJECT_ROW:
      break;
   case PST_CMW_EXIT_ABORT_PROCESS:

      //Clear the memory allocated for field definition and parameter definition
      clean(pCcnWa);
   
      // Get the length of the error message
      iMsg = wcslen(Message);
      // Is the length of the error message greater than or equal to the length of the area that will hold the message?
      if(iMsg>=iLclArea)                                              
      {                                                               
         // Yes, ...
         // Copy the error message into the buffer to be returned to Optim column map processing
         // but only copy upto the maxiumum length allowed in this buffer
         wcsncpy(pWorkArea, Message, iLclArea-1);                      
         // NULL terminate the error message buffer
         pWorkArea[iLclArea-1] = L'\0';                                
      }                                                               
      else                                                            
      {                                            
         // No, ...
         // Copy the entire error message into the buffer to be returned to the Optim column map processing                   
         wcscpy(pWorkArea, Message);                                   
      }                                                               

      break;
   } // endswitch


   return rc;
} // end PstColMapWExit()s

//----------------------------------------------------------------------------
// Function:      OutputErrorMessage()
// Purpose:       Set a message in the work area.
// Description:   This function sets the error message in pMessage
//                    to a message file.
// Parameters:    pFunctionName     A pointer to the function name
//                                    generating the error.
//                pMsg              A pointer to the message to set.
// Returns:       void
//----------------------------------------------------------------------------
void OutputErrorMessage( const wchar_t *pFunctionName,wchar_t *pMsg )
{
   FILE           *hOutput;
   time_t         DateTime;
   struct tm      *pTime;
   wchar_t         Buffer[15];

   // Get the date and time 
   time( &DateTime );
   pTime = localtime( &DateTime );
   wcsftime( Buffer,15, L"%m/%d/%y %H:%M", pTime );

   // Open the error file to write
   if ( ( hOutput = fopen(PST_ERROR_FILE_NAME,"ab") ) != NULL )
   {
      // Write the message to the end of the file
      fwprintf( hOutput, L"\n (%ls): %ls %ls\n", pFunctionName, Buffer, pMsg );
      // Close hOutput 
      fclose( hOutput );
   }
 
}  
// End OutputErrorMessage() 


//----------------------------------------------------------------------------
// Function:      CheckColumnW()
// Purpose:       Checks the source/target column specified in the PST_CM_WEXIT_COL 
//                to ensure it is configured correctly.
// Description:   This will return an error message in the pMessage buffer 
//                and a return value of PSTEXIT_GENERAL_ERROR 
//                when the input column has not been configured correctly
// Parameters:    pCol              A pointer to the source/target column defined
//                                  within the PST_CM_WEXIT_COL structure
//                pDataType         A pointer to the returned data type
//                                  in the Optim column map exit-specificr format
//                pMessage          A pointer to the caller-provided buffer
//                                  that will contain an error message
//                                  when an error has been detected in this function
// Returns:       short             Return values:
//                                  PSTEXIT_SUCCESS when everything is OK
//                                  PSTEXIT_GENERAL_ERROR when there is an error
//----------------------------------------------------------------------------
static short CheckColumnW(PST_CM_WEXIT_COL *pCol,
                          short            *pDataType,
                          wchar_t          *pMessage)
{
   wchar_t          *pError = NULL;                 
   
   // Switch based on the source/target column data type...                              
   switch (pCol->DataType)
   {
   case PST_SQL_DECIMAL:
      if (pCol->Precision < 13 || pCol->Precision > 254) 
      {
         pError = L"Precision";                                        
      }
      else if (pCol->Scale != 0) 
      {
         pError = L"Scale";                                            
      }
      *pDataType = PST_C_WCHAR_STR;
      break;

   case PST_SQL_CHAR:
   case PST_SQL_NCHAR:
   case PST_SQL_MBCHAR:
      if (pCol->Length < 13 || pCol->Length > 256)
         pError = L"Length";                                           
      *pDataType = PST_C_WCHAR_STR;
      break;

   case PST_SQL_VARCHAR:
   case PST_SQL_NVARCHAR:
   case PST_SQL_MBVARCHAR:
      if (pCol->Length < 13 || pCol->Length > 254)
         pError = L"Length";                                           
      *pDataType = PST_C_WCHAR_STR;
      break;

    default:
      pError = L"Type";                                               
      break;
       
   }  // endswitch

   // Is there an error?
   if (pError != NULL)
   {
      // Yes, ...
      // Copy the error back into the caller-provided buffer
      wcscpy(pMessage, pError);                             
      // Return an indication of an error
      return PSTEXIT_GENERAL_ERROR;
   }

   // Return an indication of success
   return PSTEXIT_SUCCESS;
} // End Of CheckColumnW()


//----------------------------------------------------------------------------
// Function:      ProcessODPPErrors()
// Purpose:       Retrieve and process the ODPP provider errors
// Description:   This function retrieves the ODPP provider errors
//                and calls a subsequent function to write the error to
//                an output error file
// Parameters:    iSvcToken     Service Token Returned by Provider_Service()
//                iRowNum       Current Record Number 
// Returns:       void
//----------------------------------------------------------------------------
void ProcessODPPErrors(CCNWA *pCcnWa, unsigned char bInfoMsg)
{
   RETVAL     retVal; //return code   
   int        iErrAreaLen; //Length of error control block storage area   
   short      sErrCnt   = 0; //Error count
   short      i         = 0; //Counter
   DP_ERR_MSG ErrMsg; //Error message definition
   DP_ECB     *pEcb     = NULL; //Pointer to DP_ECB structure
   ODPP_WCHAR ErrMsgSrc[ODPP_MAX_ERR_MSG_SRC_LENGTH + 1];//To hold source of formatted error message
   ODPP_WCHAR ErrMsgText[ODPP_MAX_ERR_MSG_TEXT_LENGTH + 1];//To hold formatted error message text
   ODPP_WCHAR ErrMsgBody[ODPP_MAX_ERR_MSG_BODY_LENGTH + 1];//To hold formatted error message boy
   wchar_t    wcErrBuffer[3500];

   //Clear the DP_ERR_MSG structure memory
   memset(&ErrMsg, 0, sizeof(DP_ERR_MSG));

   //Initialize the DP_ERR_MSG structure
   INITIALIZE_ODPP_STRUCT(ErrMsg, DP_ERR_MSG)

   //Retrieve the number of errors that have occurred and the size of the ECB storage area
   retVal = ODPP_Provider_Get_Error_Count(pCcnWa->iSvcToken, &sErrCnt, &iErrAreaLen); 
   if(ODPPSUCCESS != retVal) 
   {
      printf("Failed to get the Error Count, Err=%#x", retVal);      
      return;
   }
   
   if(sErrCnt <= 0)
   {
      printf("\nNo Error Detected\n");
      return;
   }
   
   printf("\nTotal Error Count = %d\n", sErrCnt);
    
   //Now set the required fields of DP_ERR_MSG 

   //Language to use to return the error message. Currently not used and zero must be supplied.
   ErrMsg.iLanguage  = 0; 
   // Allocate the ODPP error control block
   pEcb = (DP_ECB*)malloc(iErrAreaLen);
   if(NULL == pEcb)
   {
      printf("Error Block Malloc failed\n");
      return;
   }
   memset(pEcb, 0, iErrAreaLen);

   //Initialize the DP_ECB stucture
   INITIALIZE_ODPP_STRUCT_PTR(pEcb, DP_ECB)

   /* Set the subtype of DP_ERR_MSG to 'W' to retrieve formatted error 
      message in wide character(Unicode) format */
   ErrMsg.cSubType = 'W';
    
   //Allocate error message block
   ErrMsg.EM.pWC = (DPEM_WC_SS *) malloc(sizeof(DPEM_WC_SS));
   if(NULL == ErrMsg.EM.pWC)
   {
      printf("\nMem Alloc failed for DPEM_WC_SS \n");
      goto OnClean;
   }
   memset(ErrMsg.EM.pWC, 0, sizeof(DPEM_WC_SS));

   //pMsgSrc will hold the source of the error message in wide character (Unicode) format
   ErrMsg.EM.pWC->pMsgSrc = &ErrMsgSrc[0];
   //Set the size of the buffer ErrMsgSrc, in bytes  
   ErrMsg.EM.pWC->iMsgSrcBytes = sizeof(ErrMsgSrc);
   
   //pMsg will hold the error message text in wide character (Unicode) format
   ErrMsg.EM.pWC->pMsg = &ErrMsgText[0];
   //Set the size of the buffer ErrMsgText, in bytes  
   ErrMsg.EM.pWC->iMsgBytes = sizeof(ErrMsgText);
   
   //pMsgBody will hold the erroe message description in details in wide character (Unicode) format
   ErrMsg.EM.pWC->pMsgBody = &ErrMsgBody[0];
   //Set the size of the buffer ErrMsgBody, in bytes  
   ErrMsg.EM.pWC->iMsgBodyBytes = sizeof(ErrMsgBody);

   //Loop through number of error count and print the formatted error messages
   for(i=0; i<sErrCnt; i++)
   {
      // call this function to get the oldest ECB
      retVal = ODPP_Provider_Get_Error(pCcnWa->iSvcToken, pEcb);
      if(ODPPSUCCESS != retVal)
      {
         printf("Provider_GetError failed, Err = %#x\n",retVal);
         continue;
      }

      // Clear the error message source, text and body buffers   
      memset(ErrMsgSrc,'\0',sizeof(ErrMsgSrc));
      memset(ErrMsgText,'\0',sizeof(ErrMsgText));
      memset(ErrMsgBody,'\0',sizeof(ErrMsgBody));
   
      //Assign the error control block populated in call to Provider_GetError()
      ErrMsg.pECB = pEcb;
       
      //Retrieve the number of errors that have occurred
      retVal = ODPP_Provider_Get_Formatted_Error_Msg(&ErrMsg);
      if(retVal != ODPPSUCCESS)
      {
         printf("Error Code not found, Err = %#x\n",retVal);
         continue;
      }
      
      // If it is informational message from ODPP then no need to write it into trace file. 
      // We will skip it to speed up the program.
      if(bInfoMsg == FALSE)
      {
         // Format an error mesage
         SWPRINTF(wcErrBuffer,3500,L"\n Skipped row details ::\tRow Number:%d::\tError Code:%d\n Error Message Source:%ls\n Error Message Text:%ls\n Error Message Body:%ls\n",pCcnWa->iRowNum,pEcb->iErrCode,ErrMsgSrc,ErrMsgText,ErrMsgBody);

         // Output the error message information to the error file
         OutputErrorMessage(SOURCE,wcErrBuffer);
      }
   }

//Free the memory allocated
OnClean:
   // Free the memory for the ODPP error control block
   if(NULL != pEcb)
      free(pEcb);
   // Free the memory for the ODPP error message block
   if(NULL != ErrMsg.EM.pWC)
      free(ErrMsg.EM.pWC);
   
   return;

} // end of ProcessODPPErrors()


//----------------------------------------------------------------------------
// Function:      clean()
// Purpose:       Clear the memory allocation for field definition and parameter
//                definition.
// Description:   This function deallocates the memory which was allocated during
//                provider Initialization
// Parameters:    pCcnWa  Pointer to CCN work area structure
// Returns:       void
//----------------------------------------------------------------------------
void clean(CCNWA *pCcnWa)
{
   DP_FIELD_DEF *pFldDef = NULL,*pTmpFldDef = NULL;
   DP_INIT_OP_DEF *pParamDef = NULL, *pTmpParamDef = NULL;
   DP_ROWSET_DEF *pRowSet = NULL;
   DP_ROW_DEF *pRowDef = NULL;
   DP_FIELD_DATA_DEF *pFldDataDef = NULL;
   int i = 0;


   // Free the memory which was allocated for column name
   pFldDef = pCcnWa->pFldDef;
   for(i=0; i < MAX_COLUMN; i++)
   {
      pTmpFldDef = pFldDef;
      pFldDef = pFldDef->pNext;
      if(NULL != pTmpFldDef)
      {
         //Clear the field definition
         if(NULL != pTmpFldDef->CN.pWC)
         {
            if(NULL != pTmpFldDef->CN.pWC->pColName)
            {
               free(pTmpFldDef->CN.pWC->pColName);
               pTmpFldDef->CN.pWC->pColName = NULL;
      
               free(pTmpFldDef->CN.pWC);
               pTmpFldDef->CN.pWC = NULL;
            }
         }
         free(pTmpFldDef);
      }     
   }
   pCcnWa->pFldDef = NULL;

   // Free the memory which was allocated for parameters
   pParamDef = pCcnWa->pParamDef;
   for(i=0; i<MAX_PARAMETER; i++)
   {
      pTmpParamDef = pParamDef;
      pParamDef = pParamDef->pNext;

      //Clear the parameter defination
      if(PARAM_VAL_WC == pTmpParamDef->iValueSubType)
      {
         if(NULL != pTmpParamDef->PV.pWC)
         {
            if(NULL != pTmpParamDef->PV.pWC->pParamVal)
            {
               free(pTmpParamDef->PV.pWC->pParamVal);
               pTmpParamDef->PV.pWC->pParamVal = NULL;

               free(pTmpParamDef->PV.pWC);
               pTmpParamDef->PV.pWC = NULL;
            }
         }
      }
      free(pTmpParamDef);
   }
   pCcnWa->pParamDef = NULL;

   // Free the memory which was allocated for data definition
   pFldDataDef = pCcnWa->pDataDef;
   if(NULL != pFldDataDef)
   {
      free(pFldDataDef);
      pCcnWa->pDataDef = NULL;
   }

   // Free the memory which was allocated for row definition
   pRowDef = pCcnWa->pRowDef;
   if(NULL != pRowDef)
   {
      free(pRowDef);
      pCcnWa->pRowDef = NULL;
   }

   // Free the memory which was allocated for row set
   pRowSet = pCcnWa->pRowSet;
   if(NULL != pRowSet)
   {
      free(pRowSet);
      pCcnWa->pRowSet = NULL;
   }

   // Free the memory allocated for source buffer
   if(NULL != pCcnWa->pwcInpData)
   {
      free(pCcnWa->pwcInpData);
      pCcnWa->pwcInpData = NULL;
   }

   // Free the memory allocated for output buffer
   if(NULL != pCcnWa->pwcOutData)
   {
      free(pCcnWa->pwcOutData);
      pCcnWa->pwcOutData = NULL;
   }

}


