/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 1998,2009,2011
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*****************************************************************
** PSTCMXIT.H - This is the include file for Optim
**              Column Map Exits.  Column maps support two types
**              of exits, Column transformations specified with
**              EXIT(exitname), and source and destination exits
**              defined within the AGE() function.
**
** preconditions:
**      PSTEXIT.H
**
**
 *
 * RTC / Jazz Log
 *
 * 16 Dec 2010 takahak 7.3.1 3622
 * d15985: MBCS column name isn't found at CCN mask
 *
 * 27 Aug 2009 SagarB 7.2.0 3012 Additional Information to EXITs
 * Added Variables "RowNumCur" and "MaxRows" in structure PST_STRUCT_CM_EXIT_PARM 
 *
 * Modified:
 *
 *    $Author:   SHILPAJ  $
 *    $Date:   11 Mar 2009 17:19:46  $
 *    $Revision:   1.2  $
 *    $Log:   Y:\RTWIN\711\ARCHIVES\pstcmxit.h_v  $
 * 
 *    Rev 1.2   11 Mar 2009 17:19:46   SHILPAJ
 * Copyright Information Updated
 * 
 *    Rev 1.1   11 Apr 2008 14:16:32   BOBO
 * - Corrected PVCS comment block
 * 
 *    Rev 1.0   07 Apr 2008 08:16:24   BOBO
 * Initial revision.
 *
*********************************************************************/

#ifndef __PSTCMXIT_H
#define __PSTCMXIT_H

#ifdef __cplusplus
extern "C" {                /* Assume C declarations for C++   */
#endif  /* __cplusplus */


/* The following two structs are used by both EXIT() and AGE() type
   exits.                                                             */


/* This structure defines the attributes of a single column available
   to the exit.  There are two arrays of these structures passed
   to the Exit.  One for the Source columns and one for the Destination
   columns.                                                           */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct  s_PSTCMExitCol
{
    char   *pColName;  /* Column Name, zero delimited                 */
    short   hCol;      /* Column Handle                               */
    short   DataType;  /* Data Type constant with PST_SQL prefix      */
    long    Length;    /* For PST_SQL_CHAR, PST_SQL_MBCHAR,
                            PST_SQL_NCHAR and PST_SQL_BINARY the
                            length of the column in bytes, not
                            including any terminating NULL characters.
                          For PST_SQL_VARCHAR, PST_SQL_MBVARCHAR,
                            PST_SQL_NVARCHAR and PST_SQL_VARBINARY
                            the maximum length of the col data in
                            bytes, not including the length field.
                            Also not including any terminating
                            NULL characters.
                          For other DataTypes it is undefined         */
    short   Precision; /* Numeric precision, valid only for
                          PST_SQL_DECIMAL,  0 for all others.         */
    short   Scale;     /* Numeric scale, valid only for
                          PST_SQL_DECIMAL,  0 for all others.         */

    unsigned char Nullable;  /* Can the Col have the NULL value?
                                  0 - Col may NOT have the NULL value
                                  1 - Col may have NULL value         */
    unsigned char Available; /* The PSTGetColValue function may
                                be called to get the current value
                                for this column.  All source columns
                                are available.  All destination cols
                                are available except those that have
                                exits that haven't been called yet.
                                  0 - Col value not may be retreived
                                  1 - Col value not be retreived      */
    unsigned short CodePage; /* Code Page of column, if set           */
} PST_CM_EXIT_COL;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* Two of these structures are passed to the Column Map Exit.
   Each is an array of Column definitions for the source or
   destination Tables.                                                */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CMExitColList
{
  short   NumCols;        /* Number of columns in the following array */
#if defined(__UNIX) || defined(__unix)
  short   Resv;           /* Filler for UNIX alignment                */
#endif

  PST_CM_EXIT_COL  Col[1];      /* array of Column entries            */

} PST_STRUCT_CM_EXIT_COL_LIST;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif





/***********************************************************************
 * Structs and defines for EXIT() type exits                           *
 ***********************************************************************/

/* FuncCodes values to be passed to the Exit */
#define PST_CM_FUNC_TRANSFORM      1  /* Exit should Put a value for
                                         the identified Destination
                                         Column.                      */
#define PST_CM_FUNC_COMPILE        2  /* Used internally for Visual
                                         Basic procedures             */
#define PST_CM_FUNC_TERMINATE     99  /* End of the Process, Exit
                                         resources should be released.*/

/* Max number of passed parameters */
#define PST_CM_MAX_PARMS           8

// 12/16/2010 takahak d15985, byte size of pLclWorkArea.
#ifndef L_LCL_WORK_AREA                                               // d15985
#define L_LCL_WORK_AREA    512                                        // d15985
#endif // L_LCL_WORK_ARE                                              // d15985

/* This structure is passed to the Column Map Exits specified with
   EXIT(). It contains various control information about the Source and
   Destination Tables and the Destination Column.                     */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CMExitParm
{
  /* The reason the Exit is called.                                   */
  long    FuncCode;    /* PST_CM_FUNC_TRANSFORM or
                          PST_CM_FUNC_TERMINATE                       */

  /* The following function can be used to retrieve column values from
     the source or destination row.                                   */
  short   (*pPSTGetColValue)(short,   /* Col Handle for the Source or
                                         Destination Col              */
                             short,   /* The C data type for the result
                                         data.  This must be a
                                         PST_C_... constant.          */
                             short,   /* Precision, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370
                                             PST_C_INTEGER_CHAR_SZ    */
                             short,   /* Scale, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370        */
                             void *,  /* Pointer to the result storage
                                         area.                        */
                             long,    /* Buffer length
                                         For
                                             PST_C_CHAR_SZ
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBCHAR_SZ
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_INTEGER_CHAR_SZ
                                         it must include space for the
                                         null terminating byte.
                                         For
                                             PST_C_NCHAR_SZ
                                             PST_C_NVARCHAR_SZ
                                         it must include space for the
                                         terminating zero short integer.
                                         For
                                             PST_C_VARCHAR
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBVARCHAR
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_NVARCHAR
                                             PST_C_NVARCHAR_SZ
                                             PST_C_VARBINARY
                                         it must include space for the
                                         two byte length field.
                                         For all others, it the simple
                                         buffer length.               */
                             short *); /* Pointer to a short Indicator
                                          variable. After the call, this
                                          field will contain -1 if the
                                          column is NULL.  Otherwise,
                                          this field will be zero.
                                          This parameter may be NULL  */
  /* The following function is used to provide the destination column
     value.  If the destination column is a LOB column, then
     (*pPSTPutLOB) must be used instead.                              */
  short   (*pPSTPutColValue)(short,   /* The C data type for the input
                                         data.  This must be a
                                         PST_C_... constant.          */
                             short,   /* Precision, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_SYB_DEC
                                             PST_C_DECIMAL_370
                                             PST_C_INTEGER_CHAR_SZ    */
                             short,   /* Scale, only used for cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_SYB_DEC
                                             PST_C_DECIMAL_370        */
                             void *,  /* Pointer to the input storage
                                         area.                        */
                             long,    /* Input Value Length in bytes.
                                         Only used for
                                             PST_C_CHAR
                                             PST_C_CHAR_SZ
                                             PST_C_MBCHAR
                                             PST_C_MBCHAR_SZ
                                             PST_C_NCHAR
                                             PST_C_NCHAR_SZ
                                             PST_C_BINARY
                                         For PST_C_CHAR_SZ and
                                         PST_C_MBCHAR_SZ this length
                                         should include one byte more
                                         for the Terminating NULL.
                                         For PST_C_NCHAR_SZ this length
                                         should include two bytes more
                                         for the Terminating zero short.
                                         The length for other types is
                                         derived from the Var Length
                                         field, the precision, or the
                                         inherent length of the
                                         datatype                     */
                             short);  /* Indicator Variable.  -1 if the
                                         input value is NULL, 0 if
                                         the input value is NOT NULL  */

  /* The following field points to a global 256 byte work area that can
     be used by the exit.  This area is cleared to binary zeros before
     the first call to any exit.  This area is shared by all exits in
     the Insert, Convert, or Load process.                            */
  char   *pGlblWorkArea;

  /* The following field points to a 512 byte work area that can be used
     by the exit.  This area is cleared to binary zeros before the first
     call to the exit.  There is a work area for each column, even if
     multiple columns use the same Exit                               
     In the event that the routine returns PST_CM_EXIT_ABORT_PROCESS, the
     product will look in this area for a NULL_terminated string to pass
     to the user                                                      */
  char   *pLclWorkArea;

  char   *pSrcDBAliasName;/* Source Tbl's DBAlias name, zero delim    */
  char   *pSrcCreatorID;  /* Source Tbl's Creator/Owner ID, zero delim*/
  char   *pSrcTableName;  /* Source Tbl's Name, zero delim            */
  char   *pDstDBAliasName;/* Destination Tbl's DBAlias name,zero delim*/
  char   *pDstCreatorID;  /* Dest Tbl's Creator/Owner ID, zero delim  */
  char   *pDstTableName;  /* Dest Tbl's Name, zero delim              */
  short   DestColIndex;   /* Zero based index for the col in the
                             Destination Column Array that is the
                             Destination Column.  A value for this col
                             must be Put with the PSTPutColValue func.*/

  short   Reserved1;

  /* The following function may be used to get substrings from a LOB
     source column.                                                   */
  short (*pPSTGetLOBSegment)(short,    /* Col Handle for the Source
                                          LOB Col                     */
                             long,     /* Zero based offset of the
                                          1st byte of the LOB data
                                          to be returned.             */
                             long,     /* Length of the LOB data
                                          segment to be returned      */
                             void *,   /* Pointer to the output buffer
                                          area.  It is assumed this
                                          buffer area is at least as
                                          long as the requested segment
                                          length.                     */
                             long *,   /* Pointer to a return value
                                          that will indicate how
                                          long the full LOB data is   */
                             long *,   /* Pointer to a return value
                                          that will indicate how
                                          many bytes were actually
                                          returned.  This may be shorter
                                          than the requested length
                                          if the requested substring
                                          exceeds the end LOB data or if
                                          a Multi-byte char of NCHAR
                                          character would not be fully
                                          returned.                   */
                             short *); /* Pointer to a short Indicator
                                          variable. After the call, this
                                          field will contain -1 if the
                                          column is NULL.  Otherwise,
                                          this field will be zero.
                                          This parameter may be NULL  */

  /* The following function may be used to put the data for a LOB
     destination column.                                              */
  short (*pPSTPutLOB)(char *,  /* Pointer to a zero delimited string
                                  of a fully qualified file name for
                                  teh file that contains the LOB data
                                  for the destination column          */
                      short,   /* Indicator Variable.  -1 if the LOB
                                  is to get the NULL value.  If a -1
                                  is specified, the other function
                                  parameters are ignored.             */
                      char);   /* Delete File Flag.  0 indicates that
                                  the file should NOT be deleted after
                                  the LOB data has been copied to the
                                  destination column.  1 indicates
                                  that the file SHOULD be deleted.    */

  /* Exit Parms */
  long    NumParms;       /* Number of passed parameters.  Each parm
                             pointer points to a Zero Delimited string.
                             Parms specified as quoted strings are
                             here without the quotes.                 */
  char    *pParm[PST_CM_MAX_PARMS];

  void *  pVBParms;       /* Pointer to VB Procedure parms. 
                             IBM internal use only.                   */

  /* KSB: 08/17/2009 - Added to pass Current Row Number and Maximum 
  Rows for the current table to the EXITs*/
  long RowNumCur;         /* Current Row Number      	              */
  long MaxRows;           /* Maximum number of rows for the Extract 
                             File for the current table	              */
  void *  pODPPApi;       /* Pointer to ODPP API struct. */
  char    Reserved2[8];  /* Reserved for future use                  */

} PST_STRUCT_CM_EXIT_PARM;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif


/* Return codes to be returned from the Column Map EXIT() type exits   */
#define PST_CM_EXIT_SUCCESS         0  /* Col successfully transformed */
#define PST_CM_EXIT_REJECT_ROW      1  /* Discard row from the process */
#define PST_CM_EXIT_ABORT_PROCESS   2  /* Terminate the process        */



/* Prototype for the EXIT() user Exit */
#if !defined(__UNIX) && !defined(__unix)
__declspec(dllexport)
#endif
short  PSTColMapExit(PST_STRUCT_CM_EXIT_PARM *,
                     PST_STRUCT_CM_EXIT_COL_LIST *,    /* Source Cols */
                     PST_STRUCT_CM_EXIT_COL_LIST *);   /* Dest Cols   */



/**********************************************************************
 * Structs and defines for AGE() function Source Format Exits         *
 **********************************************************************/
/* The AGE() Function Source Format Exit has the responsibility to
 * examine a source date, character, or integer column value and convert
 * it into one of two standard date formats to be used as input to the
 * AGE function.  Alternatively there are several ways it can abort the
 * AGE function execution.  The abort modes are:
 *
 *      Skip the column - If the Process Skip Columns option is on, and
 *                        the source and destination columns have the
 *                        same column attributes, then the source
 *                        column is moved unchanged to the destination
 *                        column.  If the Option is not on, the row
 *                        is rejected.
 *      Declare the column as invalid - If the Process Invalid Columns
 *                        option is on, and the source and destination
 *                        columns have the same column attributes, then
 *                        the source column is moved unchanged to the
 *                        destination column.  If the option is not on,
 *                        the row is rejected.
 *      Reject the row -  Reject the row regardless of the Process Skip
 *                        and Process Invalid options.
 *      Abort the whole process
 *
 * The  exit uses a return code to specify which date format to use
 * or which abort mode to use.
 *
 * The exit may interrogate any columns from the input row.  It may
 * also interrogate some of the columns from the destination row.
 * A destination column that has any kind of an exit specified with
 * it, and is defined in the column map after the current destination
 * column will not be available.  All other destination columns are
 * always available.
 **********************************************************************/

/* Func Codes */
#define  PST_CM_SRCFMT_TRANSFORM          1  /* Convert source date to
                                                a standard form.      */
#define  PST_CM_SRCFMT_TERMINATE         99  /* End of the Process,
                                                Exit resources should
                                                be released.          */


/* This structure is passed to the AGE Source Format Exit.
   It contains various control information about the Source and
   Destination Tables and the Destination Column.                     */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CMAgeSrcFmt
{
  /* The reason the Exit is called.                                   */
  long    FuncCode;    /* PST_CM_SRCFMT_TRANSFORM or
                          PST_CM_SRCFMT_TERMINATE                     */

  /* The following function can be used to retrieve column values from
     the source or destination row.                                   */
  short   (*pPSTGetColValue)(short,   /* Col Handle for the Source or
                                         Destination Col              */
                             short,   /* The C data type for the result
                                         data.  This must be a
                                         PST_C_... constant.          */
                             short,   /* Precision, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370
                                             PST_C_INTEGER_CHAR_SZ    */
                             short,   /* Scale, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370        */
                             void *,  /* Pointer to the result storage
                                         area.                        */
                             long,    /* Buffer length
                                         For
                                             PST_C_CHAR_SZ
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBCHAR_SZ
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_INTEGER_CHAR_SZ
                                         it must include space for the
                                         null terminating byte.
                                         For
                                             PST_C_NCHAR_SZ
                                             PST_C_NVARCHAR_SZ
                                         it must include space for the
                                         terminating zero short integer.
                                         For
                                             PST_C_VARCHAR
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBVARCHAR
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_NVARCHAR
                                             PST_C_NVARCHAR_SZ
                                             PST_C_VARBINARY
                                         it must include space for the
                                         two byte length field.
                                         For all others, it the simple
                                         buffer length.               */
                             short *); /* Pointer to a short Indicator
                                          variable. After the call, this
                                          field will contain -1 if the
                                          column is NULL.  Otherwise,
                                          this field will be zero.
                                          This parameter may be NULL  */

  /* The following field points to a global 256 byte work area that can
     be used by the exit.  This area is cleared to binary zeros before
     the first call to any exit.  This area is shared by all exits in
     the Insert, Convert, or Load process.                            */
  char   *pGlblWorkArea;

  /* The following field points to a 256 byte work area that can be used
     by the exit.  This area is cleared to binary zeros before the first
     call to the exit.  There is a work area for each column, even if
     multiple columns use the same Exit
     In the event that the routine returns PST_CM_SRCFMT_ABORT_PROCESS, the
     product will look in this area for a NULL_terminated string to pass
     to the user                                                      */
  char   *pLclWorkArea;

  char   *pSrcDBAliasName;/* Source Tbl's DBAlias name, zero delim    */
  char   *pSrcCreatorID;  /* Source Tbl's Creator/Owner ID, zero delim*/
  char   *pSrcTableName;  /* Source Tbl's Name, zero delim            */
  char   *pDstDBAliasName;/* Destination Tbl's DBAlias name,zero delim*/
  char   *pDstCreatorID;  /* Dest Tbl's Creator/Owner ID, zero delim  */
  char   *pDstTableName;  /* Dest Tbl's Name, zero delim              */
  short   DstColIndex;    /* Zero based index for the col in the
                             Destination Column Array that is the
                             output column for the Age function       */
  short   SrcColIndex;    /* Zero based index for the col in the
                             Source Column Array that is the input
                             column to the Age function.              */

  /* The following fields define the source column value and its
     data type and length.  The ValueType #defines and structs are
     defined in PSTEXIT.H                                             */
  short  ValueType;      /* The form the following value is in
                                PST_C_CHAR_SZ
                                PST_C_INTEGER_CHAR_SZ
                                PST_C_TIMESTAMP
                                PST_C_SYB_DATETIME                    */
  short  ValueLen;      /* Length of the following value              */

  union {
     char                    CharStr[256+1]; /* PST_C_CHAR_SZ         */
     char                    IntStr[256+1];  /* PST_C_INTEGER_CHAR_SZ */
     PST_STRUCT_TIMESTAMP    TimeStamp;      /* PST_C_TIMESTAMP       */
     PST_STRUCT_SYB_DATETIME SybDateTime;    /* PST_C_SYB_DATETIME    */
  } InputValue;

  /* The following areas is used to return the output date.  This date
     will then be used as input to the AGE function.                  */
  /* Used with return code PST_CM_SRCFMT_USE_TIMESTAMP                */
  PST_STRUCT_TIMESTAMP    OutputTimeStamp;
  /* Used with return code PST_CM_SRCFMT_USE_SYB_DATETIME             */
  PST_STRUCT_SYB_DATETIME OutputSybDateTime;

  char    Reserved[24];   /* Reserved for future use                  */

} PST_STRUCT_CM_AGE_SRCFMT_PARM;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* Return codes to be returned from either AGE Source Format exits.   */
#define  PST_CM_SRCFMT_USE_TIMESTAMP     10
#define  PST_CM_SRCFMT_USE_SYB_DATETIME  11
#define  PST_CM_SRCFMT_SKIP              12
#define  PST_CM_SRCFMT_COL_INVALID       13
#define  PST_CM_SRCFMT_REJECT_ROW        14
#define  PST_CM_SRCFMT_ABORT_PROCESS     15



/* Prototype for the AGE() Source Format Exit */
#if !defined(__UNIX) && !defined(__unix)
__declspec(dllexport)
#endif
short  PSTColMapAgeSrcExit(PST_STRUCT_CM_AGE_SRCFMT_PARM *,
                           PST_STRUCT_CM_EXIT_COL_LIST *,    /* Source Cols */
                           PST_STRUCT_CM_EXIT_COL_LIST *);   /* Dest Cols   */





/**********************************************************************
 * Structs and defines for AGE() function Destination Format Exits    *
 **********************************************************************/
/* The AGE() Function Destination Format Exit has the responsibility to
 * convert a date into one of four different destination formats.
 * The input date is in both a PST_C_TIMESTAMP and PST_C_SYB_DATETIME
 * format.  The exit is directed to transform that date into either
 * PST_C_CHAR_SZ, PST_C_INTEGER_CHAR_SZ, PST_C_TIMESTAMP, or
 * PST_C_SYB_DATETIME format.  Which one is used depends on the data type
 * of the Destination column.  PST_C_CHAR_SZ is used for CHAR and VARCHAR
 * destination columns.  PST_C_INTEGER_CHAR_SZ is used for numeric
 * destination columns.  PST_C_TIMESTAMP is used for DB2 and ORACLE
 * Date/Time columns and PST_C_SYB_DATETIME is used for Sybase DateTime
 * columns.
 *
 * For PST_C_INTEGER_CHAR_SZ and PST_C_CHAR_SZ values, the ValueLen
 * field must be specified.  Either an explicit number of characters,
 * 0 thru 256, or -1 if the number of chars up to the first NULL
 * character is to be used.
 *
 * Alternatively there are several ways it can abort the AGE function
 * execution.  The abort modes are:
 *
 *      Skip the column - If the Process Skip Columns option is on, and
 *                        the source and destination columns have the
 *                        same column attributes, then the source
 *                        column is moved unchanged to the destination
 *                        column.  If the Option is not on, the row
 *                        is rejected.
 *      Declare the column as invalid - If the Process Invalid Columns
 *                        option is on, and the source and destination
 *                        columns have the same column attributes, then
 *                        the source column is moved unchanged to the
 *                        destination column.  If the option is not on,
 *                        the row is rejected.
 *      Reject the row -  Reject the row regardless of the Process Skip
 *                        and Process Invalid options.
 *      Abort the whole process
 *
 * The  exit uses a return code to specify if the conversion was
 * successful or which abort mode to use.
 *
 * The exit may interrogate any columns from the input row.  It may
 * also interrogate some of the columns from the destination row.
 * A destination column that has any kind of an exit specified with
 * it, and is defined in the column map after the current destination
 * column will not be available.  All other destination columns are
 * always available.
 **********************************************************************/

/* Func Codes */
#define  PST_CM_DSTFMT_TO_CHAR           1  /* Transform date to
                                               PST_C_CHAR_SZ          */
#define  PST_CM_DSTFMT_TO_INTEGER        2  /* Transform date to
                                               PST_C_INTEGER_CHAR_SZ  */
#define  PST_CM_DSTFMT_TO_TIMESTAMP      3  /* Transform date to
                                               PST_C_TIMESTAMP        */
#define  PST_CM_DSTFMT_TO_SYB_DATETIME   4  /* Transform date to
                                               PST_C_SYB_DATETIME     */
#define  PST_CM_DSTFMT_TERMINATE        99  /* End of the Process,
                                               Exit resources should
                                               be released.           */

/* This structure is passed to the AGE Destination Format Exit.
   It contains various control information about the Source and
   Destination Tables and the Destination Column.                     */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CMAgeDstFmt
{
  /* The reason the Exit is called.                                   */
  long    FuncCode;    /* PST_CM_DSTFMT_TO_CHAR
                          PST_CM_DSTFMT_TO_INTEGER
                          PST_CM_DSTFMT_TO_TIMESTAMP
                          PST_CM_DSTFMT_TO_SYB_DATETIME
                          PST_CM_DSTFMT_TERMINATE                     */

  /* The following function can be used to retrieve column values from
     the source or destination row.                                   */
  short   (*pPSTGetColValue)(short,   /* Col Handle for the Source or
                                         Destination Col              */
                             short,   /* The C data type for the result
                                         data.  This must be a
                                         PST_C_... constant.          */
                             short,   /* Precision, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370
                                             PST_C_INTEGER_CHAR_SZ    */
                             short,   /* Scale, only used cols
                                         with a Data Type of
                                             PST_C_DECIMAL_CHAR
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_DECIMAL_370        */
                             void *,  /* Pointer to the result storage
                                         area.                        */
                             long,    /* Buffer length
                                         For
                                             PST_C_CHAR_SZ
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBCHAR_SZ
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_DECIMAL_CHAR_SZ
                                             PST_C_INTEGER_CHAR_SZ
                                         it must include space for the
                                         null terminating byte.
                                         For
                                             PST_C_NCHAR_SZ
                                             PST_C_NVARCHAR_SZ
                                         it must include space for the
                                         terminating zero short integer.
                                         For
                                             PST_C_VARCHAR
                                             PST_C_VARCHAR_SZ
                                             PST_C_MBVARCHAR
                                             PST_C_MBVARCHAR_SZ
                                             PST_C_NVARCHAR
                                             PST_C_NVARCHAR_SZ
                                             PST_C_VARBINARY
                                         it must include space for the
                                         two byte length field.
                                         For all others, it the simple
                                         buffer length.               */
                             short *); /* Pointer to a short Indicator
                                          variable. After the call, this
                                          field will contain -1 if the
                                          column is NULL.  Otherwise,
                                          this field will be zero.
                                          This parameter may be NULL  */

  /* The following field points to a global 256 byte work area that can
     be used by the exit.  This area is cleared to binary zeros before
     the first call to any exit.  This area is shared by all exits in
     the Insert, Convert, or Load process.                            */
  char   *pGlblWorkArea;

  /* The following field points to a 256 byte work area that can be used
     by the exit.  This area is cleared to binary zeros before the first
     call to the exit.  There is a work area for each column, even if
     multiple columns use the same Exit                               
     In the event that the routine returns PST_CM_DSTFMT_ABORT_PROCESS, the
     product will look in this area for a NULL_terminated string to pass
     to the user                                                      */
  char   *pLclWorkArea;

  char   *pSrcDBAliasName;/* Source Tbl's DBAlias name, zero delim    */
  char   *pSrcCreatorID;  /* Source Tbl's Creator/Owner ID, zero delim*/
  char   *pSrcTableName;  /* Source Tbl's Name, zero delim            */
  char   *pDstDBAliasName;/* Destination Tbl's DBAlias name,zero delim*/
  char   *pDstCreatorID;  /* Dest Tbl's Creator/Owner ID, zero delim  */
  char   *pDstTableName;  /* Dest Tbl's Name, zero delim              */
  short   DstColIndex;    /* Zero based index for the col in the
                             Destination Column Array that is the
                             output column for the Age function       */
  short   SrcColIndex;    /* Zero based index for the col in the
                             Source Column Array that is the input
                             column to the Age function.              */

  /* The following dates is are passed as input to the exit.  The same
     date is in both fields.  The exit may use the one that is the
     most convienent.                                                 */
  PST_STRUCT_TIMESTAMP    InputTimeStamp;
  PST_STRUCT_SYB_DATETIME InputSybDateTime;

  /* The following fields are filled by the exit to return the output
     value and its length.                                            */
  short  ValueLen;      /* Length of the following value              */
                        /* Required for PST_C_CHAR_SZ and
                           PST_C_INTEGER_CHAR_SZ values.  -1 indicates
                           to use the chars up to the first NULL char */

  union {
     char                    CharStr[256+1]; /* PST_C_CHAR_SZ         */
     char                    IntStr[256+1];  /* PST_C_INTEGER_CHAR_SZ */
     PST_STRUCT_SYB_DATETIME SybDateTime;    /* PST_C_SYB_DATETIME    */
     PST_STRUCT_TIMESTAMP    TimeStamp;      /* PST_C_TIMESTAMP       */
  } OutputValue;

  char    Reserved[24];   /* Reserved for future use                  */

} PST_STRUCT_CM_AGE_DSTFMT_PARM;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif




#define  PST_CM_DSTFMT_SUCCESS            0
#define  PST_CM_DSTFMT_SKIP              21
#define  PST_CM_DSTFMT_COL_INVALID       22
#define  PST_CM_DSTFMT_REJECT_ROW        23
#define  PST_CM_DSTFMT_ABORT_PROCESS     24



/* Prototype for the AGE() Destination Format Exit */
#if !defined(__UNIX) && !defined(__unix)
__declspec(dllexport)
#endif
short  PSTColMapAgeDstExit(PST_STRUCT_CM_AGE_DSTFMT_PARM *,
                           PST_STRUCT_CM_EXIT_COL_LIST *,    /* Source Cols */
                           PST_STRUCT_CM_EXIT_COL_LIST *);   /* Dest Cols   */




#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */

#endif  /* #ifndef __PSTCMXIT_H */
