/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 1998,2009
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*****************************************************************
** PSTEXIT.H - This is the the main include for Optim exits.
**             PSTCMXIT.h is also need for Optim Column Map exits.
**
** preconditions:
**      none
**
**
 *
 *
 *    10 Jul 2009 BCK/RKP 7.2.0 3004 New Compiler Chnages
 *    Changes done in the file to support New Compiler VS2008
 *
 * Modified:
 *
 *    $Author:   SHILPAJ  $
 *    $Date:   11 Mar 2009 17:19:58  $
 *    $Revision:   1.2  $
 *    $Log:   Y:\RTWIN\711\ARCHIVES\pstexit.h_v  $
 *
 *    Rev 1.2   11 Mar 2009 17:19:58   SHILPAJ
 * Copyright Information Updated
 * 
 *    Rev 1.1   11 Apr 2008 14:14:04   BOBO
 * - Corrected PVCS comment area
 * 
 *    Rev 1.0   07 Apr 2008 08:16:26   BOBO
 * Initial revision.
 *
 * RTC/Jazz change log
 *
 * 28 Jul 2011 pdurga  8.1.0  4030 WI27090
 * Added Support for Teradata specific decimal and date datatypes.
 *
 *********************************************************************/

#ifndef __PSTEXIT_H
#define __PSTEXIT_H

#ifdef __cplusplus
extern "C" {                /* Assume C declarations for C++   */
#endif  /* __cplusplus */






/* Defines for the Data Types as kept internally by the product       */
#define PST_SQL_CHAR         1   /* Fixed size char, left justified
                                    and space padded                  */
#define PST_SQL_VARCHAR      2   /* short length field followed
                                    by character data                 */
#define PST_SQL_BINARY       3   /* Fixed size binary data, left
                                    justified and zero filled         */

#define PST_SQL_VARBINARY    4   /* short length field followed by
                                    binary data                       */
#define PST_SQL_BIT          5   /* Bit data in a single byte, 0 or 1 */
#define PST_SQL_TINYINT      6   /* 1 byte binary integer             */
#define PST_SQL_SMALLINT     7   /* 2 byte binary integer             */
#define PST_SQL_INTEGER      8   /* 4 byte binary integer             */
#define PST_SQL_FLOAT        9   /* four byte float                   */
#define PST_SQL_DOUBLE      10   /* eight byte float                  */
#define PST_SQL_DECIMAL     11   /* decimal number with precision
                                    and scale                         */
#define PST_SQL_ORA_NUMBER  12   /* ORACLE VARNUM numeric format with
                                    unspecified scale                 */
#define PST_SQL_DATE        13   /* Date only                         */
#define PST_SQL_TIME        14   /* Time only                         */
#define PST_SQL_DATETIME    15   /* Date and Time                     */
#define PST_SQL_TIMESTAMP   16   /* Date, Time, and Second fraction   */
#define PST_SQL_INFX_DEC_FLOAT 17   /* Informix Decimal Float         */
#define PST_SQL_INTERVAL    18   /* DateTime Interval                 */
#define PST_SQL_CLOB        19   /* Large character object            */
#define PST_SQL_BLOB        20   /* Large binary object               */

/* New Unicode DataTypes */
#define PST_SQL_MBCHAR      30   /* Multi-byte character data, e.g. UTF8 */
#define PST_SQL_MBVARCHAR   31   /* short BYTE length field followed
                                    by Multi-byte character data,
                                    e.g. UTF8                         */
#define PST_SQL_NCHAR       32   /* Char data where each character is
                                    an unsigned short integer, e.g. UTF16 */
#define PST_SQL_NVARCHAR    33   /* short BYTE length field followed
                                    Char data where each character is
                                    an unsigned short integer, e.g. UTF16 */
#define PST_SQL_MBCLOB      34   /* Large character object with
                                    multi-byte character data, e.g. UTF8 */
#define PST_SQL_NCLOB       35   /* Large character object
                                    where each character is an unsigned
                                    short integer, e.g. UTF16            */


/* Data Type defines to allow the exit to specify data format used in
   the exit's local data areas                                        */
#define PST_C_CHAR           1   /* Fixed size char, left justified
                                    and space padded                  */
#define PST_C_CHAR_SZ        2   /* Fixed size char, left justified
                                    and space padded.  With an extra
                                    byte for a NULL ZeroDlm.          */
#define PST_C_VARCHAR        3   /* short length field followed
                                    by character data                 */
#define PST_C_VARCHAR_SZ     4   /* short length field followed
                                    by character data. With an extra
                                    byte for a NULL ZeroDlm.          */
#define PST_C_BINARY         5   /* Fixed size binary data, left
                                    justified and zero filled         */
#define PST_C_VARBINARY      6   /* short length field followed by
                                    binary data                       */
#define PST_C_BIT            7   /* Bit data in a single byte, 0 or 1 */
#define PST_C_TINYINT        8   /* 1 byte signed binary integer      */
#define PST_C_SMALLINT       9   /* 2 byte signed binary integer      */
#define PST_C_INTEGER       10   /* 4 byte signed binary integer      */
#define PST_C_UTINYINT      11   /* 1 byte unsigned binary integer    */
#define PST_C_USMALLINT     12   /* 2 byte unsigned binary integer    */
#define PST_C_UINTEGER      13   /* 4 byte unsigned binary integer    */
#define PST_C_FLOAT         14   /* four byte float                   */
#define PST_C_DOUBLE        15   /* eight byte float                  */
#define PST_C_DECIMAL_CHAR  16   /* decimal number in char format.
                                    1st char is a '+' or '-'
                                       depending on sign
                                    There is always a decimal point.
                                    The digits are aligned so the dec
                                       point is always "Scale" chars
                                       from the right of the field.
                                       If needed, there are zeros
                                       between the sign char and the
                                       1st significant digit.         */
#define PST_C_DECIMAL_CHAR_SZ 17 /* same as PST_DECIMAL_CHAR with an
                                       extra byte for a NULL ZeroDlm. */
#define PST_C_DECIMAL_370     18 /* IBM 370 packed decimal number     */
#define PST_C_INTEGER_CHAR_SZ 19 /* Integer in char format with an
                                       extra byte for a NULL ZeroDlm.
                                    1st char is a '+' or '-'
                                       depending on sign
                                    There is never a decimal point.
                                    The digits are aligned so the ones
                                       digit is aligned with the right
                                       end of the the field. There are
                                       '0's between the 1st char and
                                       the 1st significant digit.     */
#define PST_C_SYB_DEC           20  /* SYBASE internal numeric format */
#define PST_C_SYB_MONEY         21  /* SYBASE MONEY format            */
#define PST_C_SYB_SMALLMONEY    22  /* SYBASE SMALLMONEY format       */
#define PST_C_DATE              23  /* ODBC Date structure format     */
#define PST_C_TIME              24  /* ODBC Time structure format     */
#define PST_C_TIMESTAMP         25  /* ODBC Timestamp structure format*/
#define PST_C_ORA_DATE          26  /* ORACLE Date format             */
#define PST_C_SYB_DATETIME      27  /* SYBASE DATETIME format         */
#define PST_C_SYB_SMALLDATETIME 28  /* SYBASE SMALLDATETIME format    */
#define PST_C_INFX_DECIMAL      29  /* Informix Decimal format        */
#define PST_C_INFX_INTERVAL     30  /* Informix Interval format       */

/* New Unicode DataTypes */
#define PST_C_MBCHAR            31   /* Fixed size Multi-byte char,
                                       e.g. UTF8, left justified and
                                       space padded                   */
#define PST_C_MBCHAR_SZ         32  /* Fixed size Multi-byte char,
                                       e.g. UTF8, left justified and
                                       space padded.  With an extra
                                       byte for a NULL ZeroDlm.       */
#define PST_C_MBVARCHAR         33  /* short byte length field followed
                                       by Multi-byte char data, e.g. UTF8 */
#define PST_C_MBVARCHAR_SZ      34  /* short byte length field followed
                                       by Multi-byte char data,
                                       e.g. UTF8, with an extra byte
                                       for a NULL ZeroDlm.             */
#define PST_C_NCHAR             35  /* Fixed size Char data where each
                                       char is an unsigned short integer,
                                       e.g. UTF16.  Data is left justified
                                       and space padded.                */
#define PST_C_NCHAR_SZ          36  /* Fixed size Char data where each
                                       char is an unsigned short integer,
                                       e.g. UTF16.  Data is left justified
                                       and space padded with an extra
                                       unsigned short at the end for
                                       for a Zero Delimiter.            */
#define PST_C_NVARCHAR          37  /* short byte length field followed
                                       by char data where each char is
                                       an unsigned short integer,
                                       e.g. UTF16.                      */
#define PST_C_NVARCHAR_SZ       38  /* short byte length field followed
                                       by char data where each char is
                                       an unsigned short integer,
                                       e.g. UTF16.  There is an extra
                                       unsigned short at the end for a
                                       Zero Delimiter. */
#define PST_C_WCHAR_STR         39  /* A standart C WCHAR, zero
                                       delimited string.                */

#define PST_C_TERA_DECIMAL      40  /* Teradata Decimal format        */
#define PST_C_TERA_DATE         41  /* Teradata Date format           */




/* Return codes from Column Get and Put functions */
#define PSTEXIT_SUCCESS                 0
#define PSTEXIT_INVALID_TYPE            1
#define PSTEXIT_INVALID_LENGTH          2
#define PSTEXIT_INVALID_PRECISION       3
#define PSTEXIT_INVALID_SCALE           4
#define PSTEXIT_NO_IND_VAR_FOR_NULL     5
#define PSTEXIT_COL_NOT_NULLABLE        6
#define PSTEXIT_INCOMPATIBLE_TYPE       7
#define PSTEXIT_TRUNC_WARNING           8
#define PSTEXIT_CONVERSION_ERROR        9
#define PSTEXIT_INVALID_THREAD         10
#define PSTEXIT_INVALID_HROW           11
#define PSTEXIT_INVALID_HCOL           12
#define PSTEXIT_COL_NOT_AVAIL          13
#define PSTEXIT_COL_NOT_UPDATABLE      14
#define PSTEXIT_COL_NOT_LOB            15
#define PSTEXIT_INVALID_LOB_SEG_OFFSET 16
#define PSTEXIT_INVALID_LOB_SEG_LENGTH 17
#define PSTEXIT_INVALID_FILE_NAME      18
#define PSTEXIT_LOB_FILE_OPEN_ERROR    19
#define PSTEXIT_LOB_FILE_IO_ERROR      20
#define PSTEXIT_LOB_FILE_TOO_LARGE     21
#define PSTEXIT_GENERAL_ERROR          99


/* Structures for the C Data Types */

/* For PST_C_VARCHAR, PST_C_VARCHAR_SZ, PST_C_VARBINARY,
   PST_C_MBVARCHAR and PST_C_MBVARCHAR_SZ Data Types     */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_VarCol
{
    unsigned short   Length;
    char             Data[1];   /* This field needs to be large enough
                                   to contain the data as specified
                                   by the length attribute.  If for a
                                   PST_C_VARCHARZ or PST_C_MBVARCHAR_SZ
                                   field, this array should be one char
                                   larger for the Zero Delimiter      */
} PST_STRUCT_VARCOL;


/* For PST_C_NVARCHAR and PST_C_NVARCHAR_SZ Data Types  */
typedef struct s_PST_NVarCol
{
    unsigned short   Length;
    unsigned short   Data[1];   /* This field needs to be large enough
                                   to contain the data as specified
                                   by the length attribute.  If for a
                                   PST_C_VARCHARZ or PST_C_MBVARCHAR_SZ
                                   field, this array should be one
                                   unsigned short larger for the Zero
                                   Delimiter      */
} PST_STRUCT_NVARCOL;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_SYB_DEC */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_SybDec
{
    unsigned char    Precision;
    unsigned char    Scale;
    unsigned char    Array[33];
} PST_STRUCT_SYB_DEC;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_SYB_MONEY */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_SybMoney
{
    long             MnyHigh;
    unsigned long    MnyLow;
} PST_STRUCT_SYB_MONEY;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_SYB_SMALLMONEY */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_SybSmallMoney
{
    long             Money;
} PST_STRUCT_SYB_SMALLMONEY;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_DATE */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CDate
{
    signed short     Year;
    unsigned short   Month;
    unsigned short   Day;
} PST_STRUCT_DATE;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_TIME */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CTime
{
    unsigned short   Hour;
    unsigned short   Minute;
    unsigned short   Second;
} PST_STRUCT_TIME;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_TIMESTAMP */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_CTimeStamp
{
    signed short     Year;
    unsigned short   Month;
    unsigned short   Day;
    unsigned short   Hour;
    unsigned short   Minute;
    unsigned short   Second;
    unsigned long    Fraction;    /* In billionth's of a second */
} PST_STRUCT_TIMESTAMP;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_ORA_DATE */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_OraDate
{
    unsigned char  Century; /* Century in excess 100, e.g. 1992 = 119 */
    unsigned char  Year;    /* Year in excess 100, e.g. 1992 = 192    */
    unsigned char  Month;   /* Month                                  */
    unsigned char  Day;     /* Day                                    */
    unsigned char  Hour;    /* Hr  in excess 1 notation, e.g. 0 = 1   */
    unsigned char  Minute;  /* Min in excess 1 notation, e.g. 0 = 1   */
    unsigned char  Second;  /* Sec in excess 1 notation, e.g. 0 = 1   */
} PST_STRUCT_ORA_DATE;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_SYB_DATETIME */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_SybDateTime
{
    long     Date;    /* Number of days since 1/1/1900        */
    long     Time;    /* Time in 1/300th's sec since midnight */
} PST_STRUCT_SYB_DATETIME;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_SYB_SMALLDATETIME */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_SybSmallDateTime
{
    unsigned short   Date;    /* Number of days since 1/1/1900 */
    unsigned short   Time;    /* Time in min's since midnight  */
} PST_STRUCT_SYB_SMALLDATETIME;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif



/* For PST_C_INFX_DECIMAL */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_InfxDecimal
{
    short dec_exp;              /* exponent base 100            */
    short dec_pos;              /* sign: 1=pos, 0=neg, -1=null  */
    short dec_ndgts;            /* number of significant digits */
    char  dec_dgts[16];         /* actual digits base 100       */
} PST_STRUCT_INFX_DECIMAL;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif

/* For PST_C_TERA_DECIMAL */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_TeraDecimal
{
   short dec_sign;                 /* sign: 1=pos, 0=neg, -1=null          */
   short dec_numdigits;            /* number of significant digits         */
   char  dec_digits[38];           /* actual digits base 100               */
} PST_STRUCT_TERA_DECIMAL ;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif




/* For PST_C_INFX_INTERVAL */
#if !defined(__UNIX) && !defined(__unix)
#pragma pack(1)
#endif
typedef struct s_PST_InfxInterval
{
    short                    in_qual;     /* Encoded Datetime qualifier   */
    PST_STRUCT_INFX_DECIMAL  in_dec;      /* Actual Datetime data         */
} PST_STRUCT_INFX_INTERVAL;
#if !defined(__UNIX) && !defined(__unix)
#pragma pack()
#endif




#ifdef __cplusplus
}                                    /* End of extern "C" { */
#endif  /* __cplusplus */

#endif  /* #ifndef __PSTEXIT_H */
