/* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2014
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/

#include "odpp_affinityexit.h"

// INTERNAL FUNCTIONS:

int SetCodePoints(UChar32 *pBuf, 
                  int iBufSizeChars,  
                  ODPP_WCHAR **pCharacterSets, 
                  short numberOfCharacterTypes, 
                  short *Sizes);

int GetCharSetSizes(ODPP_WCHAR **pCharacterSets, 
                    short numberOfCharacterTypes, 
                    short *Sizes);

int SetLanguage(UChar32 *pBuf, 
                int iBufSizeChars, 
                short *psClassCount, 
                short *Sizes, 
                ODPP_WCHAR **pCharacterSets, 
                short numberOfCharacterTypes);

// Declare the Affinity exit functions. Since the exit is always dynamically loaded, 
// pointers to the functions will be obtained at run-time by name - so the names 
// are important and are case sensitive.
Exit_GetLanguageCustom GetLanguageCustom;


int GetLanguageCustom(UChar32 *pBuf, 
                      int iBufSizeChars, 
                      short *psClassCount, 
                      short *Sizes)
{
   short numberOfCharacterTypes = 3;
   ODPP_WCHAR *pCharacterSets[3] = { ODPPCNVTOWSTR1("abcdefghivwxyz\x00e6\x00e5\x00f8"),
                                     ODPPCNVTOWSTR1("ABCUVWXYZ\x00c6\x00c5\x00d8"),
                                     ODPPCNVTOWSTR1("0123456789")
                                   };   

   return(SetLanguage(pBuf, iBufSizeChars, psClassCount, Sizes, pCharacterSets, numberOfCharacterTypes));
}

int SetCodePoints(UChar32 *pBuf, 
                  int iBufSizeChars,  
                  ODPP_WCHAR **pCharacterSets, 
                  short numberOfCharacterTypes, 
                  short *Sizes)
{
   short i, j;
   for(i = 0; i < numberOfCharacterTypes; i++)
   {
      if(iBufSizeChars < Sizes[i])
      {
         return EXIT_RC_INTERNAL_ERROR;
      }
      for(j = 0; j < Sizes[i]; j++)
      {
         *(pBuf + j) = (UChar32)pCharacterSets[i][j];
      }
      pBuf = pBuf + Sizes[i];
      iBufSizeChars = iBufSizeChars - Sizes[i];
   }
   return EXIT_RC_OK;
}

int GetCharSetSizes(ODPP_WCHAR **pCharacterSets, 
                    short numberOfCharacterTypes, 
                    short *Sizes)
{
   short i;
   if(numberOfCharacterTypes > ODPP_MAX_LANGUAGE_CLASSES)
   {
      return EXIT_RC_INVALID_CLASS_COUNT;
   }
   for(i = 0; i < numberOfCharacterTypes; i++)
   {
      Sizes[i] = (short)WCSLEN((ODPP_WCHAR *)pCharacterSets[i]);
   }
   return EXIT_RC_OK;
}

int SetLanguage(UChar32 *pBuf, 
                int iBufSizeChars, 
                short *psClassCount, 
                short *Sizes, 
                ODPP_WCHAR **pCharacterSets, 
                short numberOfCharacterTypes)
{
   RETVAL retVal;

   if(NULL == pBuf)
   {
      retVal = GetCharSetSizes(pCharacterSets, numberOfCharacterTypes, Sizes);
      if(EXIT_RC_OK != retVal)
      {
         return retVal;
      }
      *psClassCount = numberOfCharacterTypes;
      return EXIT_RC_OK;
   }
   retVal = SetCodePoints(pBuf, iBufSizeChars, pCharacterSets, numberOfCharacterTypes, Sizes);
   if(EXIT_RC_OK != retVal)
   {
      return retVal;
   }
   *psClassCount = numberOfCharacterTypes;
   return EXIT_RC_OK;
}
