/* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2014
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/

#ifndef MOD_AFFINITYEXIT_H
#define MOD_AFFINITYEXIT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <wchar.h>


// AFFINITY EXIT RETURN CODES

// The operation was successful. Should be returned by GetLanguageCustom when successful.
#define EXIT_RC_OK 0

// The number of custom character classes exceeds the maximum number of classes supported,
// that is defined by ODPP_MAX_LANGUAGE_CLASSES
#define EXIT_RC_INVALID_CLASS_COUNT 1

// The operation was unsuccessful. An error condition arose within the exit preventing it 
// from continuing.
#define EXIT_RC_INTERNAL_ERROR 2

// Defines

// The maximum number of custom classes supported by ODPP.
// This value must not be changed !!!
#define ODPP_MAX_LANGUAGE_CLASSES        15

#ifndef UChar32
/**
 * Define UChar32 as a type for single Unicode code points.
 * UChar32 is a signed 32-bit integer (same as int32_t).
 *
 * The Unicode code point range is 0..0x10ffff.
 * All other values (negative or >=0x110000) are illegal as Unicode code points.
 * They may be used as sentinel values to indicate "done", "error"
 * or similar non-code point conditions.
 */
#define UChar32                     int  
#endif

#ifndef ODPP_WCHAR
#define ODPP_WCHAR                  wchar_t   
#endif

#ifndef WCSLEN
#define WCSLEN                      wcslen
#endif

#ifndef RETVAL
#define RETVAL                      int
#endif

/* MACRO TO CONVERT the passed in value to a wide string */
#if defined(__ZOSC__) || defined(__ZOSCPP__)
   #define ODPPCNVTOWSTR1(CAT_TOWIDE) u##CAT_TOWIDE
#else
   #define ODPPCNVTOWSTR1(CAT_TOWIDE) L##CAT_TOWIDE
#endif

// AFFINITY EXIT FUNCTION TYPES

// Return the custom Language classes.
typedef int Exit_GetLanguageCustom(UChar32 *pBuf, 
                                   int iBufSizeChars, 
                                   short *psClassCount, 
                                   short *Sizes);


#ifdef __cplusplus
}
#endif


#endif
