/* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/

#ifndef MOD_EXIT_H
#define MOD_EXIT_H

#ifdef __cplusplus
extern "C" {
#endif

// HASH EXIT RETURN CODES

// The operation was successful. Should be returned by init and get_key when successful.
#define EXIT_RC_OK 0
// The hash exit has no key to provide. Should be returned by get_key when everything is 
// nominal but no key is available. Optim will proceed using the SHA-256 algorithm.
#define EXIT_RC_NO_KEY 1
// The operation was unsuccessful. An error condition arose within the exit preventing it 
// from continuing.
#define EXIT_RC_INTERNAL_ERROR 2

// HASH EXIT FUNCTION TYPES

// The init function initializes the hash exit.
typedef int exit_init(void);

// The get_key function provides a secret key (hash seed) for use with the HMAC algorithm.
// The key, for the HMAC algorithm, is simply a byte array of a given length. The ODPP Hash 
// library will keep the key in obfuscated form, with the obfuscator being supplied by the 
// exit itself. The key will only be reified just prior to the invocation of the HMAC function 
// and will be overwritten immediately after. The memory in which the key is provided by the 
// exit must be explicitly allocated by the exit and will be overwritten and explicitly freed 
// by the hashing library.
typedef int exit_get_key(unsigned char **key, unsigned int *key_length, unsigned int *obfuscator);

// The term function terminates the hash exit.
typedef void exit_term(void);
    
#ifdef __cplusplus
}
#endif


#endif
