/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*----------------------------------------------------------------------------
 * File Name:  source.c
 *
 * Purpose:    Reference Service Provider implementation file
 *
 * Functions:
 *
 *  Name              Purpose
 *  -------Public Functions-------   
 *  Init              Initialize the Provider
 *  Service           Service routine to perform masking. 
 *  Terminate         Terminates the service provider
 *  GetDetails        Gets the service provider details
 *  GetErrorMsg       Returns the error message for the supplied Error Code 
 *                    and language Supplied
 *  GetInfo           Gets provider specific information requested by caller
 *
 *  Structures:
 *
 *  Name              Purpose
 *
 *  Author:
 *----------------------------------------------------------------------------*/
#include "source.h"

/*-----------------------------------------------------------------------------
 * Function:   Init
 *
 *    Desc:
 *       Initializes the service provider.
 *       This function initializes the provider specific control block
 * 		and assigns to framework control block.
 *
 *    Parms:
 *       pCtrlBlk  :  Control Block supplied by the framework
 *       pSvcDef   :  Service Block supplied by the user contains all 
 *  	              initialization parameter for the provider
 *
 *    Returns:
 *       SUCCESS : ODPPSUCCESS
 *       FAILURE : Can return various error codes depending upon the error.
 *-----------------------------------------------------------------------------*/

RETVAL Init(DP_BLK_DEF_X *pCtrlBlk,
            DP_SVC_DEF_X *pSvcDef)
{
	
	ODPP_WCHAR	      			*pDestColName  = NULL;  // Destination column name 
	DP_MYCTRL_DEF				   *pMyCtrlBlk    = NULL;  // DP_MYCTRL_DEF is my service
									         		            //Provider control block 
   DP_INIT_OP_DEF_X 				*pParams       = NULL;  // DP_INIT_OP_DEF structure 
												                  //which contains provider 
												                  //initialization parametes
   DP_FIELD_DEF_X    			*pFieldDef     = NULL;  // Contains Columns provided 
	RETVAL			   			retval	      = 0;
	short						      sPrmCount      = 0;     // to loop through parameter list
	short                      sFldCnt        = 0;     // to loop through field list 
   short                      sDstColType    = 0;     // To search dest col by index
                                                      //or name 
	DP_SRCCOL_DEF_X            *pSrcCols      = NULL;

   //Allocate the Service Provider Control Block and assign it to framework control block
   //This would be service provider local storage that will presist between the calls to 
   //this service provider from ODPP service manager
   pMyCtrlBlk = (DP_MYCTRL_DEF *) malloc(sizeof(DP_MYCTRL_DEF));
	if(NULL == pMyCtrlBlk)
	{
      //malloc failed, Capture the error and go to errorclean.
      //You need to allocate your error codes in ODPPCmnErrCodes10.h file,
      //Check the next range to allocate in ODPPCmnErrCodes10.h file and 
      //allocate	the eror code range for your provider
		retval = MY_ERR_CTRLBLK_MALLOC_FAILED;
		goto OnErrorClean;
	}

	//Allocation of my control block is successful, initialize the memory with zeros
	memset(pMyCtrlBlk, 0, sizeof(DP_MYCTRL_DEF));

	
   //Initialize the Srvice Provider control block from the parameters provided in 
   //DP_INIT_OP_DEF
	pParams = pSvcDef->pParams; // pParams contains Parameters 

	// Process the parameters that are supported by the custom service provider.
   // If the provider is invoked using the ODPP grammar using LIB= then the parameter
   // names are set in the pParamNameW member.
	for(sPrmCount = 0; sPrmCount < pSvcDef->sParamCount; sPrmCount ++)
	{
      if(NULL != pParams->pParamNameW)
      {
         if((WCSICMP(pParams->pParamNameW, ODPPCNVTOWSTR1("u_operator")) == 0))
         {
            if(WCSICMP(pParams->pVal, ODPPCNVTOWSTR1("++")) == 0)
            {
               pMyCtrlBlk->iOperator = PARAM_OPERATOR_INC;
               pMyCtrlBlk->sSrc1ColIndex = 0;
               pMyCtrlBlk->sDestColIndex = 0;
               sDstColType = DEST_COL_TYPE_INDEX;
            }
            else if(WCSICMP(pParams->pVal, ODPPCNVTOWSTR1("+")) == 0)
            {
               pMyCtrlBlk->iOperator = PARAM_OPERATOR_ADD;
               pMyCtrlBlk->sSrc1ColIndex = 0;
               pMyCtrlBlk->sSrc2ColIndex = 1;
               pMyCtrlBlk->sDestColIndex = 0;
               sDstColType = DEST_COL_TYPE_INDEX;
            }
            else
            {
               pMyCtrlBlk->iOperator = PARAM_OPERATOR_NONE;
            }
         }         
      }
      // If the provider is invoked using the ODPP API then the parameter
      // values may be processed as shown below.
		//switch(pParams->usParameterID)
		//{
		//	// usParameterID contains parameter name 
		//	case PARAM_SRC1_COLINDEX:
		//		{
		//			pMyCtrlBlk->sSrc1ColIndex = (short)pParams->uiVal;
		//		}
		//		break;
		//	case PARAM_SRC2_COLINDEX:
		//		{
		//			pMyCtrlBlk->sSrc2ColIndex = (short)pParams->uiVal;
		//		}
		//		break;
  //       case PARAM_DEST_COLINDEX:
		//		{
		//			pMyCtrlBlk->sDestColIndex = (short)pParams->uiVal;
  //             sDstColType = DEST_COL_TYPE_INDEX;
		//		}
		//		break;
  //       case PARAM_DEST_COLNAME:
		//		{
		//			pDestColName = pParams->pVal;
  //             sDstColType  = DEST_COL_TYPE_NAME; 
		//			//pDestColName = pDestColName;
		//		}
		//		break;
  //       case PARAM_OPERATOR:
		//		{
		//			pMyCtrlBlk->iOperator = (int) pParams->uiVal;
		//		}
		//		break;
		//}// End of switch(pParams->usParameterID) 

      if(pParams->usParameterID == ODPP_OPR_SOURCE_COLNAME)
      {
         pFieldDef = pCtrlBlk->pFldDef;
         for(sFldCnt = 0; sFldCnt < pSvcDef->sFldCount; sFldCnt++)
         {
            if(memcmp(pFieldDef->pColNameW, 
                      pParams->pVal, 
                      pFieldDef->iColNameWBytes) == 0)
               break;
            MOVENEXT(pFieldDef)
         }         
         pMyCtrlBlk->sSrc1ColIndex = sFldCnt;
         pMyCtrlBlk->sDestColIndex = sFldCnt;
         sDstColType = DEST_COL_TYPE_INDEX;
      }
		MOVENEXT(pParams)

	}// End of for(sPrmCount = 0; sPrmCount < pSvcDef->sPrmCount; sPrmCount ++) 

   
   // Check if source1 and source2 column indexes are in range 
   if((pMyCtrlBlk->sSrc1ColIndex > pCtrlBlk->sSrcColCount) ||
      (pMyCtrlBlk->sSrc2ColIndex > pCtrlBlk->sSrcColCount) ||
      (pMyCtrlBlk->sSrc1ColIndex < 0)||
      (pMyCtrlBlk->sSrc2ColIndex < 0))
   {
      retval = MY_ERR_SRC_INDEX_OUT_OF_RANGE;
      goto OnErrorClean;
   }  
   //Find the destinatin column in the list of columns supplied
   switch(sDstColType)
   {
      // Search Destination ColName into the list of Columns(Fields) 
      //sent by the caller 
      case DEST_COL_TYPE_NAME:
         {
            // Check if destination column name is provided 
            if(pDestColName == NULL)
            {
	             retval = MY_ERR_DESTCOL_NULL;
	             goto OnErrorClean;
            }
            
            //Search dest col name in list of fields 
            pFieldDef = pCtrlBlk->pFldDef;
            for(sFldCnt = 0; sFldCnt < pSvcDef->sFldCount; sFldCnt++)
            {
               if(memcmp(pFieldDef->pColNameW, 
                         pDestColName, 
                         pFieldDef->iColNameWBytes) == 0)
                  break;
               MOVENEXT(pFieldDef)
            }
   
            //If we failed to find the field in the supplied list of
            //fields throw an error 
            if(sFldCnt >= pSvcDef->sFldCount)
            {
               retval = MY_ERR_INVALID_DEST_COLNAME;
	            goto OnErrorClean;
	         }
            pMyCtrlBlk->sDestColIndex = sFldCnt;
         }
         break;
      case DEST_COL_TYPE_INDEX :
         {
            
            // Search dest col index in list of fields 
            pFieldDef = pCtrlBlk-> pFldDef;
            if((pMyCtrlBlk->sDestColIndex > pSvcDef->sFldCount) ||
               (pMyCtrlBlk->sDestColIndex < 0))
            {
               retval = MY_ERR_DST_INDEX_OUT_OF_RANGE;
               goto OnErrorClean;
            }
            
            for(sFldCnt = 0; sFldCnt < pMyCtrlBlk->sDestColIndex ; sFldCnt++)
            {
               MOVENEXT(pFieldDef)
            }

         }
         break;
      default :
         retval = MY_ERR_INVALID_DEST_COLUMN;
         goto OnErrorClean; 

   }// End switch(sDstColType) 
   
   //Save the destination column supplied by the caller in our control block
   pMyCtrlBlk->pDstCol = pFieldDef;

   //Check for source columns details like datatype, length/precesion, scale etc
   pSrcCols = &pCtrlBlk->SrcCol;
   while(NULL != pSrcCols)
   {
      switch(pSrcCols->pSrcFldDef->sDatatype)
	   {
		   case ODPPDATATYPE_INTEGER:
				   break;
         case ODPPDATATYPE_WVARCHAR_SZ:
				break;
		   default:
			   // Data type not supported 
			   retval = MY_ERR_SRC_COL_DATATYPE_NOTSUPPORTED;
			   goto OnErrorClean;
	   }
      pSrcCols = pSrcCols->pNext;
   }
   
   switch(pMyCtrlBlk->pDstCol->sDatatype)
	{
		case ODPPDATATYPE_INTEGER:
				break;
      case ODPPDATATYPE_WVARCHAR_SZ:
				break;
		default:
			// Data type not supported 
			retval = MY_ERR_DST_COL_DATATYPE_NOTSUPPORTED;
			goto OnErrorClean;
	}
  
   //We have successfully initialized the Service Provider control block 
   //now assign it to the framework control block 
   //pExt of framework control block holds the provider specific control block 
   pCtrlBlk->pExt = (void *)pMyCtrlBlk; 

	return ODPPSUCCESS;
	
OnErrorClean:
	// Clear all the allocation on failure 
	if(NULL != pMyCtrlBlk)
	{ 
		// Free my control block 
		free(pMyCtrlBlk);
      pMyCtrlBlk = NULL; 
	}// end of if(NULL != pMyCtrlBlk) 
    	
	return retval;

}


/*---------------------------------------------------------------------------
 * Function:   Service
 *
 *    Desc:
 *       Service routine to perform masking. Accepts a list of records(rows)
 *       loops through all of rows masking them one by one.
 *       It performs the arithmatic operation and put results back into 
 *       destination buffer.
 *
 *    Parms:
 *       sMethod  :  Method to override the masking method setting supplied 
 *                   during Init
 *       pCtrlBlk :  Control Block supplied by the Framework
 *       pRowSet  :  Records list
 *
 *    Returns
 *       SUCCESS  : ODPPSUCCESS
 *       FAILURE  : Can return various error codes depending upon the error.             
 *
 *-------------------------------------------------------------------------*/
RETVAL Service(short sMethod,             
               DP_BLK_DEF_X *pCtrlBlk,      
               DP_ROWSET_DEF *pRowSet)
{
   RETVAL   			   retval	      = ODPPSUCCESS; 
   int					   iRowCount		= 0;     // To loop through all rows 
   int                  *piDstSize     = NULL;  // bytes of dest buffer 
   int                  *piOutLen      = NULL;  // Bytes of out data
   short                sMyColCnt      = 0;
   short					   sSrvMethod;             // Masking method 
   short					   sLoopCol		   = 0;     // to loop through src columns 
   unsigned char        *pDstBuf       = NULL;  // Destination buffer value 
   unsigned char        *pSrc1Buf      = NULL;
   unsigned char        *pSrc2Buf      = NULL;
   DP_ROW_DEF	         *pRowDef       = NULL;  // contains rows 
   DP_FIELD_DATA_DEF    *pData         = NULL;  // contains data 
   DP_MYCTRL_DEF        *pMyCtrlBlk    = NULL;  //Service provider specific control block 
   DP_FIELD_DEF_X       *pFld = NULL;
   int                  iSrcVal1       = 0;
   int                  iSrcVal2       = 0;
   int                  iResult        = 0;
   wchar_t *pMasked = L"masked";

   //Get the My Service provider control block from framework control block 
   pMyCtrlBlk = (DP_MYCTRL_DEF*)pCtrlBlk->pExt;
   if(NULL == pMyCtrlBlk)
   {
	   //There is no provider control block allocated in framework 
	   //control block. Capture the specific error code and return 
	   retval = MY_ERR_MYCTRLBLK_NOT_FOUND;
      return retval;
   }

   //Check if Rowset and dataset is empty, return if there is no record 
   if((pRowSet->iCount <= 0) || (pRowSet->pRowDefine == NULL))
   {
	   // RowSet is empty,Capture the specific error and return 
	   retval = MY_ERR_ROWSET_EMPTY;
	   return retval;
   }
   else
   {
    	// Assign row list address to DP_ROW_DEF pointer 
		pRowDef = pRowSet->pRowDefine;
   }

   // Check if Source1, source2 indexes provided by the user are in range 
   if((pMyCtrlBlk->sSrc1ColIndex >= pRowDef->sCount) || 
      (pMyCtrlBlk->sSrc2ColIndex >= pRowDef->sCount))
   {
		// Source column index is greater than number of column 
		retval = MY_ERR_SRCCOLIDX_EXCEEDS_ROWDEF_COLCOUNT;
		return retval;
   }

   // Check if destination column index provided by the user is in range 
   if(pMyCtrlBlk->sDestColIndex >= pRowDef->sCount)
   {
		// Destination column index is greater than number of column 
		retval = MY_ERR_DSTCOLIDX_EXCEEDS_ROWDEF_COLCOUNT;
		return retval;
   }

   // Check the method for masking 
   if(sMethod == ODPP_METHOD_DEFAULT)
   {
       sSrvMethod = pCtrlBlk->sMethod;
   }
   else if(sMethod == ODPP_METHOD_MASK ||
           sMethod == ODPP_METHOD_RANDOM)
   {
       sSrvMethod = sMethod;
   }
   else
   { 
	   //Invalid MASK method provided 
      retval = MY_ERR_INVALID_METHOD;
	   return retval;
   }

	// Process each row in rowset 
   for(iRowCount = 0; iRowCount < pRowSet->iCount; iRowCount++)
   {
      // Set last row processed 
	   pRowSet->iLastPrcsRow = iRowCount;

	   if(TRUE == pRowDef->bProcessed)
	   {
	     	//This row has been already processed, move to next row
         //This helps out during situations when service need to be 
         //called again because of failure of some rows or any other
         //reason
		   MOVENEXT(pRowDef)
         continue;
	   }
      else
      { 
         //Set Row processed to true.
         //In case if the same recordset is sent again then this is a way 
         //to detect if row has already been processed or not. This will 
         //reduce duplicate processing of rows.
         pRowDef->bProcessed = TRUE;
      }
	
      pFld = pCtrlBlk->pFldDef;
      pData = pRowDef->pFldDataDefine;
      // Get the data block pointer for the supplied source column 1
      for(sLoopCol=0; sLoopCol < pMyCtrlBlk->sSrc1ColIndex; sLoopCol++)
      {
         MOVENEXT(pData);
         pFld = pFld->pNext;
      }
      if(NULL == pData)
      {            
         //Could not find the source data column for this row,
         //there is no point to move ahead with this row so break 
         retval = MY_ERR_NO_SOURCE_DATA_COL;
         goto OnRowError;
      }      
      // Check if source column buffer is null 
      if(TRUE == pData->bSrcNull && NULL == pData->pSrcBuf)
      { 
          // Source column has a NULL buffer
          retval = MY_ERR_SRC_COL_BUFF_NULL;
          goto OnRowError;
      }
      // Get the source value and convert it into source datatype 
      pSrc1Buf = pData->pSrcBuf;
      switch(pFld->sDatatype)
      {
			case ODPPDATATYPE_INTEGER:
              iSrcVal1 = (int)(*(int *)pSrc1Buf);  
              break;
         case ODPPDATATYPE_WVARCHAR_SZ:
              //iSrcVal1 = (int)(*(int *)pSrc1Buf);  
              break;
      }
      if(pMyCtrlBlk->sSrc2ColIndex > 0)
      {
         pFld = pCtrlBlk->pFldDef;
         pData = pRowDef->pFldDataDefine;
         // Get the data block pointer for the supplied source column 2
         for(sLoopCol=0; sLoopCol < pMyCtrlBlk->sSrc2ColIndex; sLoopCol++)
         {
            MOVENEXT(pData);
            pFld = pFld->pNext;
         }
         if(NULL == pData)
         {            
            //Could not find the source data column for this row,
            //there is no point to move ahead with this row so break 
            retval = MY_ERR_NO_SOURCE_DATA_COL;
            goto OnRowError;
         }      
         // Check if source column buffer is null 
         if(TRUE == pData->bSrcNull && NULL == pData->pSrcBuf)
         { 
             // Source column has a NULL buffer
             retval = MY_ERR_SRC_COL_BUFF_NULL;
             goto OnRowError;
         }
         // Get the source value and convert it into source datatype 
         pSrc2Buf = pData->pSrcBuf;
         switch(pFld->sDatatype)
         {
			   case ODPPDATATYPE_INTEGER:
                 iSrcVal2 = (int)(*(int *)pSrc2Buf);  
                 break;
         }
      }

      // Find the Destination Column data block pointer 
      pData = pRowDef->pFldDataDefine;

      // Get the data block pointer for the supplied destination column 
      for(sLoopCol=0; sLoopCol < pMyCtrlBlk->sDestColIndex; sLoopCol++)
      {
         MOVENEXT(pData)
      }
 
      if(NULL == pData)
      {         
         //Could not find the destination column data block for this row,
         //there is no point to move ahead with this row so break 
         retval = MY_ERR_NO_DEST_DATA_COL;
         goto OnRowError;
      }

      //Check the destination column buffer and check where to copy result
      if(pCtrlBlk->bCopyToDest && 
        (NULL == pData->pDstBuf || TRUE == pData->bDstNull))
      {  //We were told to copy to destination buffer but destination 
         //buffer is NULL so error out
         retval = MY_ERR_NULL_DEST_BUFFER;
         goto OnRowError;
      }
      else if(!pCtrlBlk->bCopyToDest && 
        (NULL == pData->pSrcBuf || TRUE == pData->bSrcNull))
      {
         //We were told to copy to source buffer but source
         //buffer is NULL so error out
         retval = MY_ERR_NULL_DEST_BUFFER;
         goto OnRowError;
      }

      //Perform the operation
      switch(pMyCtrlBlk->iOperator)
      {
         case PARAM_OPERATOR_ADD:
            iResult = iSrcVal1 + iSrcVal2;
            break;
         case PARAM_OPERATOR_INC:
            iResult = iSrcVal1 + 1;
            break;
         case PARAM_OPERATOR_NONE:
            break;
         default:
            retval = MY_ERR_INVALID_OPERATOR;
            goto OnRowError;
            break;
      }// End switch
      

      //If bCopyToDest is TRUE then copy result in destination buffer.
      //If bCopyToDest is FALSE then copy result in source buffer.
      if(!pCtrlBlk->bCopyToDest) 
      {
         // Return value in source buffer 
         pDstBuf = pData->pSrcBuf; 
         piDstSize = &pData->iSrcBufLen;
      }
      else 
      {
         // Return value in destination buffer 
         pDstBuf = pData->pDstBuf;            
         piDstSize = &pData->iDstBufLen;
      }
      piOutLen = &pData->iOutBufLen;

      // Convert the destination value to dest datatype 
      switch(pMyCtrlBlk->pDstCol->sDatatype)
      {
         case ODPPDATATYPE_INTEGER:
            {
              memcpy(pDstBuf, (void*)&iResult, sizeof(int));
              *piOutLen = sizeof(int);
            }
            break;
         case ODPPDATATYPE_WVARCHAR_SZ:
            {
              wcscpy((wchar_t *)pDstBuf, pMasked);
              *piOutLen = wcslen(pMasked) * sizeof(wchar_t);
            }
            break;
      }

      //Everything was success Reset the error codes for the Row
      //and move to the next row 
      pData->iErrorCode = ODPPSUCCESS;
      pRowDef->bHasError = FALSE;
      
      // move to next row 
      MOVENEXT(pRowDef)
      continue;

   // Row encounter error, set the error code and move to next row 
   OnRowError:
      if(NULL != pData)
      {
         pData->iErrorCode = retval;
      }
      pRowDef->bHasError = TRUE;
          
      // Move to the next row 
      MOVENEXT(pRowDef)

	}//End for(iRowCount = 0; iRowCount <= pRowSet->iCount; iRowCount++)

	return ODPPSUCCESS;
}// END Service() 





/*---------------------------------------------------------------------------
 * Function:   Terminate
 *
 *    Desc:
 *       Terminates the my service provider. Deallocates all allocated 
 * 		memory and resources.
 *
 *    Parms:
 *       pCtrlBlk :  Control Block supplied by the Framework which holds the
 *		               provider specific control block
 *
 *    Returns:
 *       SUCCESS  : ODPPSUCCESS 
 *       FAILURE  : It can return various error codes depending upon the error.             
 *
 *-----------------------------------------------------------------------------*/

RETVAL Terminate(DP_BLK_DEF_X *pCtrlBlk)
{
	DP_MYCTRL_DEF	*pMyCtrlBlk		= NULL; //Service Provider control block 

	// We need to free the provider control block which we have allocated in Init 
	// Get the service provider control block from the framework control block 
   pMyCtrlBlk = (DP_MYCTRL_DEF*)pCtrlBlk->pExt; 
   if(NULL == pMyCtrlBlk)
   {
		// There is no my service provider control block in Framework control block 
		return MY_ERR_MYCTRLBLK_NOT_FOUND;
	}

   // Free the control block 
   // We should free all the memory allocated into the control block here
   free(pMyCtrlBlk);
   pMyCtrlBlk = NULL;

   return ODPPSUCCESS;
}// END Terminate() 


RETVAL GetErrorMsg(RETVAL ErrorCode,      
                   int iLanguage,         
                   wchar_t *pErrMsg)
{
	return ODPPSUCCESS;
}

/*---------------------------------------------------------------------------
 * Function:   GetInfo
 *
 *    Desc:
 *       Returns the Source Indexes and destination Index information 
         requested by the caller.
 *
 *    Parms:
 *       pCtrlBlk :  Control block for this service provider
 *       sRequest :  Request ID sent by the caller
 *       ptr      :  memory block pointer to copy the return data 
 *                   Provider will not make any attempt to verify the buffer 
 *                   or its length or its type it will simple copy the data 
 *                   into the buffer caller has to ensure that buffer is 
 *                   allocated sufficient memory to hold return data.
 *
 *    Returns:
 *       SUCCESS  :  ODPPSUCCESS 
 *       FAILURE  :  Can return various error codes depending upon the error. 
 *--------------------------------------------------------------------------*/

RETVAL GetInfo(DP_BLK_DEF_X *pCtrlBlk,
               short sRequest,
               void *ptr,
               int *pBufLen)
{

   RETVAL            retval              = ODPPSUCCESS;
   short             sCount              = 0; // To loop through src col 
   short             *pIndex             = NULL; // Holds value of index              
   short             sSrcColCount        = 0;
   int               iBufLen             = 0;
   DP_MYCTRL_DEF     *pMyCtrlBlk       =  NULL;
   DP_SRCCOL_DEF_X   *pSrcCol            =  NULL;


   // Get the CtrlBlk allocated in Init 
   pMyCtrlBlk = (DP_MYCTRL_DEF*)pCtrlBlk->pExt;
   if(NULL == pMyCtrlBlk)
   {
      return MY_ERR_MYCTRLBLK_NOT_FOUND;
   }
   
   // Check the requested info and return information 
   switch(sRequest)
   {
      case ODPP_REQ_SRC_COL_INDEX:
         {
            sSrcColCount = pCtrlBlk->sSrcColCount;
            // Check if pointer is NULL 
            if(NULL == ptr)
            { 
                // Just Return the buffer size required 
                *pBufLen = (int)(sSrcColCount * sizeof(short));
            } 
            else
            {
               //Check if buffer length is less than then size required
               //for total number of source column(s)
               if(*pBufLen < (int)(sSrcColCount * sizeof(short)))
               {
                  return MY_ERR_GETINFO_BUFFER_SHORT;
               }
               
               // Get source column list address 
               pSrcCol = &pCtrlBlk->SrcCol;
               if(pSrcCol == NULL)
               {
                  // Source column list is empty 
                  return ODPPFAILURE;
               }
               
               pIndex = (short *)ptr;
               for(sCount = 0; 
                   sCount < pCtrlBlk->sSrcColCount && (pIndex != NULL);
                   sCount++)
               {
                  // Store source column index 
                  *pIndex = pSrcCol->sColIndex;
                  pIndex++;
                                    
                  // move to next column 
                  pSrcCol = pSrcCol->pNext;
                  iBufLen = iBufLen + (int)sizeof(short);
                  
               }
               // return total size required for source columns 
               *pBufLen = iBufLen;
            }
         }
         break;
      case ODPP_REQ_DEST_COL_INDEX:
         {
            // Get the destination col Index and return 
            if(NULL == ptr)
            { 
               // Just Return the buffer size required 
               *pBufLen = sizeof(short);
            } 
            else
            {
               // Check if buffer length is less than size of dest col 
               if(*pBufLen < sizeof(short))
               {
                  return MY_ERR_GETINFO_BUFFER_SHORT;
               }
               
               // Store source column index and size 
               pIndex = (short *)ptr;
               *pIndex = pMyCtrlBlk->sDestColIndex;
               *pBufLen = sizeof(short);  
            }
         }
         break;
      default :
         // Requested Information not supported 
         retval = MY_INFO_REQUEST_NOT_SUPPORTED;
         break;
   }
	return retval;
} // END GetInfo(); 


