/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* Copyright IBM Corp. 2012,2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 
//---------------------------------------------------------------------------
// ODPPCCNExitS.c
// The following is the source for an Optim column map exit sample
// that uses the ODPP Credit Card Number (CCN) service provider
// for masking credit card number data.
//----------------------------------------------------------------------------
// Include the required standard C-type headers
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

//Include the required ODPP headers
#include "ODPPCmnAPI.h"
#include "ODPPDataTypesH.h"
#include "ODPPCmnOprH.h"
#include "ODPPCmnErrCodes.h"
#include "foppsmex.h"

// Tracing specific defines
#define TRACELEVEL "1"

// Define ODPPSUCCESS and ODPPFAILURE
#define FALSE                       0
#define TRUE                        1
#define ODPPSUCCESS                 0
#define ODPPFAILURE                 -1

//Maximum number of parameters
#define MAX_PARAMETER               3
#define MAX_COLUMN                  1
#define MAX_COLNAME_SIZE            256

typedef struct s_Exit_Work
{
    SQLDA          *pSrcSQLDA;
    SQLDA          *pDstSQLDA;
    int             iSvcToken;  // Service token
    short           dVarSub;    // destination column subscript
    short           sVarSub;    // matched source col subscript
}   EXIT_WORK;

typedef struct s_Column_Workarea
{
    UCHAR           return_msg[252];  // CWA space used for abort message
    EXIT_WORK      *pExitWork;
} COL_WORKAREA;


//----------------------------------------------------------------------------
// Function:      PSTColMapWExit()
// Purpose:       Main entry point for an Optim Column Map User Exit.
// Description:   This function is called by Optim for each row
//                in a mapped table when the DLL name is specified
//                as a user exit for a column map source.
//
// Parameters:    pExitParms        A pointer to a COLMAP_PARMS structure
//                                  This structure contains information
//                                  about the source and destination tables,
//                                  the nature of the current call, and
//                                  pointers to work areas
//                pSrcSQLDA         A pointer to an SQLDA
//                                  structure describing the source columns
//                pDstSQLDA         A pointer to an SQLDA
//                                  structure describing the destination columns
//                pColWorkArea      A pointer to a COL_WORKAREA structure describing
//                                  a work area for the exit
// Returns:       0                           Column successfully transformed
//                8                           Various errors
//----------------------------------------------------------------------------
#pragma linkage(OdppCcnExitS, fetchable)
short OdppCcnExitS(COLMAP_PARMS *pExitParms, SQLDA *pSrcSQLDA, SQLDA *pDstSQLDA, COL_WORKAREA *pColWorkArea)
{
   short          rc = 0;
   //Return Code Of ODPP Function 
   RETVAL         retVal     = 0;
   int colI;
   // Service identifier
   static char    *PrvName = "CCN";
   // ODPP row set definition structure 
   static DP_ROWSET_DEF rowSet;
   // ODPP row definition structure
   static DP_ROW_DEF    sRowDef;
   // ODPP data definition structure
   static DP_FIELD_DATA_DEF  sDataDef;
   //Service definition
   static DP_SVC_DEF sSvcDef;
   // Array of DP_FIELD_DEF structures
   static DP_FIELD_DEF sFldDef[MAX_COLUMN];
   // Array of DP_INIT_OP_DEF structures
   static DP_INIT_OP_DEF sParamDef[MAX_PARAMETER];

   if (pExitParms->DSQLVarSub) {
     // If this is the first row, initialize the ODPP framework
     if (pExitParms->CurrentRow == 1) {
       if (NULL == (pColWorkArea->pExitWork = calloc(1, sizeof(EXIT_WORK)))) {
         sprintf(pColWorkArea->return_msg, "Failed to allocate storage for column work area");
         return 8;
       }

       // Match the destination column name to the source
       for (colI=0; colI<pSrcSQLDA->NumCols; colI++) {
         if (0 == strcmp(pSrcSQLDA->Col[colI].Name, pDstSQLDA->Col[pExitParms->DSQLVarSub-1].Name)) {
           pColWorkArea->pExitWork->sVarSub = colI;
           break;
         }
       }

       // Return an error if the column name was not found
       if (colI == pSrcSQLDA->NumCols) {
         printf("Match to source column not found\n");
         return 8;
       }

       retVal = Provider_FrmwInit(NULL);
       if (ODPP_OS_LIB_MUTEX_ALREADY_INITIALIZED == retVal)
         retVal = ODPPSUCCESS;
       else
         if (ODPPSUCCESS != retVal)
           return retVal;

       // Clear ODPP structures 
       memset(&sSvcDef,0,sizeof(DP_SVC_DEF));
       memset(&sFldDef,0,sizeof(DP_FIELD_DEF));
       memset(&sParamDef,0,MAX_PARAMETER*(sizeof(DP_INIT_OP_DEF)));

       // Initialize the ODPP DP_SVC_DEF structures
       INITIALIZE_ODPP_STRUCT(sSvcDef, DP_SVC_DEF)

       // Initialize ODPP DP_FIELD_DEF structure. Here we are using only one column hence initializing for 
       // first member of sFldDef. If multiple columns are used then you need to initialize with loop similar
       // to DP_INIT_OP_DEF structure below.
       INITIALIZE_ODPP_STRUCT(sFldDef[0], DP_FIELD_DEF)

       // Initialize the ODPP DP_INIT_OP_DEF for total number of parameters
       INITIALIZE_ODPP_STRUCT(sParamDef[0], DP_INIT_OP_DEF)

       // Enable ODPP tracing: the following are examples of setting ODPP trace levels
       // See the ODPP Developer's Guide for a detailed description of the function ODPPSetTraceLevel
       // for setting ODPP trace levels
       // Turn entry/exit tracing on (level 1) ODPP system wide
       // retVal = ODPPSetTraceLevel("1");

       // Turn all tracing off ODPP system wide (other than exception tracing which cannot be turned off)
       // retVal = ODPPSetTraceLevel("0");

       // Turn entry/exit tracing on for the AGE provider
       // retVal = ODPPSetTraceLevel("AGEP=1");

       // Turn entry/exit tracing on for the AGE and CCN providers
       // retVal = ODPPSetTraceLevel("AGEP=1/CCNP=1");

       // Turn entry/exit tracing on ODPP system wide
       // Turn on level 3 tracing for the AGE provider
       // Turn on all trace levels (1-32) for the CCN provider's Init function
       // retVal = ODPPSetTraceLevel("1/AGEP=3/CCNP.Init=1:32");

       // Set the ODPP trace level
       retVal = ODPPSetTraceLevel(TRACELEVEL);
       if (ODPPSUCCESS != retVal)
       {
         printf("ODPP failed to set the trace level.  Return code is %d\n", retVal);
         return 8;
       }

       // ODPP service definition structure initialization
       sSvcDef.pParams                =   &sParamDef[0];
       sSvcDef.pFldDef                =   &sFldDef[0];
       sSvcDef.bCopyToDest            =   TRUE;
      
       // ODPP field definition structure initialization
       // To indicate that column name is wide character and USE_ODPP_MBCS_CHAR_CALLS is not defined
       sFldDef[0].cSubType = 'M';

       // Allocate the pMC struct
       sFldDef[0].CN.pMC = (DPFD_MC_SS *) malloc(sizeof(DPFD_MC_SS));
       if(NULL == sFldDef[0].CN.pMC)
       {
          printf("\nFailed to allocate pMC struct of DP_FIELD_DEF\n");
          return 8;
       }
       memset(sFldDef[0].CN.pMC, 0, sizeof(DPFD_MC_SS));
   
       // Code page is zero for wide character (Unicode) format
       sFldDef[0].iDataCodePage = 0; 

       // DBMS type is zero for wide character (Unicode) format
       sFldDef[0].cDataDBMSType = 0; 

       // Store the maximum size for column name buffer
       sFldDef[0].iColNameBytes = (MAX_COLNAME_SIZE + 1) * sizeof(ODPP_WCHAR);

       // Allocate buffer for column name
       sFldDef[0].CN.pMC->pColName = (char *)malloc(strlen(pSrcSQLDA->Col[colI].Name)+1);
       if(NULL == sFldDef[0].CN.pMC->pColName)
       {
          printf("\nFailed to allocate memory for Column name\n");
          return 8;
       }

       // Set the source column name in sFldDef
       strcpy(sFldDef[0].CN.pMC->pColName, pSrcSQLDA->Col[colI].Name);

       // Set the source column data type
       sFldDef[0].sDatatype           =   ODPPDATATYPE_CHAR;

       //Set the number of ODPP CCN columns
       sSvcDef.sFldCount              =   1;

       //Set the number Of ODPP CCN Parameters
       sSvcDef.sParamCount            =   3;

       // Clear the ODPP rowset structure
       memset(&rowSet,0,sizeof(DP_ROWSET_DEF));
       // Initialize the DP_ROWSET_DEF structure
       INITIALIZE_ODPP_STRUCT(rowSet, DP_ROWSET_DEF)

       // Setup the number of rows in the ODPP rowset structure
       rowSet.iCount      =  1;

       // Clear the ODPP row and field definition structures
       memset(&sRowDef,0,sizeof(DP_ROW_DEF));
       // Initialize the DP_ROW_DEF structure
       INITIALIZE_ODPP_STRUCT(sRowDef, DP_ROW_DEF)

       // Clear the ODPP field data definition structure
       memset(&sDataDef,0,sizeof(DP_FIELD_DATA_DEF));
       // Initialize the DP_FIELD_DATA_DEF structure
       INITIALIZE_ODPP_STRUCT(sDataDef, DP_FIELD_DATA_DEF)

       /* Switch is not needed but it is a mandatory operand
       so set it to ODPP_OPR_SWITCH_NA (Switch Not Applicable)*/
       sParamDef[0].usParameterID = ODPP_OPR_SWITCH_NA;

       // To indicate that this parameter does not hold any value.
       sParamDef[0].iValueSubType = PARAM_VAL_NONE;

       // Index of the source column, to use column name above commented code can be used
       sParamDef[1].usParameterID = ODPP_OPR_SOURCE_COLINDEX;

       // To indicate that current parameter will hold numeric value.
       sParamDef[1].iValueSubType = PARAM_VAL_NUM;

       // index values always start with 0
       sParamDef[1].PV.uiVal = 0;

       // Then setup ODPP for default masking
       sParamDef[2].usParameterID = ODPP_OPR_METHOD;

       // To indicate that current parameter will hold numeric value.
       sParamDef[2].iValueSubType = PARAM_VAL_NUM;

       // ODPP_METHOD_MASK:  Generates a randomly masked output
       sParamDef[2].PV.uiVal = ODPP_METHOD_RANDOM;

       // Initialize the ODPP CCN provider
       retVal = Provider_Init(&(pColWorkArea->pExitWork->iSvcToken),&PrvName[0], strlen(PrvName),&sSvcDef,TRUE);
       if (ODPPSUCCESS != retVal)
         return 8;
     }

     // Populate the ODPP rowset structure
     rowSet.pRowDefine   =   &sRowDef;
     rowSet.iLastPrcsRow =   0;
     sRowDef.bProcessed  =   FALSE;
     sRowDef.bHasError   =   FALSE;
     sDataDef.bSrcNull =     FALSE;

     // Setup the ODPP data definition structure...
     // Set the length of the source data
     sDataDef.iSrcBufLen =   pSrcSQLDA->Col[pColWorkArea->pExitWork->sVarSub].Length;
     sDataDef.pSrcBuf    =   pSrcSQLDA->Col[pColWorkArea->pExitWork->sVarSub].pData;
     sDataDef.iDstBufLen =   pDstSQLDA->Col[pExitParms->DSQLVarSub-1].Length;
     sDataDef.pDstBuf    =   pDstSQLDA->Col[pExitParms->DSQLVarSub-1].pData;

     // Setup the ODPP row definition structure   
     sRowDef.pFldDataDefine =    &sDataDef;
     // Number of Column
     sRowDef.sCount         =    1; 

     // Call the ODPP provider service to mask the credit card number-type data
     retVal = Provider_Service(pColWorkArea->pExitWork->iSvcToken, ODPP_METHOD_DEFAULT, NULL, &rowSet);
     if (ODPPSUCCESS != retVal)
       return 8;
   }
   else {
     if (sFldDef[0].CN.pMC)
       free(sFldDef[0].CN.pMC);
     if (sFldDef[0].CN.pMC->pColName)
       free(sFldDef[0].CN.pMC->pColName);
     retVal = Provider_Term(pColWorkArea->pExitWork->iSvcToken);
   }

   return rc;

}
