       CBL RENT,DLL,PGMN(LM),LIB,LIST,MAP
      *----------------------------------------------------------------
      *  EXAMPLE OF COBOL CALLING THE ODPP FUNCTION AFF
      *  THIS PROGRAM MUST BE COMPILED WITH THE FOLLOWING OPTIONS:
      *     RENT,DLL,PGMN(LM),LIB
      *----------------------------------------------------------------
      *  The syntax diagram used for this exit follows:
      *
      *  TRANS PRO=AFF,MTD=REPEATABLE,KEY="Zyz123",
      *  algo=fpe,rule=num,
      *  CODEPAGE=1047,CPTYPE=DB2ZOS,
      *  FLDDEF1=(NAME=AFFCH,DT=char)
      *
       ID DIVISION.
       PROGRAM-ID. "ODPPAFFS".
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
           SELECT INPUT-FILE
           ASSIGN TO INDD
           ORGANIZATION IS SEQUENTIAL
           ACCESS IS SEQUENTIAL.
       DATA DIVISION.
       FILE SECTION.
       FD  INPUT-FILE
           RECORDING MODE IS V.
       01  INPUT-RECORD PIC X(252).
      *----------------------------------------------------------------
       WORKING-STORAGE SECTION.
      *----------------------------------------------------------------

      *
      * The following copybook contains ODPP structure definitions
      * and constants
      *
       COPY ODPPWAPI.

       01  INPUT-DONE PIC X VALUE 'N'.
       01  INPUT-VALUE PIC X(20).
       01  OUTPUT-VALUE PIC X(20).
       01  SVC-TOKEN       PIC S9(9) COMP-5.

       01 COLUMN-NAME PIC X(30).
       77 NULL-PTR POINTER VALUE NULL.

      *
      * Provider name and country definition (which determines
      * rules for national id format)
      *
       77 PRV-NAME PIC X(4) VALUE Z"AFF".
       01 AFFKEY-VAL PIC X(7) VALUE Z"Zyz123".
       01 AFFKEY.
          05 AFFKEY-DBMS-TYPE PIC X VALUE 'N'.
          05 FILLER PIC X(3).
          05 AFFKEY-CODE-PAGE PIC S9(9) COMP-5 VALUE -1.
          05 AFFKEY-VAL-PTR POINTER.

       77 CM-ARG-PTR POINTER.
       77 ALLOC-INIT-BLOCK PIC 9 BINARY VALUE 1.

       PROCEDURE DIVISION.
           MOVE 0 TO RETURN-CODE

           OPEN INPUT INPUT-FILE.
           PERFORM MASK-INIT.
           PERFORM MASK-SERVICE
             WITH TEST BEFORE UNTIL INPUT-DONE EQUAL 'Y'.
           PERFORM MASK-TERM.
           CLOSE INPUT-FILE.
           GOBACK.

       MASK-INIT.
      *
      * Initialize the ODPP framework
      *
           SET CM-ARG-PTR TO NULL.
           CALL 'Provider_FrmwInit'
             USING
               BY VALUE CM-ARG-PTR
             RETURNING RETURN-CODE.
           IF RETURN-CODE = ODPP-OS-LIB-MUTEX-ALREADY-INIT THEN
             MOVE 0 TO RETURN-CODE
           ELSE
             IF RETURN-CODE NOT = 0 THEN
               DISPLAY
                 "Framework initialization failed rc=" RETURN-CODE
               GOBACK
             END-IF
           END-IF.

      *
      * Initialize the provider
      *
      *                       Service Definition body
           MOVE LOW-VALUES TO DP-SVC-DEF-BODY
      *                       Field Definition body
           MOVE LOW-VALUES TO DP-FIELD-DEF-BODY(1)
      *                       Initialization parms for
      *                       current service.
           MOVE LOW-VALUES TO DP-INIT-OP-DEF-BODY(1),
                         DP-INIT-OP-DEF-BODY(2),
                         DP-INIT-OP-DEF-BODY(3),
                         DP-INIT-OP-DEF-BODY(4),
                         DP-INIT-OP-DEF-BODY(5),
                         DP-INIT-OP-DEF-BODY(6),
                         DP-INIT-OP-DEF-BODY(7).

           MOVE LENGTH OF DP-SVC-DEF TO STRUCT-LEN OF DP-SVC-DEF.
           COMPUTE STRUCT-LEN OF DP-FIELD-DEF-ARRAY(1)
             = LENGTH OF DP-FIELD-DEF-HEADER
               + LENGTH OF DP-FIELD-DEF-BODY
           COMPUTE STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(1),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(2),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(3),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(4),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(5),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(6),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(7)
             = LENGTH OF DP-INIT-OP-DEF-HEADER
               + LENGTH OF DP-INIT-OP-DEF-BODY.

      *   * Show where the first Initialization parameter
      *   *  block is located.
           SET PARAMS-PTR TO ADDRESS OF DP-INIT-OP-DEF-ARRAY(1).
      *   * Show where the first field definition block is
      *   * located.
           SET FLD-DEF-PTR TO ADDRESS OF DP-FIELD-DEF-ARRAY(1).
           MOVE X"01" TO COPY-TO-DEST.

      *    "M" represents specifying column name in
      *    character (SBCS/MBCS) format.
      *    This is normally used for z/OS environment.
           MOVE "M" TO SUB-TYPE(1).

           MOVE RDB-NONE TO DATA-DBMS-TYPE(1).
      *    Use -1 to default to system environment
      *    defined code page.
           MOVE -1 TO DATA-CODE-PAGE(1).
           MOVE ODPPDATATYPE-VARCHAR-SZ TO DATA-TYPE(1).
      *    Number of DP-FIELD-DATA-DEF blocks
           MOVE 1 TO FLD-COUNT.
      *    Set the number of parameters
      *    (DP-INIT-OP-DEF-ARRAY blocks) for this
      *    Provider Service.
           MOVE 7 TO PARAM-COUNT.

      *
      * Initialize the ODPP rowset, row and field  structures
      *
           MOVE LOW-VALUES TO DP-ROWSET-DEF-BODY.
           SET NEXT-PTR OF DP-ROW-DEF TO NULL.
           MOVE LENGTH OF DP-ROWSET-DEF
             TO STRUCT-LEN OF DP-ROWSET-DEF.
      *    Will always be 1 in an Optim/z Column Map Exit.
           MOVE 1 TO ROW-COUNT OF DP-ROWSET-DEF.

           MOVE LOW-VALUES TO DP-ROW-DEF.
           MOVE LENGTH OF DP-ROW-DEF
             TO STRUCT-LEN OF DP-ROW-DEF.

           MOVE LOW-VALUES TO DP-FIELD-DATA-DEF-BODY(1).
           SET NEXT-PTR OF DP-FIELD-DATA-DEF(1) TO NULL.
           MOVE LENGTH OF DP-FIELD-DATA-DEF
             TO STRUCT-LEN OF DP-FIELD-DATA-DEF(1).

      *    *
      *    * Initialization parameters for AFF
      *    *
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(1) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(2)
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(2) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(3)
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(3) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(4)
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(4) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(5)
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(5) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(6)
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(6) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(7)
      *    * Define block 1 - SWITCH-NA
           MOVE ODPP-OPR-SWITCH-NA TO PARAMETER-ID(1)
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(1)
           MOVE 0 TO PARAM-VAL-INT(1).

      *    * Define block 2 - SOURCE-COLINDEX
           MOVE ODPP-OPR-SOURCE-COLINDEX TO PARAMETER-ID(2)
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(2)
           MOVE 0 TO PARAM-VAL-INT(2).

      *    * Define block 3 - METHOD
           MOVE ODPP-OPR-METHOD TO PARAMETER-ID(3)
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(3)
           MOVE ODPP-METHOD-MASK TO PARAM-VAL-INT(3).

      *    * Define block 4 - Affinity Algorithm
           MOVE ODPP-OPR-COL-ALGORITHM TO PARAMETER-ID(4)
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(4)
           MOVE ODPP-COL-ALGORITHM-FPE TO PARAM-VAL-INT(4).

      *    * Define block 5 - Affinity KEY parameter
           MOVE ODPP-OPR-COL-KEY TO PARAMETER-ID(5)
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(5)
           SET AFFKEY-VAL-PTR TO ADDRESS OF AFFKEY-VAL
           SET PARAM-VAL-PTR(5) TO ADDRESS OF AFFKEY
           MOVE LENGTH OF AFFKEY-VAL TO VALUE-BUF-BYTES(4).

      *    * Define block 6 - Affinity NUM
           MOVE ODPP-OPR-COL-NUM TO PARAMETER-ID(6)
           MOVE PARAM-VAL-NONE TO VALUE-SUB-TYPE(6).

      *    * Define block 7 - Affinity Mask type
           MOVE ODPP-OPR-COL-MASK TO PARAMETER-ID(7)
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(7)
           MOVE ODPP-COL-MASK-UNIQUE TO PARAM-VAL-INT(7).

           CALL 'Provider_Init'
             USING
               BY VALUE ADDRESS OF SVC-TOKEN,
               BY VALUE ADDRESS OF PRV-NAME,
               BY VALUE 3,
               BY VALUE ADDRESS OF DP-SVC-DEF,
               BY VALUE ALLOC-INIT-BLOCK
             RETURNING RETURN-CODE.

           IF RETURN-CODE NOT = 0 THEN
             DISPLAY
               "Provider initialization failed rc=" RETURN-CODE
             GOBACK
           END-IF.

       MASK-SERVICE.
           READ INPUT-FILE INTO INPUT-VALUE
             AT END MOVE 'Y' TO INPUT-DONE.
           IF INPUT-DONE EQUAL 'Y' THEN
             GOBACK
           END-IF.

      *
      * Populate the rowset structure
      *
           SET ROW-DEFINE-PTR TO ADDRESS OF DP-ROW-DEF.
           MOVE 0 TO LAST-PRCS-ROW.
           MOVE X"00" TO PROCESSED.
           MOVE X"00" TO HAS-ERROR.
           MOVE X"00" TO SRC-NULL(1).

      *
      * Set up the data definition structure
      *
           COMPUTE SRC-BUF-LEN(1) = FUNCTION LENGTH(INPUT-VALUE).
           SET SRC-BUF-PTR(1) TO ADDRESS OF INPUT-VALUE.
           COMPUTE DST-BUF-LEN(1) = FUNCTION LENGTH(OUTPUT-VALUE).
           SET DST-BUF-PTR(1) TO ADDRESS OF OUTPUT-VALUE.

      *
      * Set up the row definition structure
      *
           SET FLD-DATA-DEFINE-PTR TO
              ADDRESS OF DP-FIELD-DATA-DEF-ARRAY(1).
           MOVE 1 TO S-COUNT OF DP-ROW-DEF.
           MOVE SPACES TO OUTPUT-VALUE.

      *
      * Call the provider service
      *
           CALL 'Provider_Service'
             USING
               BY VALUE SVC-TOKEN,
               BY VALUE ODPP-METHOD-DEFAULT,
               BY VALUE NULL-PTR,
               BY VALUE ADDRESS OF DP-ROWSET-DEF
             RETURNING RETURN-CODE.
           IF RETURN-CODE NOT = 0 THEN
             DISPLAY "NID service failed rc=" RETURN-CODE
             GOBACK
           END-IF.

           DISPLAY "Original Value=" INPUT-VALUE "  Masked Value="
              OUTPUT-VALUE.
           MOVE 0 TO RETURN-CODE.

       MASK-TERM.
           CALL 'Provider_Term'
             USING
               BY VALUE SVC-TOKEN
             RETURNING RETURN-CODE.
