       CBL RENT,DLL,PGMN(LM),LIB,LIST,MAP
      *----------------------------------------------------------------
      *  EXAMPLE OF COBOL CALLING THE ODPP FUNCTION HASH LOOKUP
      *  THIS PROGRAM MUST BE COMPILED WITH THE FOLLOWING OPTIONS:
      *     RENT,DLL,PGMN(LM),LIB
      *----------------------------------------------------------------
       ID DIVISION.
       PROGRAM-ID. "ODPPHLKP".
       ENVIRONMENT DIVISION.
       INPUT-OUTPUT SECTION.
       FILE-CONTROL.
           SELECT INPUT-FILE
           ASSIGN TO INDD
           ORGANIZATION IS SEQUENTIAL
           ACCESS IS SEQUENTIAL.
       DATA DIVISION.
       FILE SECTION.
       FD  INPUT-FILE
           RECORDING MODE IS F.
       01  INPUT-RECORD PIC X(11).

      *----------------------------------------------------------------
       WORKING-STORAGE SECTION.
      *----------------------------------------------------------------

      *
      * The following copybook contains ODPP structure definitions
      * and constants
      *
       COPY ODPPWAPI.

       01  INPUT-DONE PIC X VALUE 'N'.
       01  INPUT-SSN PIC X(11).
       01  OUTPUT-SSN PIC X(20).
       01  SVC-TOKEN       PIC S9(9) COMP-5.

       77 NULL-PTR POINTER VALUE NULL.

       77 CONNECTION-STRING PIC X(50) VALUE Z"".
       01 ODPP-CONN-STR.
          COPY ODPPSVAL.

       77 SCHEMA-STRING PIC X(50) VALUE Z"SHARKMN".
       01 ODPP-LDS-SCHEMA.
          COPY ODPPSVAL.

       77 LDS-ID-STRING PIC X(50) VALUE Z"HASH_LOOKUP_SSN".
       01 ODPP-LDS-ID.
          COPY ODPPSVAL.

       77 REPLACE-COLS-STRING PIC X(50) VALUE Z"SSN2".
       01 ODPP-REPLACE-COLS.
          COPY ODPPSVAL.

       01 SEARCH-COLS-STRING PIC X(5) VALUE Z"SSN1".
       01 ODPP-SEARCH-COLS.
          COPY ODPPSVAL.

       77 HASH-SEQ-COL-NAME-STR PIC X(50) VALUE Z"ID".
       01 HASH-SEQ-COL-NAME.
          COPY ODPPSVAL.

       01 FIELD-1-NAME PIC X(5) VALUE Z"SSN1".
       01 FIELD-1.
          COPY ODPPCOLN.

       77 FIELD-2-NAME PIC X(5) VALUE Z"SSN2".
       01 FIELD-2.
          COPY ODPPCOLN.

       01 FIELD-DATA-DEF-1.
          COPY ODPPFLDD.

       01 FIELD-DATA-DEF-2.
          COPY ODPPFLDD.

      *
      * Provider name and country definition (which determines
      * rules for national id format)
      *
       77 PRV-NAME PIC X(4) VALUE Z"LKP".
       77 SOURCE-DATE-FORMAT-VAL PIC X(14) VALUE Z"%YYYY/%MM/%DD".
       01 SOURCE-DATE-FORMAT.
          05  DATE-FORMAT-DBMS-TYPE PIC X VALUE 'N'.
          05  FILLER PIC X(3).
          05  DATE-FORMAT-CODE-PAGE PIC S9(9) COMP-5 VALUE -1.
          05  DATE-FORMAT-PTR POINTER.

       77 CM-ARG-PTR POINTER.
       77 ALLOC-INIT-BLOCK PIC 9 BINARY VALUE 1.

       PROCEDURE DIVISION.
           MOVE 0 TO RETURN-CODE

           OPEN INPUT INPUT-FILE.
           PERFORM MASK-INIT.
           PERFORM MASK-SERVICE
             WITH TEST BEFORE UNTIL INPUT-DONE EQUAL 'Y'.
           PERFORM MASK-TERM.
           CLOSE INPUT-FILE.
           GOBACK.

       MASK-INIT.
      *
      * Initialize the ODPP framework
      *
           SET CM-ARG-PTR TO NULL.
           CALL 'Provider_FrmwInit'
             USING
               BY VALUE CM-ARG-PTR
             RETURNING RETURN-CODE.
           IF RETURN-CODE = ODPP-OS-LIB-MUTEX-ALREADY-INIT THEN
             MOVE 0 TO RETURN-CODE
           ELSE
             IF RETURN-CODE NOT = 0 THEN
               DISPLAY
                 "Framework initialization failed rc=" RETURN-CODE
               GOBACK
             END-IF
           END-IF.

      *
      * Initialize the provider
      *
           MOVE LOW-VALUES TO DP-SVC-DEF-BODY.
           MOVE LOW-VALUES TO DP-FIELD-DEF-BODY(1).
           MOVE LOW-VALUES TO DP-FIELD-DEF-BODY(2).
           MOVE LOW-VALUES TO DP-INIT-OP-DEF-BODY(1),
                              DP-INIT-OP-DEF-BODY(2),
                              DP-INIT-OP-DEF-BODY(3).

           MOVE LENGTH OF DP-SVC-DEF TO STRUCT-LEN OF DP-SVC-DEF.
           COMPUTE STRUCT-LEN OF DP-FIELD-DEF-ARRAY(1)
             = LENGTH OF DP-FIELD-DEF-HEADER
               + LENGTH OF DP-FIELD-DEF-BODY.
           COMPUTE STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(1),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(2),
                   STRUCT-LEN OF DP-INIT-OP-DEF-ARRAY(3)
             = LENGTH OF DP-INIT-OP-DEF-HEADER
               + LENGTH OF DP-INIT-OP-DEF-BODY.

           SET NEXT-PTR OF DP-FIELD-DEF-ARRAY(1)
             TO ADDRESS OF DP-FIELD-DEF-ARRAY(2).

           MOVE "M" TO SUB-TYPE OF DP-FIELD-DEF(1).
           MOVE 1047 TO DATA-CODE-PAGE OF DP-FIELD-DEF(1).
           MOVE RDB-NONE TO DATA-DBMS-TYPE OF DP-FIELD-DEF(1).
           MOVE ODPPDATATYPE-VARCHAR-SZ TO DATA-TYPE OF DP-FIELD-DEF(1).
           MOVE LOW-VALUES TO FIELD-1.
           SET COL-NAME-PTR OF FIELD-1 TO ADDRESS OF FIELD-1-NAME.
           SET MC-PTR OF DP-FIELD-DEF-ARRAY(1) TO ADDRESS OF FIELD-1.
           MOVE 1047 TO COL-NAME-CODE-PAGE OF FIELD-1.
           COMPUTE COL-NAME-BYTES OF DP-FIELD-DEF(1) = 4.
           COMPUTE COL-LENGTH OF DP-FIELD-DEF(1) = 11.

           MOVE "M" TO SUB-TYPE OF DP-FIELD-DEF(2).
           MOVE 1047 TO DATA-CODE-PAGE OF DP-FIELD-DEF(2).
           MOVE RDB-NONE TO DATA-DBMS-TYPE OF DP-FIELD-DEF(2).
           MOVE ODPPDATATYPE-VARCHAR-SZ TO DATA-TYPE OF DP-FIELD-DEF(2).
           MOVE LOW-VALUES TO FIELD-2.
           SET COL-NAME-PTR OF FIELD-2 TO ADDRESS OF FIELD-2-NAME.
           MOVE 1047 TO COL-NAME-CODE-PAGE OF FIELD-2.
           SET MC-PTR OF DP-FIELD-DEF-ARRAY(2) TO ADDRESS OF FIELD-2.
           COMPUTE COL-NAME-BYTES OF DP-FIELD-DEF(2) = 4.
           COMPUTE COL-LENGTH OF DP-FIELD-DEF(2) = 11.

           SET PARAMS-PTR TO ADDRESS OF DP-INIT-OP-DEF.
           SET FLD-DEF-PTR TO ADDRESS OF DP-FIELD-DEF.
           MOVE X"01" TO COPY-TO-DEST.

           MOVE 2 TO FLD-COUNT.
           MOVE 10 TO PARAM-COUNT.

      *
      * Initialize the ODPP rowset, row and field  structures
      *
           MOVE LOW-VALUES TO DP-ROWSET-DEF-BODY.
           SET NEXT-PTR OF DP-ROW-DEF TO NULL.
           MOVE LENGTH OF DP-ROWSET-DEF
             TO STRUCT-LEN OF DP-ROWSET-DEF.
           MOVE 1 TO ROW-COUNT OF DP-ROWSET-DEF.

           MOVE LOW-VALUES TO DP-ROW-DEF.
           MOVE LENGTH OF DP-ROW-DEF
             TO STRUCT-LEN OF DP-ROW-DEF.

           SET NEXT-PTR OF FIELD-DATA-DEF-1 TO
             ADDRESS OF FIELD-DATA-DEF-2.
           MOVE LENGTH OF FIELD-DATA-DEF-1
             TO STRUCT-LEN OF FIELD-DATA-DEF-1.
           MOVE LENGTH OF FIELD-DATA-DEF-2
             TO STRUCT-LEN OF FIELD-DATA-DEF-2.

           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(1) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(2).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(2) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(3).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(3) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(4).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(4) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(5).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(5) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(6).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(6) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(7).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(7) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(8).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(8) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(9).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(9) TO
             ADDRESS OF DP-INIT-OP-DEF-ARRAY(10).
           SET NEXT-PTR OF DP-INIT-OP-DEF-ARRAY(10) TO NULL.

           MOVE ODPP-OPR-SWITCH-NA TO PARAMETER-ID(1).
           MOVE PARAM-VAL-NONE TO VALUE-SUB-TYPE(1).

           MOVE ODPP-OPR-LOOKUP-SEARCH-COLS TO PARAMETER-ID(2).
           MOVE "M" TO PARAM-NAME-SUB-TYPE(2).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(2).
           SET PARAM-VAL-PTR(2) TO ADDRESS OF ODPP-SEARCH-COLS.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF ODPP-SEARCH-COLS.
           MOVE 1047 TO PARAM-VAL-CODE-PAGE OF ODPP-SEARCH-COLS.
           SET VALUE-PTR OF ODPP-SEARCH-COLS
             TO ADDRESS OF SEARCH-COLS-STRING.
           COMPUTE VALUE-BUF-BYTES(2) = 4.

           MOVE ODPP-OPR-LOOKUP-REPLACE-COLS TO PARAMETER-ID(3).
           MOVE "M" TO PARAM-NAME-SUB-TYPE(3).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(3).
           SET PARAM-VAL-PTR(3) TO ADDRESS OF ODPP-REPLACE-COLS.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF ODPP-REPLACE-COLS.
           MOVE 1047 TO PARAM-VAL-CODE-PAGE OF ODPP-REPLACE-COLS.
           SET VALUE-PTR OF ODPP-REPLACE-COLS
             TO ADDRESS OF REPLACE-COLS-STRING.
           COMPUTE VALUE-BUF-BYTES(3) = 4.

           MOVE ODPP-OPR-LDS-ID TO PARAMETER-ID(4).
           MOVE "M" TO PARAM-NAME-SUB-TYPE(4).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(4).
           SET PARAM-VAL-PTR(4) TO ADDRESS OF ODPP-LDS-ID.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF ODPP-LDS-ID.
           MOVE 1047 TO PARAM-VAL-CODE-PAGE OF ODPP-LDS-ID.
           SET VALUE-PTR OF ODPP-LDS-ID
             TO ADDRESS OF LDS-ID-STRING.
           COMPUTE VALUE-BUF-BYTES(4) = 15.

           MOVE ODPP-OPR-LDS-CONN-STRING TO PARAMETER-ID(5).
           MOVE "M" TO PARAM-NAME-SUB-TYPE(5).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(5).
           SET PARAM-VAL-PTR(5) TO ADDRESS OF ODPP-CONN-STR.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF ODPP-CONN-STR.
           MOVE 1047 TO PARAM-VAL-CODE-PAGE OF ODPP-CONN-STR.
           SET VALUE-PTR OF ODPP-CONN-STR
             TO ADDRESS OF CONNECTION-STRING.
           COMPUTE VALUE-BUF-BYTES(5) = 1.

           MOVE ODPP-OPR-LDS-SCHEMA TO PARAMETER-ID(6).
           MOVE "M" TO PARAM-NAME-SUB-TYPE(6).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(6).
           SET PARAM-VAL-PTR(6) TO ADDRESS OF ODPP-LDS-SCHEMA.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF ODPP-LDS-SCHEMA.
           MOVE 1047 TO PARAM-VAL-CODE-PAGE OF ODPP-LDS-SCHEMA.
           SET VALUE-PTR OF ODPP-LDS-SCHEMA
             TO ADDRESS OF SCHEMA-STRING.
           COMPUTE VALUE-BUF-BYTES(6) = 7.

           MOVE ODPP-OPR-METHOD TO PARAMETER-ID(7).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(7).
           MOVE ODPP-METHOD-MASK TO PARAM-VAL-INT(7).

           MOVE ODPP-OPR-LDS-LIB TO PARAMETER-ID(8).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(8).
           MOVE ODPP-LDS-DB2-ZOS TO PARAM-VAL-INT(8).

           MOVE ODPP-OPR-LOOKUP-FLAGS TO PARAMETER-ID(9).
           MOVE PARAM-VAL-NUM TO VALUE-SUB-TYPE(9).
           MOVE ODPP-FLAG-LOOKUP-HASH TO PARAM-VAL-INT(9).

           MOVE ODPP-OPR-LKP-HASH-SEQ-COL-NAME TO PARAMETER-ID(10).
           MOVE PARAM-VAL-MC TO VALUE-SUB-TYPE(10).
           SET PARAM-VAL-PTR(10) TO ADDRESS OF HASH-SEQ-COL-NAME.
           MOVE RDB-NONE TO PARAM-VAL-DBMS-TYPE OF HASH-SEQ-COL-NAME.
           MOVE -1 TO PARAM-VAL-CODE-PAGE OF HASH-SEQ-COL-NAME.
           SET VALUE-PTR OF HASH-SEQ-COL-NAME
             TO ADDRESS OF HASH-SEQ-COL-NAME-STR.
           COMPUTE VALUE-BUF-BYTES(10) = 2.

           CALL 'Provider_Init'
             USING
               BY VALUE ADDRESS OF SVC-TOKEN,
               BY VALUE ADDRESS OF PRV-NAME,
               BY VALUE 3,
               BY VALUE ADDRESS OF DP-SVC-DEF,
               BY VALUE ALLOC-INIT-BLOCK
             RETURNING RETURN-CODE.

           IF RETURN-CODE NOT = 0 THEN
             DISPLAY
               "Provider initialization failed rc=" RETURN-CODE
             GOBACK
           END-IF.

       MASK-SERVICE.
           READ INPUT-FILE INTO INPUT-SSN
             AT END MOVE 'Y' TO INPUT-DONE.
           IF INPUT-DONE EQUAL 'Y' THEN
             GOBACK
           END-IF.

      *
      * Populate the rowset structure
      *
           SET ROW-DEFINE-PTR TO ADDRESS OF DP-ROW-DEF.
           MOVE 0 TO LAST-PRCS-ROW.
           MOVE X"00" TO PROCESSED.
           MOVE X"00" TO HAS-ERROR.

      *
      * Set up the data definition structure
      *
           MOVE LOW-VALUES TO
             DP-FIELD-DATA-DEF-BODY OF FIELD-DATA-DEF-1.
           COMPUTE SRC-BUF-LEN OF FIELD-DATA-DEF-1 
             = FUNCTION LENGTH(INPUT-SSN).
           SET SRC-BUF-PTR OF FIELD-DATA-DEF-1 TO ADDRESS OF INPUT-SSN.

           MOVE LOW-VALUES TO
             DP-FIELD-DATA-DEF-BODY OF FIELD-DATA-DEF-2.
           COMPUTE DST-BUF-LEN OF FIELD-DATA-DEF-2
             = FUNCTION LENGTH(OUTPUT-SSN).
           COMPUTE SRC-BUF-LEN OF FIELD-DATA-DEF-2
             = FUNCTION LENGTH(OUTPUT-SSN).
           SET DST-BUF-PTR OF FIELD-DATA-DEF-2 TO ADDRESS OF OUTPUT-SSN.

      *
      * Set up the row definition structure
      *
           SET FLD-DATA-DEFINE-PTR TO ADDRESS OF FIELD-DATA-DEF-1.
           MOVE 2 TO S-COUNT OF DP-ROW-DEF.
           MOVE SPACES TO OUTPUT-SSN.

      *
      * Call the provider service
      *
           CALL 'Provider_Service'
             USING
               BY VALUE SVC-TOKEN,
               BY VALUE ODPP-METHOD-DEFAULT,
               BY VALUE NULL-PTR,
               BY VALUE ADDRESS OF DP-ROWSET-DEF
             RETURNING RETURN-CODE.
           IF RETURN-CODE NOT = 0 THEN
             DISPLAY PRV-NAME " service failed rc=" RETURN-CODE
             GOBACK
           END-IF.

           DISPLAY "Original ssn=" INPUT-SSN "  Masked ssn="
              OUTPUT-SSN.
           MOVE 0 TO RETURN-CODE.

       MASK-TERM.
           CALL 'Provider_Term'
             USING
               BY VALUE SVC-TOKEN
             RETURNING RETURN-CODE.
